#ifndef _CRYPTO_CPP
#define _CRYPTO_CPP

#ifdef __cplusplus
extern "C" {
#endif

#include "crypto.h"
#include "global.h"
#include "utils.h"

#include <stdio.h>

#include <wincrypt.h>
#include <wincrypt_extra.h>
#include <winerror_extra.h>

  PCCERT_CONTEXT locateCertificate(HCERTSTORE hStore,BYTE* certificate, DWORD certificate_length) {
    PCCERT_CONTEXT pCertContext = NULL;
    while ( pCertContext = CertEnumCertificatesInStore(hStore,pCertContext) ) {
      if ( pCertContext ) {
	if ( pCertContext->cbCertEncoded == certificate_length ) {
	  if ( !memcmp(pCertContext->pbCertEncoded,certificate,certificate_length) ) {
	    break;
	  }
	}
      }
    }
    return pCertContext;
  }

  int getCertificateDn(BYTE *certificate,
		       DWORD certificate_length,
		       DWORD issuerFlag,
		       WCHAR **dn,
		       DWORD *dnLen) {
 
    BOOL b;
    HCERTSTORE hSystemStore = 0;
    PCCERT_CONTEXT pCertContext = NULL;
    
    if ( hSystemStore = CertOpenStore((LPCSTR)CERT_STORE_PROV_SYSTEM,
				      0,
				      0,
				      CERT_SYSTEM_STORE_CURRENT_USER,
				      L"My"
				      ) ) {
      
      
      pCertContext = locateCertificate(hSystemStore,certificate,certificate_length);
      
      if ( pCertContext ) {
	
	DWORD cchNameString;
	DWORD dwTypePara;
	
	dwTypePara = CERT_X500_NAME_STR;

	*dn = NULL;
	*dnLen = 0;
	
	if ( cchNameString = CertGetNameStringW(pCertContext,
					       CERT_NAME_RDN_TYPE,
					       issuerFlag,
					       (void*)&dwTypePara,
					       *dn,
					       cchNameString) ) {

	  *dn = (WCHAR*)malloc(cchNameString*sizeof(WCHAR));
	  
	  if ( *dn ) {
	    if ( cchNameString = CertGetNameStringW(pCertContext,
						   CERT_NAME_RDN_TYPE,
						   issuerFlag,
						   (void*)&dwTypePara,
						   *dn,
						   cchNameString) ) {
	      *dnLen = cchNameString;
	      lastError = OPENSIGN_ERROR_NONE;	    
	    } else {
	      lastError = OPENSIGN_ERROR_CERTGETNAMESTRING_VALUE_FAILURE;
	    }
	  } else {
	    lastError = OPENSIGN_ERROR_MEMORY_ALLOCATION_FAILURE;
	  }
	} else {
	  lastError = OPENSIGN_ERROR_CERTGETNAMESTRING_SIZE_FAILURE;
	}
      }
      
      b = CertCloseStore(hSystemStore,0);
      if ( b ) {
	lastError = OPENSIGN_ERROR_NONE;
      } else {
	lastError = OPENSIGN_ERROR_CERTSTORE_CLOSE_FAILURE;
      }
    } else {
      lastError = OPENSIGN_ERROR_CERTSTORE_OPEN_FAILURE;
    }
    
    return lastError == OPENSIGN_ERROR_NONE ? 0 : -1;
  }

  int sign(BYTE *certificate, DWORD certificate_length, BYTE *to_be_signed, DWORD to_be_signed_length, BYTE **signature, DWORD *signature_length) {
  
    BOOL b;
    HCERTSTORE hSystemStore = 0;
    PCCERT_CONTEXT pCertContext = NULL;
  
    if ( hSystemStore = CertOpenStore((LPCSTR)CERT_STORE_PROV_SYSTEM,
				      0,
				      0,
				      CERT_SYSTEM_STORE_CURRENT_USER,
				      L"My"
				      ) ) {

      pCertContext = locateCertificate(hSystemStore,certificate,certificate_length);

      if ( pCertContext ) {
	DWORD pcbData;
	CRYPT_KEY_PROV_INFO *keyProvInfo;
      
	if ( CertGetCertificateContextProperty(pCertContext,CERT_KEY_PROV_INFO_PROP_ID,NULL,&pcbData) ) {
	  keyProvInfo = (CRYPT_KEY_PROV_INFO*)malloc(pcbData);
	
	  if ( CertGetCertificateContextProperty(pCertContext,CERT_KEY_PROV_INFO_PROP_ID,keyProvInfo,&pcbData) ) {
	    HCRYPTPROV hCryptProv;
	    HCRYPTHASH hHash;
	    char *containerName, *providerName;
 
	    containerName = utf16string_convert(keyProvInfo->pwszContainerName);
	    providerName = utf16string_convert(keyProvInfo->pwszProvName);
	  
	    if(CryptAcquireContext(
				   &hCryptProv,
				   containerName,
				   providerName,
				   keyProvInfo->dwProvType,
				   0) ) {
	    
	      if ( CryptCreateHash(
				   hCryptProv,
				   CALG_SHA1,
				   0,
				   0,
				   &hHash) ) {
		BYTE* data;
		DWORD dataLen;

		if ( CryptHashData(
				   hHash,
				   to_be_signed,
				   to_be_signed_length,
				   0)) {
		
		  int success = 1;
		
		  DWORD dwKeySpec = AT_KEYEXCHANGE;
		
		  if ( CryptSignHash(
				     hHash,
				     dwKeySpec,
				     NULL,
				     0,
				     NULL,
				     signature_length) ) {
		  } else {
		    dwKeySpec = AT_SIGNATURE;
		    if ( CryptSignHash(
				       hHash,
				       dwKeySpec,
				       NULL,
				       0,
				       NULL,
				       signature_length) ) {
		    } else {
		      success = 0;
		    }
		  }
		
		  if ( success ) {
		    *signature = (BYTE*)malloc(sizeof(BYTE)*(*signature_length));

		    if ( CryptSignHash(
				       hHash,
				       dwKeySpec,
				       NULL,
				       0,
				       *signature,
				       signature_length) ) {
		      reverse(*signature,*signature_length);
		      lastError = OPENSIGN_ERROR_NONE;
		    } else {
		      lastError = OPENSIGN_ERROR_HHASH_SIGNATURE_VALUE_UNAVAILABLE;
		    }
		  } else {
		    lastError = OPENSIGN_ERROR_HHASH_SIGNATURE_SIZE_UNAVAILABLE;
		  }
		
		  if ( !CryptDestroyHash(hHash) ) {
		    lastError = OPENSIGN_ERROR_HHASH_DESTROY_FAILURE;
		  }
		} else {
		  lastError = OPENSIGN_ERROR_HHASH_HASHDATA_FAILURE;
		}
		if ( CryptReleaseContext(hCryptProv,0) ) {
		} else {
		  lastError = OPENSIGN_ERROR_HCRYPTPROV_RELEASE_FAILURE;
		}
	      } else {
		lastError = OPENSIGN_ERROR_HHASH_CREATE_FAILURE;
	      }
	    } else {
	      lastError = OPENSIGN_ERROR_HCRYPTPROV_ACQUIRE_FAILURE;
	    } 
	    if ( providerName ) {
	      free(providerName);
	    }
	    if ( containerName ) {
	      free(containerName);
	    }
	  } else {
	    lastError = OPENSIGN_ERROR_CCONTEXT_PROVINFO_VALUE_UNAVAILABLE;
	  }    
	} else {
	  lastError = OPENSIGN_ERROR_CCONTEXT_PROVINFO_SIZE_UNAVAILABLE;
	}
      
	b = CertCloseStore(hSystemStore,0);
	if ( !b && ( lastError == OPENSIGN_ERROR_NONE ) ) {
	  lastError = OPENSIGN_ERROR_CERTSTORE_CLOSE_FAILURE;
	}
      } else {
	lastError = OPENSIGN_ERROR_CERTSTORE_CERTIFICATE_NOT_FOUND;
      }
    } else {
      lastError = OPENSIGN_ERROR_CERTSTORE_OPEN_FAILURE;
    }
  
    return lastError == OPENSIGN_ERROR_NONE ? 0 : -1;
  }

  int digest_sha1(BYTE *to_be_hashed, DWORD to_be_hashed_length, BYTE **digest_value, DWORD *digest_value_length) {

    HCRYPTPROV hCryptProv;  

    if ( CryptAcquireContext(&hCryptProv,
			     NULL,
			     NULL,
			     PROV_RSA_FULL,
			     CRYPT_VERIFYCONTEXT) ) {

      HCRYPTHASH hHash;

      if ( CryptCreateHash(hCryptProv,
			   CALG_SHA1,
			   0,
			   0,
			   &hHash) ) {

	if ( CryptHashData(hHash,
			   to_be_hashed,
			   to_be_hashed_length,
			   0) ) {

	  *digest_value = NULL;

	  if ( CryptGetHashParam(hHash,
				 HP_HASHVAL,
				 *digest_value,
				 digest_value_length,
				 0)) {

	    *digest_value = (BYTE*)malloc(sizeof(BYTE)*(*digest_value_length));

	    if ( CryptGetHashParam(hHash,
				   HP_HASHVAL,
				   *digest_value,
				   digest_value_length,
				   0) ) {
	      /* all ok */
	      lastError = OPENSIGN_ERROR_NONE;
	    } else {
	      lastError = OPENSIGN_ERROR_DIGEST_VALUE_UNAVAILABLE;
	    }
	  } else {
	    lastError = OPENSIGN_ERROR_DIGEST_SIZE_UNAVAILABLE;
	  }
	}
			 
	if ( CryptDestroyHash(hHash) ) {
	} else {
	  lastError = OPENSIGN_ERROR_HHASH_DESTROY_FAILURE;
	}
      } else {
	lastError = OPENSIGN_ERROR_HHASH_CREATE_FAILURE;
      }

      if ( CryptReleaseContext(hCryptProv,
			       0) ) {
      } else {
	lastError = OPENSIGN_ERROR_HCRYPTPROV_RELEASE_FAILURE;
      }
    } else {
      lastError = OPENSIGN_ERROR_HCRYPTPROV_ACQUIRE_FAILURE;
    }
  
    return ( lastError == OPENSIGN_ERROR_NONE ) ? 0 : -1;
  }

  void freeCertificateList(struct certificate_data* head) {
    while ( head != NULL ) {
      struct certificate_data *l;
      l = head;
      if ( l->pbCertEncoded ) {
	free(l->pbCertEncoded);
      }
      head = head->prev;
      free(l);
    }
  }

  struct certificate_data* getCertificatesInMyStore() {

    struct certificate_data *certificate_list_head = NULL;

    BOOL b;
    HCERTSTORE hSystemStore = 0;
    PCCERT_CONTEXT pCertContext = NULL;
    int count = 0;

    hSystemStore = CertOpenStore((LPCSTR)CERT_STORE_PROV_SYSTEM,
				 0,
				 0,
				 CERT_SYSTEM_STORE_CURRENT_USER,
				 L"My"
				 );

    if ( hSystemStore ) {
    } else {
      lastError = OPENSIGN_ERROR_CERTSTORE_OPEN_FAILURE;
      return NULL;
    }

    while ( pCertContext = CertEnumCertificatesInStore(hSystemStore,pCertContext) ) {
      if ( pCertContext ) {
	DWORD pcbData;

	/* check whether we can obtain a key provider info struct
	   we use this as a way of telling whether we can access
           the private key associated with the certificate
	*/
	if ( CertGetCertificateContextProperty(pCertContext,CERT_KEY_PROV_INFO_PROP_ID,NULL,&pcbData) ) {
	  struct certificate_data *entry;

	  entry = (certificate_data*)malloc(sizeof(struct certificate_data));
	  if ( !entry ) {
	    // FIXME: insert error handling
	    return NULL;
	  }

	  entry->cbCertEncoded = pCertContext->cbCertEncoded;
	  entry->pbCertEncoded = (byte*)malloc(pCertContext->cbCertEncoded*sizeof(byte));
	  
	  // FIXME: this is probably a rather slow copy
	  for ( int i=0; i<entry->cbCertEncoded; i++ ) {
	    entry->pbCertEncoded[i] = pCertContext->pbCertEncoded[i];
	  }
	  entry->prev = certificate_list_head;
	  
	  certificate_list_head = entry;
	  count++;
	}
      }
    }
    
    if ( CertCloseStore(hSystemStore,0) ) {
      //FIXME: insert error handling
    }

    return certificate_list_head;
  }

  int getCertificateTimestamp(BYTE *certificate,
			      DWORD certificate_length,
			      BOOL notBefore,
			      FILETIME *timestamp) {

    BOOL b;
    HCERTSTORE hSystemStore = 0;
    PCCERT_CONTEXT pCertContext = NULL;
    lastError = OPENSIGN_ERROR_NONE;
    
    if ( hSystemStore = CertOpenStore((LPCSTR)CERT_STORE_PROV_SYSTEM,
				      0,
				      0,
				      CERT_SYSTEM_STORE_CURRENT_USER,
				      L"My"
				      ) ) {
      
      pCertContext = locateCertificate(hSystemStore,certificate,certificate_length);

      if ( pCertContext ) {
	*timestamp = notBefore ? pCertContext->pCertInfo->NotBefore : pCertContext->pCertInfo->NotAfter;
      } else {
	lastError = OPENSIGN_ERROR_CERTSTORE_CERTIFICATE_NOT_FOUND;
      }
        
      b = CertCloseStore(hSystemStore,0);
      if ( b && lastError == 0  ) {
	lastError = OPENSIGN_ERROR_NONE;
      } else if ( !b ) {
	lastError = OPENSIGN_ERROR_CERTSTORE_CLOSE_FAILURE;
      }
    } else {
      lastError = OPENSIGN_ERROR_CERTSTORE_OPEN_FAILURE;
    }
    
    return lastError == OPENSIGN_ERROR_NONE ? 0 : -1;
  }

  int getCertificateVersion(BYTE *certificate,
			    DWORD certificate_length,
			    DWORD *version) {
	
    BOOL b;
    HCERTSTORE hSystemStore = 0;
    PCCERT_CONTEXT pCertContext = NULL;
    
    if ( hSystemStore = CertOpenStore((LPCSTR)CERT_STORE_PROV_SYSTEM,
				      0,
				      0,
				      CERT_SYSTEM_STORE_CURRENT_USER,
				      L"My"
				      ) ) {
      
      pCertContext = locateCertificate(hSystemStore,certificate,certificate_length);
      
      if ( pCertContext ) {
	*version = pCertContext->pCertInfo->dwVersion;
      }
      
      b = CertCloseStore(hSystemStore,0);
      if ( b ) {
	lastError = OPENSIGN_ERROR_NONE;
      } else {
	lastError = OPENSIGN_ERROR_CERTSTORE_CLOSE_FAILURE;
      }
    } else {
      lastError = OPENSIGN_ERROR_CERTSTORE_OPEN_FAILURE;
    }
    return lastError == OPENSIGN_ERROR_NONE ? 0 : -1;
  }

  int getCertificateSerialNumber(BYTE* certificate,
				 DWORD certificate_length,
				 CRYPT_INTEGER_BLOB *blSerialNumber) {

    BOOL b;
    HCERTSTORE hSystemStore = 0;
    PCCERT_CONTEXT pCertContext = NULL;
    
    if ( hSystemStore = CertOpenStore((LPCSTR)CERT_STORE_PROV_SYSTEM,
				      0,
				      0,
				      CERT_SYSTEM_STORE_CURRENT_USER,
				      L"My"
				      ) ) {
      
      pCertContext = locateCertificate(hSystemStore,certificate,certificate_length);
      
      if ( pCertContext ) {
	*blSerialNumber = pCertContext->pCertInfo->SerialNumber;
      }
      
      b = CertCloseStore(hSystemStore,0);
      if ( b ) {
	lastError = OPENSIGN_ERROR_NONE;
      } else {
	lastError = OPENSIGN_ERROR_CERTSTORE_CLOSE_FAILURE;
      }
    } else {
      lastError = OPENSIGN_ERROR_CERTSTORE_OPEN_FAILURE;
    }
    return lastError == OPENSIGN_ERROR_NONE ? 0 : -1;
  }

  int getCertificateKeyUsage(BYTE *certificate,
			     DWORD certificate_length,
			     BYTE* keyusage) {
	
    BOOL b;
    HCERTSTORE hSystemStore = 0;
    PCCERT_CONTEXT pCertContext = NULL;
    
    if ( hSystemStore = CertOpenStore((LPCSTR)CERT_STORE_PROV_SYSTEM,
				      0,
				      0,
				      CERT_SYSTEM_STORE_CURRENT_USER,
				      L"My"
				      ) ) {
      
      pCertContext = locateCertificate(hSystemStore,certificate,certificate_length);
      
      if ( pCertContext ) {
	BYTE keyUsage[2];
	if (CertGetIntendedKeyUsage(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
				    pCertContext->pCertInfo,
				    keyusage,
				    2)) {
	} else {
	  lastError = OPENSIGN_ERROR_KEYUSAGE_NOT_PRESENT;
	}
      }
      
      b = CertCloseStore(hSystemStore,0);
      if ( !b && lastError == OPENSIGN_ERROR_NONE ) {
	lastError = OPENSIGN_ERROR_CERTSTORE_CLOSE_FAILURE;
      } 
    } else {
      lastError = OPENSIGN_ERROR_CERTSTORE_OPEN_FAILURE;
    }
    return lastError == OPENSIGN_ERROR_NONE ? 0 : -1;
  }

#ifdef __cplusplus
}
#endif
#endif
