/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

package org.openoces.opensign.appletsupport.mail;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.*;
import java.util.Iterator;
import java.util.Properties;

import org.bouncycastle.asn1.ASN1InputStream;
import org.bouncycastle.cms.CMSEnvelopedData;
import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.RecipientId;
import org.bouncycastle.cms.RecipientInformation;
import org.bouncycastle.cms.RecipientInformationStore;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.openoces.opensign.appletsupport.AppletSupport;
import org.openoces.opensign.appletsupport.MailSupport;
import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.CertificateInfo;
import org.openoces.opensign.certificate.DecryptingCertificateHandler;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import javax.mail.*;

/**
 * This class implements s/mime utilities
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

public class SmimeUtil implements MailSupport {

    public SmimeUtil() {
        super();
        // we cannot add BC as security provider from the bootstrap applet.
        // BC must be a part of the signed code then as it must be loaded first.
        // BUT as we anyways aren't going to run it as an applet, this is fine!
        // we just add it directly! (we are a trustworthy application!)
        try {
            AccessController.doPrivileged(new PrivilegedExceptionAction() {
                public Object run() throws PrivilegedActionException {
                    Security.addProvider(new BouncyCastleProvider());
                    return null;
                }
            });
        } catch (PrivilegedActionException e) {
        }
/*		try {
			OS.getRestrictedAction().addSecurityProvider("org.bouncycastle.jce.provider.BouncyCastleProvider");
		} catch (IOException e) {
			e.printStackTrace();
		}
*/
    }
    protected DecryptingCertificateHandler findCertificate(RecipientId id){
        // todo: go through the keystores and check if they have an
        // issuerDN and
        // serialNumber matching id
         AbstractCertificateHandler currentCertificate = null;// TODO: make do something useful
         if (currentCertificate instanceof DecryptingCertificateHandler){
             return (DecryptingCertificateHandler)currentCertificate;
         } else {
             return null;
         }

    }

    public String parsePkcs7Content(byte [] data, DecryptingCertificateHandler suggestedCert) throws SecurityException, IOException, Exception {
             ASN1InputStream in = new ASN1InputStream(new ByteArrayInputStream(data));
             CMSEnvelopedData envData = null;
            try {
                envData = new CMSEnvelopedData(in);
            } catch (CMSException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

             RecipientInformationStore   recipients = envData.getRecipientInfos();
             Iterator recIter = recipients.getRecipients().iterator();
             boolean first = true;
             while (recIter.hasNext()){
                 RecipientInformation recipient = (RecipientInformation) recIter.next();
                 RecipientId id = recipient.getRID();
                 FileLog.debug("Need for decryption: subject:"+id.getSubjectAsString()+" issuer: "+id.getIssuerAsString()+"serial: "+id.getSerialNumber());
                 if (first){
                     first = false;
                     continue;
                 }
                 DecryptingCertificateHandler h = suggestedCert;
                 // if wrong cert: call
                 // h = findCertificate(id);
                 CertificateInfo info = (CertificateInfo) h;
//	        	 FileLog.debug("XX Current cert: subject: "+info.getSubjectDN().getName()+" issuer: "+info.getIssuerDN().getName()+" serial: "+info.getSerialNumber());
                 if (h != null){
                     PrivateKey key = h.getPrivateKey();
                     if (key != null){
                          // MimeBodyPart res = SMIMEUtil.toMimeBodyPart(recipient.getContent(key, "BC"));
                         // return res.getContent();
                         // if we do not want to depend on mail.jar - and activation.jar:
//	        			 testKey(recipient, key);
                         try {
                             return new String(recipient.getContent(key, "BC")); // TODO: find out why "BC" here!
                         } catch (CMSException e){
                             FileLog.debug("CMSEx: "+e.getMessage());
                             FileLog.debug("underlying: "+e.getUnderlyingException().getMessage());
                         }
                     } else {
                         // will do it itself
                         return new String(h.decrypt(data));
                     }
                 }
             }
             return "Failed to decrypt mail";
        }
    /**
     * @param args
     */
    public static void main(String[] args) throws MessagingException, IOException {
        // TODO Auto-generated method stub
        checkMail(args[0], args[1], args[2]);
    }

    private static void checkMail(String pop3server, final String account, final String pw) throws MessagingException, IOException {
        // Get system properties
        Properties props = System.getProperties();
        props.put("mail.pop3.host", pop3server);

        // Setup authentication, get session
        Authenticator auth = new Authenticator(){
              protected javax.mail.PasswordAuthentication getPasswordAuthentication() {
                    return new PasswordAuthentication(account, pw);
                }
          };
          Session session =
            Session.getDefaultInstance(props, auth);

          // Get the store
          Store store = session.getStore("pop3");
          store.connect();

          // Get folder
          Folder folder = store.getFolder("INBOX");
          folder.open(Folder.READ_ONLY);

          // Get directory
          Message message[] = folder.getMessages();
          for (int i=0, n=message.length; i<n; i++) {

             System.out.println(i + ": "
               + message[i].getFrom()[0]
               + "\t" + message[i].getSubject());
             String content =
               message[i].getContent().toString();
             if (content.length() > 200) {
               content = content.substring(0, 200);
             }
             System.out.print(content);
          }

          // Close connection
          folder.close(false);
          store.close();
          System.exit(0);
    }

}
