/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: PwCertInfo.java,v 1.4 2006/10/22 09:32:38 cara Exp $ */

package org.openoces.opensign.certificate;

import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.security.Principal;
import java.security.PrivateKey;
import java.util.Date;

import org.openoces.opensign.certificate.x509.KeyUsage;
import org.openoces.opensign.client.applet.CallBackHandler;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.exceptions.UserCancel;
/**
 * This class represents ..
 *
 * @author Preben Valeur  <prv@tdc.dk
 */
public abstract class PwCertInfo implements CertificateInfo, DecryptingCertificateHandler {
	protected File file;
	protected CallBackHandler handler; 
    private CertificateInfo certInfo;
    private byte [][] chain;
    protected Object keyStore;
    private boolean infoIsAvailable;
    private KeyUsage keyusage = null;
    
    public PwCertInfo(File file) {
        this.file = file;
    }
    public PwCertInfo(File file, char[] password) throws IOException, Exception {
    	this(file);
        getKeyStore(password);
        getCertificateChain(password);
    }

    public void setCallBackHandler(CallBackHandler handler) {
        this.handler = handler;
    }
    private CertificateInfo getCertInfo(char[] pw) throws IOException, Exception {
        if (certInfo == null) {
            certInfo = OS.getCryptoSupport().getCertInfo(getKeyStore(pw));
        }
        return certInfo;
    }
    /**
     * @param password
     * @return KeyStore
     * @throws java.io.IOException
     */

    protected synchronized Object getKeyStore (char[] password) throws IOException, Exception {
        if (keyStore == null){
            keyStore = OS.getCryptoSupport().getKeyStore(file, password);
        }
        return keyStore;
    }

    public Principal getSubjectDN() throws UserCancel, Exception, IOException {
        return getCertInfo(certInfo == null ? getPassword() : null).getSubjectDN();
    }

    public Principal getIssuerDN() throws UserCancel, Exception, IOException {
        return getCertInfo(certInfo == null ? getPassword() : null).getIssuerDN();
    };

    public BigInteger getSerialNumber() throws UserCancel, Exception {
        return getCertInfo(certInfo == null ? getPassword() : null).getSerialNumber();
    }

    public Date getNotBefore() throws UserCancel, Exception {
        return getCertInfo(certInfo == null ? getPassword() : null).getNotBefore();
    }

    public Date getNotAfter() throws UserCancel, Exception {
        return getCertInfo(certInfo == null ? getPassword() : null).getNotAfter();
    }

    public int getVersion() throws UserCancel, Exception {
        return getCertInfo(certInfo == null ? getPassword() : null).getVersion();
    }

    public byte[] sign(byte[] toBeSigned, char[] password) throws IOException, Exception {
        if (password == null || password.length == 0){
            password = getPassword();
        }
        if (canSign(password)){
           PrivateKey pk = OS.getCryptoSupport().getPrivateKey(getKeyStore(password),password);
           return OS.getCryptoSupport().sign(toBeSigned, pk);
        } else
            throw new Exception("certificate not signable");
    }

    public byte[] digest(byte[] toBeHashed) throws Exception {
        return OS.getCryptoSupport().sha1(toBeHashed);
    }

    public byte[] getCertificate() throws UserCancel, Exception, IOException {
        return getCertInfo(certInfo == null ? getPassword() : null).getCertificate();
    }

    /**
     *
     */
    public byte[][] getCertificateChain(char[] password) throws IOException, Exception {
        infoIsAvailable = true;
        if (chain == null){
            chain = OS.getCryptoSupport().getCertificateChain(getKeyStore(password));
        }
        return chain;
    }

    public String getKeyUsage() throws Exception {
        return getCertInfo(certInfo == null ? getPassword() : null).getKeyUsage();
    }

    private boolean canSign(char[] pw) {
        try {
            return getCertInfo(certInfo == null ? getPassword() : null).canSign();
        } catch (Exception e) {
            return false;
        }
    }

    public boolean canSign() throws IOException, UserCancel {
        return canSign(certInfo == null ? getPassword() : null);
    }

    private char[] getPassword() throws UserCancel {
        char[] pw = null;
        boolean passwordOK = false;
        while ( !passwordOK ) {
            pw = handler.promptPassword(ResourceManager.getString("DLG_PASSWORD_HEADER_PREFIX") + " " + file).toCharArray();
            // verify correctness of the password
            try {
                getCertInfo(pw);
                passwordOK = true;
            } catch (Exception e) {
                FileLog.info("Unable to extract information from certificate. An invalid password is a likely cause of this");
            }
        }

        return pw;
    }

    public KeyUsage getIntendedKeyUsage() throws Exception, IOException {
        if (keyusage == null) {
            keyusage = getCertInfo(certInfo == null ? getPassword() : null).getIntendedKeyUsage();
        }
        return keyusage;
    }

    public boolean isInfoAvailable() {
        return infoIsAvailable;  //To change body of implemented methods use File | Settings | File Templates.
    }

    /**
     * @return File
     */
    public File getFile() {
        return file;
    }
    
    /*
     * Decrypting stuff
     */
	private boolean canDecrypt(char[] pw) {
        try {
        	// TODO: find out where and if to find it in certificate
        	return true;
        	//           return getCertInfo(certInfo == null ? getPassword() : null).canSign();
        } catch (Exception e) {
            return false;
        }
    }    
	public boolean canDecrypt() throws IOException, UserCancel {
        return canDecrypt(certInfo == null ? getPassword() : null);
    }
    public PrivateKey getPrivateKey() throws Exception {
           char [] password = getPassword();
           if (canDecrypt(password)){
              PrivateKey pk = OS.getCryptoSupport().getPrivateKey(getKeyStore(password),password);
              return pk;
           } else {
        	   return null;
           }
    }
	// if above returns null: support this one
    public byte[] decrypt(byte[] encrypted) throws Exception {
    	return null;
    }
}
