/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: CertificateFilter.java,v 1.8 2006/08/12 19:02:31 cara Exp $ */

package org.openoces.opensign.certificate.filter;

/**
 * This class represents an abstract certificate filter
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.exceptions.InputDataError;
import org.openoces.opensign.exceptions.UserCancel;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.util.Enumeration;
import java.util.Vector;

public abstract class CertificateFilter {
    private static boolean used;
    private static Vector filters = new Vector();

    static {
        FileLog.debug("xxx CertificateFilter static cons " + CertificateFilter.class.getClassLoader());
    }

    /**
     * Check if the filter has been reset allready - and is then about to be reused
     *
     * @return used
     */
    public static boolean isUsed() {
        return used;
    }

    public static void reset() {
//        filters.clear(); not supported in 1.1
        filters = new Vector(); // is then!
        used = true;
    }

    public static void add(CertificateFilter filter) {
        FileLog.debug("Filter added: " + filter);
        filters.addElement(filter);
    }

    public static boolean acceptByAll(AbstractCertificateHandler cert) throws UserCancel, InputDataError {
        for (Enumeration enumeration = filters.elements(); enumeration.hasMoreElements();) {
            CertificateFilter filter = (CertificateFilter) enumeration.nextElement();
            if (!filter.accept(cert)) {
                FileLog.debug("Certificate rejected by filter " + filter);
                return false;
            }
        }
        return true;
    }

    public abstract boolean accept(AbstractCertificateHandler cert) throws UserCancel, InputDataError;

    public abstract void setParameter(String parameter);
}
