/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: CapiKeyStoreHandler.java,v 1.6 2006/08/12 19:02:32 cara Exp $ */

package org.openoces.opensign.certificate.plugin.capi;

/**
 * This class represents the interface to the Microsoft cryptoapi
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.plugin.capi.wrapper.Installer;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;
import org.openoces.opensign.client.applet.CallBackHandler;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.client.applet.resources.ResourceManager;

import java.io.IOException;
import java.awt.*;

public class CapiKeyStoreHandler extends KeyStoreHandler {
    private Installer installer = null;
    public CapiKeyStoreHandler(){
//        install();
    }
    public CapiKeyStoreHandler(CallBackHandler handler, RestrictedAction ra)throws IOException {
        super(handler, ra);
//        install();
    }

    public void setRestrictedAction(RestrictedAction ra){
        super.setRestrictedAction(ra);
        install(ra);
    }

    private boolean install(RestrictedAction ra){
        installer = new Installer(ra);
        boolean ok = installer.install(Installer.OS_WIN32);
        if (!ok){
            FileLog.error("Unable to install Microsoft cryptoapi wrapper library");
        }
        return ok;
    }

    public void refreshKeystore() {
        certs = MicrosoftWindowsCertificateCache.getInstance(ra).getCertificates();
    }

    public boolean isSupported(RestrictedAction ra){
        this.ra = ra;
        return hasCAPI();
    }

    public String getName() {
        return "KEYSTORE_MS";  // or KEYSTORE_MSCAPI ?
    }

    public String getLocalizedName() {
        return ResourceManager.getString(getName());
    }

    private boolean canLoad(String lib) {
        try {
            FileLog.info("Just returning true from canLoad in capikeystorehandler!");
            return true;// todo: insert again somehow
//            return ra.loadLibrary(lib);
        } catch (UnsatisfiedLinkError ul) {
            return false;
        }
    }

    private static final String CAPI_LIB = "crypt32";

    public boolean hasCAPI() {
        boolean hasCAPISupport = OS.isOSWindows() && canLoad(CAPI_LIB);
        if (FileLog.isDebugEnabled()) {
            if (hasCAPISupport)
                FileLog.debug("CAPI is available");
            else
                FileLog.debug("CAPI is not available");
        }
        return hasCAPISupport;
    }
    public boolean supportsCopy(){
        return false;
    }
    public void finalize(){
        // or move to installer finalize but this is more explicit..
        installer.cleanup();
    }

    public void addPreferencesComponents(Panel panel, Frame frame) {
        // no preferences yet
    }

    public void execImportDialog(Frame frame) throws IOException {
    }

    public boolean supportsImport() {
        return false;
    }

    public boolean supportsExport() {
        return false;
    }

    public boolean supportsDelete() {
        return false;
    }
}
