/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: MicrosoftWindowsCertificateCache.java,v 1.5 2006/08/12 19:02:32 cara Exp $ */

package org.openoces.opensign.certificate.plugin.capi;

/**
 * This class represents the interface to the Microsoft cryptoapi where data
 * are cached instead of fetched from cryptoapi using expensive native calls
 * through the wrapper dll.
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.filter.CertificateFilter;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.exceptions.InputDataError;
import org.openoces.opensign.exceptions.UserCancel;
import org.openoces.opensign.wrappers.microsoftcryptoapi.MicrosoftCryptoApi;

public class MicrosoftWindowsCertificateCache {
    private static MicrosoftWindowsCertificateCache instance;

    public static MicrosoftWindowsCertificateCache getInstance(RestrictedAction ra) {
        if (instance == null) {
            instance = new MicrosoftWindowsCertificateCache(ra);
        }
        return instance;
    }

    private AbstractCertificateHandler[] certificates;
    private RestrictedAction ra;

    private MicrosoftWindowsCertificateCache(RestrictedAction ra) {
        this.ra = ra;
    }

    /**
     * Call if certificates may have been added or removed from MS keystore
     */
    public void clearCache() {
        certificates = null;
    }

    /**
     * call if filter has been changed
     * (currently just clears the cache so everything is done again)
     */
    public void refilter() {
        clearCache();
    }

    public AbstractCertificateHandler[] getCertificates() {
        if (certificates == null) {
            certificates = getFilteredCertificates();
        }
        return certificates;
    }

    private MicrosoftWindowsCertificateHandler[] getFilteredCertificates() {
        MicrosoftCryptoApi capi = new MicrosoftCryptoApi();
        byte[][] rs = capi.getCertificatesInSystemStore("My");
//        byte[][] rs = null;// = capi.getCertificatesInSystemStore("My");
        // reflection:
//        try {
//            rs = (byte [][])ra.invoke(capi, "getCertificatesInSystemStore", new Class []{String.class}, new Object [] {"My"});
//        } catch (Exception e){
//            e.printStackTrace();
//            return null;  // bummer
//        }

        FileLog.debug("Returned " + rs.length + " certificates from the microsoft cryptoapi");
        MicrosoftWindowsCertificateHandler[] rawList = new MicrosoftWindowsCertificateHandler[rs.length];
        int rawListIndex = 0;
//        int skipAfterCertCountForDevelopmentPurposes = 2;//todo: remove again later
        if (rs != null) {
            for (int i = 0; i < rs.length; i++) {
                try {
                    MicrosoftWindowsCertificateHandler h;
                    h = new MicrosoftWindowsCertificateHandler(rs[i], ra);
                    FileLog.debug("About to create microsoft certificate for certificate with index " + i);
                    if (CertificateFilter.acceptByAll(h)) {
                        rawList[rawListIndex++] = h;
                    }
//                    if (i >= skipAfterCertCountForDevelopmentPurposes){
//                        FileLog.debug("****************** developer edition - skipping rest of certs to speed up initialization *******");
//                        break;
//                    }
                } catch (InputDataError e) {
                    // N/A with MS certs
                } catch (UserCancel e) {
                    // N/A with MS certs
                }
            }
            MicrosoftWindowsCertificateHandler[] filteredList = new MicrosoftWindowsCertificateHandler[rawListIndex];
            System.arraycopy(rawList, 0, filteredList, 0, rawListIndex);
            return filteredList;
        } else {
            return null; // throw exception?
        }
    }
}
