/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: Installer.java,v 1.8 2006/10/15 08:38:19 cara Exp $ */

package org.openoces.opensign.certificate.plugin.capi.wrapper;

import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.utils.HexDump;
import org.openoces.opensign.wrappers.microsoftcryptoapi.MicrosoftCryptoApi;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * This class represents the Microsoft CryptoAPI wrapper dll installation
 *
 * @author Carsten Raskgaard   <carsten@raskgaard.dk>
 * @author Preben Valeur  <prv@tdc.dk>
 */

public class Installer {

    public static final int OS_WIN32 = 0;
    public static final int OS_LINUX = 1;

    private static String OCES_INSTALL_DIRECTORY = ".oces";
    private static String OPENSIGN_INSTALL_DIRECTORY = "opensign";
    private static String LIBRARY_INSTALL_DIRECTORY = "lib";

    /* should we verify the checksum of the dll on the client */
    private final static boolean doVerifyChecksum = true;

    private static final String LIBRARYNAME_WIN32 = "MicrosoftCryptoApi_0_2.dll";

    private static final byte[] LIBRARY_WIN32_DIGESTVALUE = {(byte) 0x42, (byte) 0x09, (byte) 0xee, (byte) 0x23, (byte) 0xb0, (byte) 0xc8, (byte) 0x12, (byte) 0xea,
                                                             (byte) 0x29, (byte) 0x8a, (byte) 0xe5, (byte) 0xbe, (byte) 0x9a, (byte) 0x8b, (byte) 0xed, (byte) 0x04,
                                                             (byte) 0x56, (byte) 0x6a, (byte) 0xe6, (byte) 0xb8};
    private RestrictedAction ra;
    public Installer(RestrictedAction ra){
        this.ra = ra;
    }
    // Create runtime dir.
    private void createRuntimeDir(File localDirectory){
        String tempDirName = "runtime-" + (int) (Math.random() * 10000000);
        runtimeDirectory = new File(localDirectory, tempDirName);
        ra.mkdirs(runtimeDirectory);
    }
    public void cleanup(){
        // delete what can be deleted - maybe not the dll - but try:
        FileLog.debug("installer cleaning up");
        if (runtimeDirectoryDll != null){
            if (ra.delete(runtimeDirectoryDll) && ra.delete(runtimeDirectory)){
                FileLog.debug("deleted dll and directory");
            } else {
                FileLog.debug("Couldn't clean up - could try on some of the other files/dirs");
                String [] otherDirs = ra.list(new File(runtimeDirectory.getParent()), new FilenameFilter(){
                    public boolean accept(File f, String s){
                        return true;
                    }
                });
                for (int i = 0; i < otherDirs.length; i++) {
                    String dirName = otherDirs[i];
                    File dir = new File(runtimeDirectory.getParent(), dirName);
                    FileLog.debug("checking dir "+dir);
//                    if (!dir.isDirectory()) continue;
                    if (ra.delete(new File(dir, LIBRARYNAME_WIN32)) && ra.delete(dir)){
                        FileLog.debug("succes in deleting dir "+dir);
                    } else {
                        FileLog.debug("failed in deleting dir "+dir);
                    }
                }
            }
        }
    }
    private File runtimeDirectory;
    private File runtimeDirectoryDll;
    private File getRuntimeDirectory() {
        return runtimeDirectory;
    }
    /**
     * Creates a copy of the file that can be used runtime and returns the path to the file.
     * @param installedFile
     * @return
     */
    private File createRuntimeCopy(File installedFile) {
        File src = installedFile;
        File dest = null;

        int index = installedFile.toString().lastIndexOf('.');
        if (index == -1)
            throw new IllegalArgumentException("");

        dest = new File(getRuntimeDirectory(), installedFile.getName());

        if (!ra.exists(src))
            throw new IllegalStateException("This method requires that the library '" + installedFile.toString() + "' is installed. ");

        FileInputStream is = null;
        try {
            is = ra.getFileInputStream(src);
            saveToFile(is, dest);
        } finally {
            try {
                if (is != null)
                    is.close();
            } catch (IOException e3) {
                e3.printStackTrace();
            }
        }

        return dest;
    }

    void saveToFile(InputStream data, File file) {
        if (data == null)
            throw new IllegalArgumentException("Missing input stream.");

        OutputStream os = null;
        os = ra.getFileOutputStream(file);

        int bufsize = 10000;
        byte[] buf = new byte[bufsize];
        int count;
        try {
            while ((count = data.read(buf)) > 0) {
                os.write(buf, 0, count);
            }
        } catch (IOException e1) {
            e1.printStackTrace();
        } finally {
            if (data != null)
                try {
                    data.close();
                } catch (IOException e3) {
                    e3.printStackTrace();
                }
            if (os != null)
                try {
                    os.close();
                } catch (IOException e4) {
                    e4.printStackTrace();
                }
        }
    }

    public boolean install(int os) {
        String relativePath;

        relativePath = OCES_INSTALL_DIRECTORY + File.separator + OPENSIGN_INSTALL_DIRECTORY +
                File.separator + LIBRARY_INSTALL_DIRECTORY;

        File installDirectory = new File(ra.getUserHome() + File.separator + relativePath);

        FileLog.info("microsoft crypto api wrapper library installation directory: " + installDirectory.getAbsolutePath());

//        if (!OS.javaplugin && OS.isOSWindows()) {
//            try {
//                com.ms.security.PolicyEngine.assertPermission((com.ms.security.PermissionID.FILEIO));
//            } catch (Throwable t) {
//            }
//        }

        if (!ra.exists(installDirectory)) {
            FileLog.warn("directories for the microsoft cryptoapi wrapper dll do not exist");
            if (!ra.mkdirs(installDirectory)) {
                FileLog.warn("Unable to create directories for microsoft cryptoapi wrapper dll");
                return false;
            }
        } else {
            FileLog.warn("directories for the microsoft cryptoapi wrapper dll do exist");
        }

        return install(os, installDirectory);
    }

    public boolean install(int os, File location) {

        File f = getLibraryFile(os,location);

        if ( f == null ) {
            FileLog.warn("Unable to generate filename for the microsoft cryptoapi wrapper dll");
            return false;
        }

        if ( !ra.exists(f) ) {
            if ( !copyLibrary(os,f) ) {
                FileLog.warn("Unable to copy library");
                return false;
            }
        }

        if ( !verifyLibrary(f) ) {
            FileLog.warn("Library verification failed. Library may have been tampered with. Installing fresh library.");

            /* try installing the library again */
            if ( !copyLibrary(os,f) ) {
                FileLog.warn("Unable to copy library");
                return false;
            }

            if ( !verifyLibrary(f) ) {
                FileLog.warn("Second library verification failed. It may have been tampered with. Bailing out.");
                return false;
            }
        }

        if (!loadLibrary(os, f)) {
            FileLog.warn("Unable to load microsoft cryptoapi wrapper dll");
            return false;
        }

        MicrosoftCryptoApi mscapi = new MicrosoftCryptoApi();
//        FileLog.info("Microsoft cryptoapi wrapper dll installation and loading task completed");
//        int majorVersion = 0;
//        int minorVersion = 0;
//        try {
//            System.out.println("ra: "+ra+" mscapi: "+mscapi);
//            Object o = ra.invoke(mscapi, "getMajorVersion", null, null);
//            System.out.println("o: "+o);
//            majorVersion = ((Integer) ra.invoke(mscapi, "getMajorVersion", null, null)).intValue();
//            minorVersion = ((Integer) ra.invoke(mscapi, "getMinorVersion", null, null)).intValue();
//        } catch (Exception e){
//            e.printStackTrace();
//        }
//        FileLog.info("Microsoft cryptoapi wrapper dll version: "+majorVersion+"."+minorVersion);
        FileLog.info("Microsoft cryptoapi wrapper dll version: "+mscapi.getMajorVersion()+"."+mscapi.getMinorVersion());
        return true;
    }

    private boolean loadLibrary(int os, File f) {
        if (os == OS_WIN32) {
            return loadLibraryOnWindowsPlatform(f);
        } else if (os == OS_LINUX) {
            FileLog.info("Microsoft cryptoapi wrapper dll can not be loaded on the Linux platform");
            return false;
        } else {
            FileLog.info("Microsoft cryptoapi wrapper dll can not be loaded on an unknown platform");
            return false;
        }
    }

    private File getLibraryFile(int os, File location) {
        if (os == OS_WIN32) {
            return new File(location, LIBRARYNAME_WIN32);
        } else {
            return null;
        }
    }

    private boolean copyLibrary(int os, File f) {
        if (os == OS_WIN32) {
//            if (!OS.javaplugin && OS.isOSWindows()) {
//                try {
//                    com.ms.security.PolicyEngine.assertPermission((com.ms.security.PermissionID.FILEIO));
//                } catch (Throwable t) {
//                }
//            }
            return copyLibraryOnWindowsPlatform(f);
        } else if (os == OS_LINUX) {
            return false;
        } else {
            return false;
        }
    }

    private boolean loadLibraryOnWindowsPlatform(File f) {
        try {
            if (!isLoaded()) {
                ra.load(f.toString());
//                System.load(f.toString());
            }
/* FIXME: How can we test whether the native library is working?
      One test can be calling some trivial native method
*/
        } catch (UnsatisfiedLinkError e) {
            // copy it somewhere and try again!
            FileLog.debug("Creating a runtime copy of the dll - remember to delete it afterwards...");
            createRuntimeDir(new File(f.getParent()));
            runtimeDirectoryDll = createRuntimeCopy(f);
            ra.load(runtimeDirectoryDll.toString());
        }
        return true;
    }

    private boolean verifyLibrary(File f) {
        if ( !doVerifyChecksum ) {
            return true;
        }

        FileInputStream ios = null;
        MessageDigest sha;

        try {
            byte[] data;
            byte[] hash;
            int readBytes;

            data = new byte[256];

            sha = MessageDigest.getInstance("SHA-1");
            ios = ra.getFileInputStream(f);

            while ((readBytes = ios.read(data)) != -1) {
                sha.update(data, 0, readBytes);
            }
            ios.close();
            hash = sha.digest();

            if (hash.length != LIBRARY_WIN32_DIGESTVALUE.length) {
                FileLog.warn("Calculated and stored digest have different lengths");
                return false;
            }

            for (int i = 0; i < hash.length; i++) {
                byte b = hash[i];
                byte c = LIBRARY_WIN32_DIGESTVALUE[i];
                if (b != c) {
                    FileLog.warn("Calculated and stored digest do not match at position " + i);
                    FileLog.warn("Calculated hash:\n" + HexDump.xdump(hash));
                    FileLog.warn("Stored hash:\n" + HexDump.xdump(LIBRARY_WIN32_DIGESTVALUE));
                    return false;
                }
            }
            FileLog.debug("Library verification was successful");
            return true;

        } catch (FileNotFoundException e) {
            FileLog.warn("Library to be verified not found");
        } catch (IOException e) {
            FileLog.warn("IO error occured while verifying library integrity");
        } catch (NoSuchAlgorithmException e) {
            FileLog.warn("Missing algorithm while verifying library integrity");
        }
        return false;
    }

    private boolean copyLibraryOnWindowsPlatform(File f) {
        URLConnection urlCon = null;
        InputStream fis = null;

        try {
//            FileLog.debug("Base url is "+DownloadItem.getBaseUrl());
            FileLog.debug("Base url is "+ra.getBaseUrl());
            urlCon = new URL(ra.getBaseUrl() + "/" + LIBRARYNAME_WIN32).openConnection();
            ra.connect(urlCon);
            FileLog.debug("connected to "+urlCon);
            fis = ra.getInputStream(urlCon);
        } catch (IOException e) {
            FileLog.warn("failed to open url connection");
            return false;
        }

        if (urlCon == null) {
            FileLog.warn("library not available");
            return false;
        }

        FileLog.debug("Downloading microsoft crypto api wrapper dll from "+urlCon);

        try {

//            if (!OS.javaplugin && OS.isOSWindows()) {
//                try {
//                    com.ms.security.PolicyEngine.assertPermission((com.ms.security.PermissionID.FILEIO));
//                } catch (Throwable t) {
//                }
//            }



/* FIXME: We should restrict the permissions set on the file
*/
            FileOutputStream fos = ra.getFileOutputStream(f);
            int ch;

            while ((ch = fis.read()) != -1) {
                fos.write(ch);
            }

            fis.close();
            fos.close();

        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    private boolean isLoaded() {
        boolean isLoaded = false;
        try {
            MicrosoftCryptoApi capi = new MicrosoftCryptoApi();
            capi.getMajorVersion();
            isLoaded = true;
            FileLog.debug("mscapi wrapper dll is already loaded");
        } catch (UnsatisfiedLinkError e) {
            FileLog.debug("mscapi wrapper dll has not already been loaded");
        } catch (Exception e) {
            FileLog.warn("Unexpected exception occured while checking if the mscapi wrapper is loaded: " + e.toString());
        }

        return isLoaded;
    }
}
