/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: CdCardKeyStoreHandler.java,v 1.6 2006/08/12 19:02:34 cara Exp $ */

package org.openoces.opensign.certificate.plugin.cdcard;

/**
 * This class represents the interface to the CdCard keystore
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.filter.CertificateFilter;
import org.openoces.opensign.certificate.plugin.cdcard.dialogs.DlgImport;
import org.openoces.opensign.certificate.plugin.cdcard.util.AksClient;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;
import org.openoces.opensign.client.applet.CallBackHandler;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.dialogs.AbstractDialog;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.applet.Applet;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Vector;
import java.awt.*;

public class CdCardKeyStoreHandler extends KeyStoreHandler {
    public CdCardKeyStoreHandler() {
//        init();
    }

    public CdCardKeyStoreHandler(CallBackHandler handler, RestrictedAction ra) throws IOException {
        super(handler, ra);
        init(ra);
    }

    protected void init(RestrictedAction ra) {
        cardDir = new File(ra.getUserHome(), CDCARD_DIR_NAME);
    }

    public void refreshKeystore() {
//        certs = CdCardCertificateHandler.scanForCerts();
        certs = scanForCerts();
    }

    public boolean isSupported(RestrictedAction ra) {
        return true;
    }

    public String getName() {
        return "KEYSTORE_CDCARD";
    }

    public String getLocalizedName() {
        return ResourceManager.getString(getName());
    }

    RestrictedAction getRestrictedAction() {
        return ra;
    }

    private static final String CDCARD_DIR_NAME = "CDCARDS";

    protected static final String CDCARD_FILE_NAME_PREFIX = "cdkort";
    protected static final String CDCARD_FILE_NAME_POSTFIX = ".html";
    protected static final String CDCARD_FILE_NAME_ALT_POSTFIX = ".htm";

    private boolean scannedForCards = false; // determine policy for when to scan
    private Vector homeCardPaths = null; // paths to cards in home dir
    private Vector otherCardPaths = null; // paths to cards not in home dir
    private File cardDir;
    private AbstractCertificateHandler[] allCerts;

    protected File getCardDir() {
        return cardDir;
    }

    public void forgetScan() {
        scannedForCards = false;
    }

    public boolean hasCard() {
        // scan the machine for cd cards
        if (!scannedForCards) {
            otherCardPaths = scanForRootCards();
            homeCardPaths = scanForHomeCards();
            readAllCerts();
            scannedForCards = true;
        }
        return (otherCardPaths.size() > 0) || (homeCardPaths.size() > 0);
    }

    /**
     * Find the CD CARD files in "dir" and add them to the vector "cardPaths" if any found
     */
    protected void addFromDir(File dir, Vector cardPaths) {
        String[] fileNames = ra.list(dir, null);
        if (fileNames == null) return;
        for (int i = 0; i < fileNames.length; i++) {
            String fileName = fileNames[i];
//            if (getFilenameFilter().accept(dir, fileName)) { // alternative and maybe nicer way
            if (fileName.indexOf(CDCARD_FILE_NAME_PREFIX) == 0 && (fileName.endsWith(CDCARD_FILE_NAME_POSTFIX) || fileName.endsWith(CDCARD_FILE_NAME_ALT_POSTFIX))) {
                String path = dir.getAbsolutePath() + File.separator + fileName;
                cardPaths.addElement(path);
                FileLog.debug("scanForCard: adding path: " + path);
            }
        }
    }

    private Vector scanForHomeCards() {
        FileLog.debug("CopyableCdCardCertificateHandler.scanForCards: it is a windows machine");
        Vector cardPaths = new Vector();
        File cardHome = getCardDir();
        ra.mkdirs(cardHome);
        addFromDir(cardHome, cardPaths);
        return cardPaths;
    }

    private Vector scanForRootCards() {
        Vector cardPaths = new Vector();
        if (OS.isOSWindows()) {
            FileLog.debug("RestoreableCdCardCertificateHandler.scanForCards: it is a windows machine");
            for (char c = 'a'; c <= 'z'; c++) {
                if (ra.exists(new File(c + ":\\"))) {
                    FileLog.debug("scanForCard: drive " + c + " exists");
                    addFromDir(new File(c + ":\\"), cardPaths);
                }
            }
        } else {
            FileLog.debug("scanForCard: it is NOT a windows machine - CD Card support not yet implemented");
        }
        return cardPaths;
    }


    /**
     * Scan the system for CD Card certs
     *
     * @return array of CD Card certs or null if none found
     */

    public AbstractCertificateHandler[] scanForCerts() {
        if (!scannedForCards) {
            otherCardPaths = scanForRootCards();
            homeCardPaths = scanForHomeCards();
            readAllCerts();
            scannedForCards = true;
        }
        return allCerts;
    }

    private void readAllCerts() {
        // some of the cards found may contain errors - so
        Vector certList = new Vector();
        for (int i = 0; i < otherCardPaths.size(); i++) {
            String cardPath = (String) otherCardPaths.elementAt(i);
            try {
                AbstractCertificateHandler h = new RestoreableCdCardCertificateHandler(this, cardPath);
                if (CertificateFilter.acceptByAll(h)) {
                    // you COULD check for duplicates here but we choose not to
                    certList.addElement(h);
                } else {
                    FileLog.debug("Cert filtered out: " + h);
                }
            } catch (Exception e) {
                FileLog.error("Error in cd card file from path " + cardPath + " : " + e.getMessage());
                e.printStackTrace();
            }
        }
        AbstractCertificateHandler[] cards = new AbstractCertificateHandler[certList.size()];
        certList.copyInto(cards);
        // as toArray() is not 1.1
        // AbstractCertificateHandler [] cards = (RestoreableCdCardCertificateHandler []) certList.toArray(new RestoreableCdCardCertificateHandler[0]);
        // then the homecards
        // todo: find out what to do with duplicates!
        certList = new Vector();
        for (int i = 0; i < homeCardPaths.size(); i++) {
            String cardPath = (String) homeCardPaths.elementAt(i);
            try {
                certList.addElement(new CopyableCdCardCertificateHandler(this, cardPath));
            } catch (Exception e) {
                FileLog.error("Error in cd card file from path " + cardPath + " : " + e.getMessage());
                e.printStackTrace();
            }
        }
        AbstractCertificateHandler[] homeCards = new AbstractCertificateHandler[certList.size()];
        certList.copyInto(homeCards);
        //AbstractCertificateHandler [] homeCards = (CopyableCdCardCertificateHandler []) certList.toArray(new CopyableCdCardCertificateHandler[0]);
        // combine the two:
        allCerts = new AbstractCertificateHandler[cards.length + homeCards.length];
        System.arraycopy(cards, 0, allCerts, 0, cards.length);
        System.arraycopy(homeCards, 0, allCerts, cards.length, homeCards.length);
        FileLog.debug("CdCard.scanForCerts() returning " + allCerts.length);
    }

    public boolean supportsCopy() {
        return true;
    }

    private FilenameFilter filenameFilter;

    public FilenameFilter getFilenameFilter() {
        if (filenameFilter == null) {
            filenameFilter = new FilenameFilter() {
                public boolean accept(File path, String file) {
//                    return ((file.indexOf("cdkort-") == 0));
                    return ((file.indexOf("cdkort-") == 0) && (file.endsWith(".html") || file.endsWith(".htm")));
                }
            };
        }
        return filenameFilter;
    }

    public AbstractCertificateHandler getCertificateFromFile(File file) {
        File parentDir = new File(file.getParent());
        if (!getFilenameFilter().accept(parentDir, file.getName())) {
            return null;
        }
        AbstractCertificateHandler cert = null;
        try {
            cert = new RestoreableCdCardCertificateHandler(this, file.getAbsolutePath());
        } catch (Exception e) {
            FileLog.info("Not a cdcard though it looked like one");
        }
        return cert;
    }

    private class ShowDialog implements Runnable {
        private AbstractDialog dialog;
        public ShowDialog(AbstractDialog dialog) {
            this.dialog = dialog;
        }
        public void run() {
            dialog.show();
        }
    }
    public void service(Applet applet, String serviceName, String serviceParam) {
        String prodParam = applet.getParameter("cdkortprod");
        boolean testEnv = (prodParam == null || prodParam.length()==0);
        if (testEnv){
            AksClient.setEnv(AksClient.ENV_TEST);
        } else {
            if (prodParam.equalsIgnoreCase("dev")){
                AksClient.setEnv(AksClient.ENV_DEV);
            } else if (prodParam.equalsIgnoreCase("stub")){
                AksClient.setEnv(AksClient.ENV_STUB);
            } else {
                AksClient.setEnv(AksClient.ENV_PROD);
            }
        }
        DlgImport dlg = new DlgImport(applet, ra, handler.getFrame(), "Indl�s Digital Signatur", serviceParam);
        Thread showDialog = ra.createThread(new ShowDialog(dlg));
        showDialog.start();
    }

    public void addPreferencesComponents(Panel panel, Frame frame) {
        // no preferences yet
    }

    public boolean supportsImport() {
        return false;
    }

    public boolean supportsExport() {
        return false;
    }

    public boolean supportsDelete() {
        return false;
    }

    public void execImportDialog(Frame frame) throws IOException {
    }
}
