/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: ChangePinCodePanel.java,v 1.6 2006/10/15 08:38:20 cara Exp $ */

package org.openoces.opensign.certificate.plugin.cdcard.dialogs;

/**
 * This class represents an special panel used when the cdcard pincode is being changed
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;

import org.openoces.opensign.certificate.plugin.cdcard.util.AksClient;
import org.openoces.opensign.certificate.plugin.cdcard.util.AksResponse;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.client.applet.resources.ResourceManager;

public class ChangePinCodePanel extends Panel {
    // interface
	private Component focusComponent;
    public boolean isOk() {
        return validateInput();
    }

    public AksResponse getAksResponse() {
        return aksResponse;
    }

    // end interface
    static final String PARAM_BGCOLOR = "BGCOLOR";
    static final String PARAM_FGCOLOR = "FGCOLOR";
    static final String PARAM_FONTFACE = "FONTFACE";
    static final String PARAM_FONTSIZE = "FONTSIZE";
    private String oldPin = "";
    private String newPin;
    private String newPinCopy;
    private String nid;
    private String service;
    private AksResponse aksResponse;

    private boolean debug;

    private TextArea resultText;
    private Button cancelButton;
    private Button okButton;
    private boolean displayEventsInResultText = true;

    private RestrictedAction ra;
    private ActionListener actionListener;

    private void setResultText(String txt, boolean overwriteMsg) {
    	if (txt == null){
    		txt = "";
    	}
        if (displayEventsInResultText) {
        	if (!overwriteMsg){
        		String oldText = resultText.getText();
        		if (oldText == null || oldText.trim().length() == 0){
                    resultText.setText(txt);
        		}
        	} else {
                resultText.setText(txt);
        	}
        }
    }
    private void setResultText(String txt) {
    	setResultText(txt, true);
    }

    public void enableDialog(boolean doEnable){
        String wait = ResourceManager.getString("PLEASE_WAIT");
        if (doEnable){
            if (wait.equals(resultText.getText())){
                setResultText("");
            } else {
                // we didn't write it so cannot erase it
            }
        } else {
            setResultText(wait);
        }
        cancelButton.setEnabled(doEnable);
        okButton.setEnabled(doEnable);
    }


    /**
     * Relevant params:
     *
     * @param params
     */
    public ChangePinCodePanel(ActionListener actionListener, RestrictedAction ra, String oldPin, String nid, String service) {
        debug("ImportPanel <cons>(" + ra + "," + oldPin);
        this.ra = ra;
        this.actionListener = actionListener;
        this.oldPin = oldPin;
        this.nid = nid;
        this.service = service;
        initialize();
    }
    public void takeFocus(){
    	focusComponent.requestFocus();
    }
    private Component createPinCodeField(){
        final TextField pinCodeField = new TextField();
        pinCodeField.setEchoChar('*');
        pinCodeField.setColumns(AksClient.PIN_CODE_LENGTH);
        pinCodeField.addTextListener(new TextListener() {
            public void textValueChanged(TextEvent e) {
            	newPin = pinCodeField.getText();
                displayEventsInResultText = true;
                setResultText(getPinCodeStatusText(newPin, newPinCopy));
            }
        });
        pinCodeField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
            	pinCodeField.selectAll();
                setResultText(getPinCodeStatusText(newPin, newPinCopy), false);
            }
            public void focusLost(FocusEvent e) {
//                setResultText("");
            }
        });
        return pinCodeField;
    }
    private Component createPinCodeCopyField(){
	    final TextField pinCodeCopyField = new TextField();
	    pinCodeCopyField.setEchoChar('*');
	    pinCodeCopyField.setColumns(AksClient.PIN_CODE_LENGTH);
	    pinCodeCopyField.addTextListener(new TextListener() {
	        public void textValueChanged(TextEvent e) {
	            newPinCopy = pinCodeCopyField.getText();
	            displayEventsInResultText = true;
	            setResultText(getPinCodeStatusText(newPin, newPinCopy));
	        }
	    });
	    pinCodeCopyField.addFocusListener(new FocusListener() {
	        public void focusGained(FocusEvent e) {
	        	pinCodeCopyField.selectAll();
	            setResultText(getPinCodeStatusText(newPin, newPinCopy));
	        }
	        public void focusLost(FocusEvent e) {
	//            setResultText("");
	        }
	    });
	    return pinCodeCopyField;
    }
    public Component getOkButton(){
    	return okButton;
    }
    public Component getCancelButton(){
    	return cancelButton;
    }
  public void initialize() {
        setLayout(new BorderLayout());

        Panel p = new Panel(new GridLayout(1,0));
        
        focusComponent = null;

        p.add(new Label("Opret PIN-kode"));
//        passphraseField.setFocusable(true);
        Component pinCodeField = createPinCodeField();
        
        p.add(pinCodeField);
        if (focusComponent == null) {
            focusComponent = pinCodeField;
        }
        p.add(new Label("Gentag PIN-kode"));
        Component pinCodeCopyField = createPinCodeCopyField();
        
        p.add(pinCodeCopyField);

        cancelButton = new Button("Annull�r");
        cancelButton.setActionCommand("CANCEL");
        cancelButton.addActionListener(actionListener);
        
        okButton = new Button("OK");
        okButton.setActionCommand("OK");
        okButton.addActionListener(actionListener);
//        button.addActionListener(new ButtonHandler());
        
        // if you want to put the buttons so they are quite wide in a grid-layout:
       
        add(p, BorderLayout.CENTER);

        resultText = new TextArea("", 3, 50, TextArea.SCROLLBARS_NONE);
        resultText.setEditable(false);
        resultText.setBackground(this.getBackground());
		KeyListener kl = new KeyListener() {
			public void keyTyped(KeyEvent evt) {
			}
			public void keyPressed(KeyEvent evt) {
				if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
					cancelClicked(evt.getSource());
				}
				if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
					okClicked(evt.getSource());
				}
			}
			public void keyReleased(KeyEvent evt) {
			}
		};
		KeyListener klCancel = new KeyListener() {
			public void keyTyped(KeyEvent evt) {
			}
			public void keyPressed(KeyEvent evt) {
				if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
					cancelClicked(evt.getSource());
				}
			}
			public void keyReleased(KeyEvent evt) {
			}
		};

		cancelButton.addKeyListener(kl);
		okButton.addKeyListener(kl);
		pinCodeField.addKeyListener(klCancel);
		pinCodeCopyField.addKeyListener(kl);
		
        add(resultText, BorderLayout.SOUTH);
        setFocusability();
    }
  	// not supported by MS JVM
  	private void setFocusability(){
/*        resultText.setFocusable(false);
        okButton.setFocusable(true);
        cancelButton.setFocusable(true);
        */
  	}
  	private void cancelClicked(Object source){
        actionListener.actionPerformed(new ActionEvent(source, 0, "Cancel"));
  	}
  	private void okClicked(Object source){
        actionListener.actionPerformed(new ActionEvent(source, 1, "OK"));
  	}
    private String getPinCodeStatusText(String p, String p2) {
        if (p == null) {
            p = "";
        }
        if (p2 == null) {
            p2 = "";
        }

        boolean pinCodeIsValid = true;
        StringBuffer b = new StringBuffer("Opret PIN-kode til beskyttelse af Digital Signatur.\n");
        if (p.length() != AksClient.PIN_CODE_LENGTH) {
            b.append("- PIN-koden skal indeholde "+AksClient.PIN_CODE_LENGTH+" cifre\n");
            pinCodeIsValid = false;
        }

        if (!allDigits(p)) {
            b.append("- PIN-koden skal v�re et "+AksClient.PIN_CODE_LENGTH+"-cifret tal\n");
            pinCodeIsValid = false;
        }

        if (p.equals("") || !p.equals(p2)) {
            b.append("- De to adgangskoder skal v�re ens\n");
            pinCodeIsValid = false;
        }

        if (p.equals(oldPin)) {
            b.append("- Den nye PIN-kode skal v�re forskellig fra den gamle.\n");
            pinCodeIsValid = false;
        }

        if (pinCodeIsValid) {
            return "Du har oprettet og gentaget en gyldig PIN-kode.";
        }
        return b.toString();

    }

    private static boolean allDigits(String s) {
        for (int i = 0; i < s.length(); i++) {
            if (!Character.isDigit(s.charAt(i))) {
                return false;
            }
        }
        return true;
    }


    private static boolean pinCodeValid(String pinCode) {

        if (pinCode == null) {
        	return false;
        }
        // pinCode skal v�re PIN_CODE_LENGTH tegn
	  if (pinCode.length() != AksClient.PIN_CODE_LENGTH) {
		  return false;
      }

      if (!allDigits(pinCode)) {
    	  return false;
      }
      return true;
    }

    private boolean validateInput() {
        displayEventsInResultText = true;
//            setResultText("");

        if (newPin == null || newPin.equals("")) {
            setResultText("Indtast PIN-kode");
            return false;
        }

        if (!pinCodeValid(newPin)) {
            setResultText("Du skal indtaste en "+AksClient.PIN_CODE_LENGTH+"-cifret PIN-kode.");
            return false;
        }
        if (newPinCopy == null) {
            setResultText("Gentag PIN-koden.");
            return false;
        }
        if (!newPin.equals(newPinCopy)) {
            setResultText("De to PIN-koder skal v�re ens.");
            return false;
        }
        // ok, now check with AKS server!
        AksClient client = AksClient.getInstance(ra);
        try {
        	aksResponse = client.getPasswordAndChangePin(nid, oldPin, newPin, service);
        	if (aksResponse.getStatus() != AksResponse.OK){
        		setResultText(aksResponse.getMsg());
        		return false;
        	}
        } catch (Exception e){
    		setResultText("Der skete en fejl ("+e.getMessage()+")");
        	return false;
        }
        return true;
    }


    private void debug(String message) {
        FileLog.debug(message);
    }

}

