/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: ConditionsHeaderPanel.java,v 1.7 2006/08/12 19:02:33 cara Exp $ */

package org.openoces.opensign.certificate.plugin.cdcard.dialogs;

/**
 * This class represents the header of the gui used to display the conditions that must be agreed to
 * before a cdcard can be used.
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */


import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.utils.X500Name;

import java.awt.*;
import java.util.StringTokenizer;


public class ConditionsHeaderPanel {
	public static final int ACTIVATE = 0;
	public static final int LRA_ACTIVATE = 1;
	public static final int LRA_RE_ACTIVATE = 2;
    public static String prependHeader(int state, AbstractCertificateHandler certificate, String conditions) throws Exception {
    	return prependHeader(state, certificate, conditions, null);
    }
    public static String prependHeader(int state, AbstractCertificateHandler certificate, String conditions, String lraInfo) throws Exception {
        String subjectDN = certificate.getSubjectDN().getName();
        X500Name name = new X500Name(subjectDN);
        String [] headerLines = getHeaderLines(name);
        StringBuffer buf = new StringBuffer();
        buf.append(getIntro(state));
        buf.append("\n");
        buf.append(getCertContentLine(name));
        buf.append("\n");
        buf.append(getOutro(state, lraInfo));
        if (conditions != null && conditions.indexOf("null")!=0){
            buf.append("\n");
            buf.append(conditions);
        }
        return buf.toString();
    }
    public static String getOkPrompt(int state){
    	switch (state){
    		case ACTIVATE:
    			return "Du aktiverer din nye digitale signatur ved at indtaste dit CPR nummer herunder og trykke \"Ok\".";
    		case LRA_ACTIVATE:
    	        return "Du aktiverer din nye digitale signatur ved at v�lge en PIN-kode herunder og trykke  \"Ok\".";
    		case LRA_RE_ACTIVATE:
    	        return "Du gen�bner din digitale signatur ved at v�lge en ny PIN-kode herunder og trykke  \"Ok\".";
    	}
    	return "error - unknown state";
    }
    private AbstractCertificateHandler certificate;
    private static String getCertContentLine(X500Name name){
        StringBuffer buf = new StringBuffer();
        buf.append("Indhold af signatur: Navn: ");
        buf.append(name.getCN());
        buf.append(", Organisation: ");
        buf.append(name.getO());
        return buf.toString();
    }
    private static String getIntro(int state){
    	switch (state){
    		case ACTIVATE:
    		case LRA_ACTIVATE:
    	        return "Du har modtaget en digital signatur p� CD kort, og dette er f�rste gang du anvender den.";
    		case LRA_RE_ACTIVATE:
    	        return "Den digitale signatur p� CD kort kan nu gen�bnes.";
    	}
    	return "internal error - unknown state";
    }
    private static String getOutro(int state, String lraInfo){
    	String lraInfoString = null;
    	if (lraInfo != null){
			StringTokenizer strTok = new StringTokenizer(lraInfo, ";");
			if (strTok.countTokens() == 2){
				StringBuffer buf = new StringBuffer();
				String lraName = strTok.nextToken();
				buf.append(lraName);
				String lraEmail = strTok.nextToken();
				if (lraEmail != null && !lraEmail.trim().equals("null") && lraEmail.trim().length() > 0){
					buf.append(" (");
					buf.append(lraEmail);
					buf.append(").\n");
				} else {
					buf.append("\n");
				}
				lraInfoString = buf.toString();
			} else {
				// no lra info for some reason...
//				lraInfoString = "internal error - malformed lra info";
			}
    	}
    	switch (state){
			case ACTIVATE:
		        return "Hvis indholdet af signaturen ikke er korrekt, skal du trykke \"Annuller\" herunder \n"+
		        	"og derefter destruere CD kort og pinkode.\n"+
		            "Du kan efterf�lgende bestille en ny signatur p� internet adressen erhverv.tdc.dk/digital.";
			case LRA_ACTIVATE:{
				StringBuffer buf = new StringBuffer();
		        buf.append("Hvis indholdet af signaturen ikke er korrekt, skal du trykke \"Annuller\" herunder \n");
	        	buf.append("og derefter destruere CD kortet.\n");
	        	if (lraInfoString != null){
	        		buf.append("Denne digitale signatur er udstedt af din LRA:\n");
					buf.append(lraInfoString);
	        	}
				return buf.toString();
			}
			case LRA_RE_ACTIVATE:{
				StringBuffer buf = new StringBuffer();
		        buf.append("Hvis indholdet af signaturen ikke er korrekt, skal du trykke \"Annuller\" herunder \n");
	        	buf.append("og derefter destruere CD kortet.\n");
				buf.append("Gen�bningen af din digitale signatur er foranlediget af din LRA:\n");
				buf.append(lraInfoString);
				return buf.toString();
			}
		}
		return "internal error - unknown state";
    }
    private static String [] getHeaderLines(X500Name name){
        String cn = name.getCN();
        String organization = name.getO();
        String [] res = {
        "Du har modtaget en digital signatur p� CD kort, og dette er f�rste gang du anvender den.",
        "Indhold af signatur: Navn: "+cn+", Organisation: "+organization,
        "Hvis indholdet af signaturen ikke er korrekt, skal du trykke \"Annuller\" herunder ",
        "og derefter destruere CD kort og pinkode.",
        "Du kan efterf�lgende bestille en ny signatur p� internet adressen erhverv.tdc.dk/digital.",
        "Hvis indholdet er korrekt, beder vi dig gennemg� de vilk�r der g�lder for din digitale signatur herunder.",
        "Du aktiverer din nye digitale signatur ved at indtaste dit CPR nummer herunder og trykke \"Ok\"."
        };
        return res;
    }
}
