/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DlgChangePinCode.java,v 1.5 2006/08/12 19:02:33 cara Exp $ */

package org.openoces.opensign.certificate.plugin.cdcard.dialogs;

/**
 * This class represents the dialog displayed when the cdcard pin code is being changed.
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.plugin.cdcard.util.AksResponse;
import org.openoces.opensign.certificate.plugin.cdcard.util.CprChecker;
import org.openoces.opensign.certificate.plugin.cdcard.util.PrintUtil;
import org.openoces.opensign.client.applet.SetPollApplet;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.dialogs.AbstractDialog;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Hashtable;
import java.util.Vector;

public class DlgChangePinCode extends AbstractDialog implements ActionListener {
    private ChangePinCodePanel changePinCodePanel;
    private Applet applet;
    private boolean cancelled;
    private AksResponse response;
    private Frame frame;
    private RestrictedAction ra;
    private String fullConditions;
    
    
    public boolean isCancelled() {
        return cancelled;
    }
    public AksResponse getAksResponse(){
    	return response;
    }
    /**
     * Used when reactivating cd card.
     * There are no conditions then.
     * @param applet
     * @param ra
     * @param owner
     * @param certificate
     * @param title
     * @param oldPin
     * @param nid
     * @param service
     * @param lraInfo
     * @param rows
     * @param cols
     */
    public DlgChangePinCode(int state, final Applet applet, RestrictedAction ra, Frame owner, AbstractCertificateHandler certificate, String title, String oldPin, String nid, String service, String lraInfo, int rows, int cols) {
    	this(state, applet, ra, owner, certificate, title, oldPin, nid, service, null, lraInfo, rows, cols);
    }
    /**
     * Used for first time activation
     * @param applet
     * @param ra
     * @param owner
     * @param certificate
     * @param title
     * @param oldPin
     * @param nid
     * @param service
     * @param conditions
     * @param lraInfo
     * @param rows
     * @param cols
     */
    public DlgChangePinCode(int state, final Applet applet, RestrictedAction ra, Frame owner, AbstractCertificateHandler certificate, String title, String oldPin, String nid, String service, String conditions, String lraInfo, int rows, int cols) {
        super(ra, owner, title);
        this.applet = applet;
        this.ra = ra;
        this.frame = owner;
        dialog.setBackground(Color.lightGray);
        dialog.setLayout(new BorderLayout());
        
        // first the conditions:
        try {
            FileLog.debug("adding certInfo");
//            FileLog.debug("adding certInfoPanel");
//            Panel certificateInfoPanel = new ConditionsHeaderPanel(certificate, 6, cols);
//            dialog.add(certificateInfoPanel, BorderLayout.NORTH);
            fullConditions = ConditionsHeaderPanel.prependHeader(state, certificate, conditions, lraInfo);
        } catch (Exception e) {
            FileLog.error("Exception when adding cert info " + e.getMessage());
        }

        TextArea ta = new TextArea(fullConditions, rows, cols, TextArea.SCROLLBARS_VERTICAL_ONLY);
        ta.setBackground(Color.white);
        ta.setEditable(false);
//        ta.setFocusable(false);
        // wrap in a panel for borders..
        Panel textAreaPanel = new Panel();
        textAreaPanel.add(ta);
        dialog.add(textAreaPanel, BorderLayout.CENTER);

        Panel okPromptPanel = new Panel();
        okPromptPanel.add(new Label(ConditionsHeaderPanel.getOkPrompt(state)));
        Panel southPanel = new Panel();
        southPanel.setLayout(new BorderLayout());
        southPanel.add(okPromptPanel, BorderLayout.NORTH);

        // todo: think about parsing the param and passing it in the Hashtable
        changePinCodePanel = new ChangePinCodePanel(this, ra, oldPin, nid, service);
        // wrap again:
        Panel wrapper = new Panel();
        wrapper.add(changePinCodePanel);
        southPanel.add(wrapper, BorderLayout.WEST);
        
        // put print button at the top, rightflowed
        // and ok/cancel at buttom, rightflowed
        Panel pnlButtons = new Panel();
        pnlButtons.setLayout(new BorderLayout());
        
        Panel northButtons = new Panel();
        northButtons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        pnlButtons.add(northButtons, BorderLayout.NORTH);

        Panel southButtons = new Panel();
        southButtons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        pnlButtons.add(southButtons, BorderLayout.SOUTH);
        
        if (conditions != null && conditions.trim().length()> 0 && conditions.indexOf("null")!= 0){
	        Button bPrint = new Button("Udskriv...");
	        northButtons.add(bPrint);
	        bPrint.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                printClicked();
	            }
	        });
        }
        southButtons.add(changePinCodePanel.getOkButton());
        southButtons.add(changePinCodePanel.getCancelButton());
        
        southPanel.add(pnlButtons, BorderLayout.EAST);
        
        dialog.add(southPanel, BorderLayout.SOUTH);        
        
        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            	cancelClicked();
            }
        });
        dialog.pack();
    }
	// we try setting focus here:
    public void show() {
    	super.show();
    	changePinCodePanel.takeFocus();
    }

    private void cancelClicked() {
        FileLog.debug("Cancelled");
        cancelled = true;
        dialog.hide();
    }


    public void actionPerformed(ActionEvent e) {
    	changePinCodePanel.enableDialog(false);
        String action = e.getActionCommand();
        if (action.equals("OK")) {
            if (changePinCodePanel.isOk()) {
                FileLog.debug("OK, so get the values");
                response = changePinCodePanel.getAksResponse();
                // and then:
                dialog.hide();
             } else {
                // nothing
                FileLog.debug("NOT OK");
            }
        } else {
        	cancelClicked();
        }
        changePinCodePanel.enableDialog(true);
    }
    private void printClicked() {
        PrintUtil.print(ra, frame, "Aftalevilk�r", fullConditions);
    }
    
 }
