/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DlgConditions.java,v 1.6 2006/08/12 19:02:33 cara Exp $ */

package org.openoces.opensign.certificate.plugin.cdcard.dialogs;

/**
 * This class represents the dialog displayed when the cdcard conditions are being displayed.
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.plugin.cdcard.util.CprChecker;
import org.openoces.opensign.certificate.plugin.cdcard.util.PrintUtil;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.dialogs.AbstractDialog;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

public class DlgConditions extends AbstractDialog {
    boolean cancelled;

    public boolean isCancelled() {
        return cancelled;
    }

    private String fullConditions;
    private String lraProps;
    private Applet applet;
    private TextField cprField;
    private Label msgLabel;
    private String cprNumber;
    private static String ERROR_CPR = "Fejl i CPR nummer - pr�v igen";
    private static String blanks = "                                     ".substring(0, ERROR_CPR.length());
    private Frame frame;
    private RestrictedAction ra;

    public String getCprNumber() {
        return cprNumber;
    }

    public DlgConditions(Applet applet, RestrictedAction ra, Frame owner, AbstractCertificateHandler certificate, String title, String conditions, String lraProps, int rows, int cols) {
        super(ra, owner, title);
        this.frame = owner;
        this.ra = ra;
        this.dialog.setBackground(Color.lightGray);
//        this.dialog.setBackground(Color.white);
        this.applet = applet;
        this.lraProps = lraProps;
        dialog.setLayout(new BorderLayout());
        try {
            FileLog.debug("adding certInfo");
//            FileLog.debug("adding certInfoPanel");
//            Panel certificateInfoPanel = new ConditionsHeaderPanel(certificate, 6, cols);
//            dialog.add(certificateInfoPanel, BorderLayout.NORTH);
            fullConditions = ConditionsHeaderPanel.prependHeader(ConditionsHeaderPanel.ACTIVATE, certificate, conditions);
        } catch (Exception e) {
            FileLog.error("Exception when adding cert info " + e.getMessage());
        }

        TextArea ta = new TextArea(fullConditions, rows, cols, TextArea.SCROLLBARS_VERTICAL_ONLY);
        ta.setBackground(Color.white);
        ta.setEditable(false);
        // wrap in a panel for borders..
        Panel textAreaPanel = new Panel();
        textAreaPanel.add(ta);
        dialog.add(textAreaPanel, BorderLayout.CENTER);

        Panel cprPanel = new Panel(new FlowLayout(FlowLayout.RIGHT));
        cprPanel.add(new Label("CPR:"));
        cprField = new TextField(11);
        cprField.addTextListener(new TextListener() {
            public void textValueChanged(TextEvent e) {
                cprNumber = cprField.getText();
            }
        });

        cprField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                msgLabel.setText(blanks);
            }

            public void focusLost(FocusEvent e) {
                cprNumber = cprField.getText();
                // validate
                if (!CprChecker.valid(cprNumber)){
                    msgLabel.setForeground(Color.red);
                    msgLabel.setText(ERROR_CPR);
                }
            }
        });

        cprPanel.add(cprField);

        Button bCancel = new Button(ResourceManager.getString("DLG_PASSWORD_BUTTON_CANCEL"));
        Button bOk = new Button(ResourceManager.getString("DLG_PASSWORD_BUTTON_OK"));

        cprPanel.add(bOk);
        cprPanel.add(bCancel);

        msgLabel = new Label();
        msgLabel.setFont(new Font("Monospaced", Font.PLAIN, 12));
        msgLabel.setText(blanks);
        cprPanel.add(msgLabel);

        Panel southPanel = new Panel();
        southPanel.setLayout(new BorderLayout());
//        southPanel.add(cprPanel, BorderLayout.NORTH);
        Panel okPromptPanel = new Panel();
        okPromptPanel.add(new Label(ConditionsHeaderPanel.getOkPrompt(ConditionsHeaderPanel.ACTIVATE)));
        southPanel.add(okPromptPanel, BorderLayout.NORTH);
//        southPanel.add(new Label(ConditionsHeaderPanel.getOkPrompt()), BorderLayout.NORTH);

        southPanel.add(cprPanel, BorderLayout.CENTER);

        Panel pnlButtons = new Panel();
        pnlButtons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        Button bPrint = new Button("Udskriv betingelser");
        pnlButtons.add(bPrint);

        southPanel.add(pnlButtons, BorderLayout.SOUTH);

        dialog.add(southPanel, BorderLayout.SOUTH);

        bPrint.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                printClicked();
            }
        });
        bCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelClicked();
            }
        });
        bOk.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okClicked();
            }
        });
        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                cancelClicked();
            }

        });
        dialog.pack();
    }

    protected void okClicked() {
        if (!CprChecker.valid(cprNumber)){
//            cprField.requestFocus();
            msgLabel.setForeground(Color.red);
            msgLabel.setText(ERROR_CPR);
            return;
        }
        cancelled = false;
        dialog.hide();
    }

    private void cancelClicked() {
        cancelled = true;
        dialog.hide();
    }

    private void printClicked() {
        PrintUtil.print(ra, frame, "Aftalevilk�r", fullConditions);
    }
}
