/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DlgImport.java,v 1.6 2006/10/14 21:44:20 cara Exp $ */

package org.openoces.opensign.certificate.plugin.cdcard.dialogs;

/**
 * This class represents the dialog displayed when the import dialog is being displayed.
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.client.applet.SetPollApplet;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.dialogs.AbstractDialog;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Hashtable;
import java.util.Vector;

public class DlgImport extends AbstractDialog implements ActionListener {
    private ImportPanel importPanel;
    private Applet applet;

    public DlgImport(final Applet applet, RestrictedAction ra, Frame owner, String title, String param) {
        super(ra, owner, title);
        this.applet = applet;
        dialog.setLayout(new BorderLayout());
        // todo: think about parsing the param and passing it in the Hashtable
        importPanel = new ImportPanel(this, ra, param, new Hashtable());
        dialog.add(importPanel, BorderLayout.CENTER);
        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                ((SetPollApplet) applet).setAppletState("CANCELLED");
                dialog.hide();
            }
        });
        dialog.pack();
    }

    public void actionPerformed(ActionEvent e) {
        importPanel.enableDialog(false);
        String action = e.getActionCommand();
        if (action.equals("OK")) {
            if (importPanel.isOk()) {
                FileLog.debug("OK, so get the values");
                String capiAlias = importPanel.getCAPIAlias();
                String capiPassword = importPanel.getCAPIPassword();
                String pkcs12password = importPanel.getPKCS12Password();
                // and then:
                dialog.hide();
                // and call JavaScript:
                String[] argNames = {"PKCS12PASSWORD", "CAPIALIAS", "CAPIPASSWORD"};
                String[] args = {pkcs12password, capiAlias, capiPassword};
                ((SetPollApplet) applet).setOutputData(argNames, args);
//                javascript(applet, "doImport", args);
            } else {
                // nothing
                FileLog.debug("NOT OK");
//                ((SetPollApplet) applet).setAppletState("ERROR");// is this ever used?
            }
        } else {
            FileLog.debug("Cancelled");
            ((SetPollApplet) applet).setAppletState("CANCELLED");
            dialog.hide();
        }
        importPanel.enableDialog(true);
    }

    // copied from AbstractApplet - but unconditional JavaScript here..
    protected Object javascript(Applet applet, String function, String args[]) {
        try {
            Vector list = new Vector();
            if (args != null)
                for (int i = 0; i < args.length; i++)
                    list.addElement(args[i]);
            if (FileLog.isDebugEnabled())
                FileLog.debug("callback: function=" + function + " with args:" + list);
            if (list.size() > 0) {
                Object[] objs = new Object[list.size()];
                list.copyInto(objs);
                return netscape.javascript.JSObject.getWindow(applet).call(function, objs);
            } else {
                netscape.javascript.JSObject.getWindow(applet).call(function, new Object[]{
                });
                return null;
            }
            //		} catch (JSException e) {
            //			FileLog.error("error executing javascript",e);
            //			throw new RuntimeException("no javascript available: " + e.getMessage());
        } catch (UnsatisfiedLinkError e) {
            FileLog.error("no javascript available: " + e.getMessage());
            throw new RuntimeException("no javascript available: " + e.getMessage());
        } catch (NoClassDefFoundError e) {
            FileLog.error("no javascript available: " + e.getMessage());
            throw new RuntimeException("no javascript available: " + e.getMessage());
        }
    }
}
