/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: ImportPanel.java,v 1.7 2006/10/15 08:38:20 cara Exp $ */

package org.openoces.opensign.certificate.plugin.cdcard.dialogs;

/**
 * This class represents the panel used in the dialog displayed when the import dialog
 * is being displayed.
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.CertificateInfo;
import org.openoces.opensign.certificate.plugin.cdcard.util.AksClient;
import org.openoces.opensign.certificate.plugin.cdcard.util.AksResponse;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.utils.X500Name;

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.util.StringTokenizer;

public class ImportPanel extends Panel {
    // interface
    public boolean isOk() {
        return validateInput();
    }

    public String getPKCS12Password() {
        return pkcs12password;
    }

    public String getCAPIAlias() {
        return alias;
    }

    public String getCAPIPassword() {
        return passphrase;
    }

    // end interface
    static final String PARAM_BGCOLOR = "BGCOLOR";
    static final String PARAM_FGCOLOR = "FGCOLOR";
    static final String PARAM_FONTFACE = "FONTFACE";
    static final String PARAM_FONTSIZE = "FONTSIZE";
    private String nid = "";
    private String pinCode = "";
    private String alias;
    private String pkcs12password;
    private String passphrase;
    private String passphrase1;

    private String keyStoreType;
    private String retrieveURL;

    private boolean debug;

    private TextArea resultText;
    private TextField pinCodeField;
    private Button cancelButton;
    private Button okButton;
    private boolean displayEventsInResultText = true;

    private RestrictedAction ra;
    private ActionListener actionListener;

    private String firstNameOwns = "Min";

    private void setResultText(String txt) {
        if (displayEventsInResultText) {
            resultText.setText(txt);
        }
    }

    public void enableDialog(boolean doEnable){
        String wait = ResourceManager.getString("PLEASE_WAIT");
        if (doEnable){
            if (wait.equals(resultText.getText())){
                setResultText("");
            } else {
                // we didn't write it so cannot erase it
            }
        } else {
            setResultText(wait);
        }
        cancelButton.setEnabled(doEnable);
        okButton.setEnabled(doEnable);
    }

    private String dummyGetRealPassword(String pin, String nid, RestrictedAction ra) throws Exception {
        int intPin = Integer.parseInt(pin);
        switch (intPin) {
            case AksResponse.OK:
                return "passwordpasswordpassword";
            case AksResponse.FIRST_TIME:
                throw new Exception("Du kan ikke importere f�r kortet har v�ret brugt mindst een gang");
            case AksResponse.FAIL:
                throw new Exception("Forkert PIN-kode - pr�v igen");
            case AksResponse.SYSTEM_ERROR:
                throw new Exception("Der skete en fejl - pr�v igen senere");
            case AksResponse.REVOKED:
                throw new Exception("Kortet er blevet sp�rret");
            case AksResponse.TOO_MANY:
                throw new Exception("Der har v�ret for mange fejlagtige PIN-kode-fors�g");
            default:
                throw new Exception("Uventet fejlsituation (" + intPin + ")");
        }
    }

    private String getRealPassword(String pin, String nid, RestrictedAction ra) throws Exception {
        AksClient client = AksClient.getInstance(ra);
        AksResponse response = client.getPassword(nid, pin, null, "import");
        switch (response.getStatus()) {
            case AksResponse.OK:
                return response.getMsg();
            case AksResponse.FIRST_TIME:
                throw new Exception("Du kan ikke importere f�r kortet har v�ret brugt mindst een gang");
            case AksResponse.FAIL:
                throw new Exception("Forkert PIN-kode - pr�v igen");
            case AksResponse.SYSTEM_ERROR:
                throw new Exception("Der skete en fejl - pr�v igen senere");
            case AksResponse.REVOKED:
                throw new Exception("Kortet er blevet sp�rret");
            case AksResponse.TOO_MANY:
                throw new Exception("Der har v�ret for mange fejlagtige PIN-kode-fors�g");
            default:
                throw new Exception("Uventet fejlsituation (" + response.getStatus() + ")");
        }
    }

    /**
     * Relevant params:
     *
     * @param params
     */
    public ImportPanel(ActionListener actionListener, RestrictedAction ra, String serviceParams, Hashtable params) {
        debug("ImportPanel <cons>(" + ra + "," + params + "" + params + ")");
        StringTokenizer strtok = new StringTokenizer(serviceParams, ",");
        nid = strtok.nextToken();
        if (strtok.hasMoreTokens()){
            // we also have a certificate!
            CertificateInfo certInfo = OS.getCryptoSupport().getCertInfo(strtok.nextToken());
            try {
                X500Name name = new X500Name(certInfo.getSubjectDN().getName());
                String cn = name.getCN();
                String firstName = cn.substring(0, cn.indexOf(" "));
                if (firstName.endsWith("s")){
                    firstNameOwns = firstName + "�";
                } else {
                    firstNameOwns = firstName + "s";
                }
                FileLog.debug("firstName: "+firstName);
            } catch (IndexOutOfBoundsException e){
                FileLog.debug("failed to get name from cert");
            } catch (Exception e){
                FileLog.debug("failed to get name from cert");
            }
        }
        this.ra = ra;
        this.actionListener = actionListener;
        if (params.containsKey(ImportPanel.PARAM_BGCOLOR)) {
            String bgColor = (String) params.get(ImportPanel.PARAM_BGCOLOR);
            if (bgColor.startsWith("#")) {
                this.setBackground(getColorFromString(bgColor));
                debug("Setting bgColor to " + bgColor);
            } else {
                FileLog.warn("Unable to set bgColor to " + bgColor);
            }
        }

        if (params.containsKey(ImportPanel.PARAM_FGCOLOR)) {
            String fgColor = (String) params.get(ImportPanel.PARAM_FGCOLOR);
            if (fgColor.startsWith("#")) {
                this.setForeground(getColorFromString(fgColor));
                debug("Setting fgColor to " + fgColor);
            } else {
                FileLog.warn("Unable to set fgColor to " + fgColor);
            }
        }

        if (params.containsKey(ImportPanel.PARAM_FONTFACE)) {
            String face = (String) params.get(ImportPanel.PARAM_FONTFACE);
            int size = 11;
            if (params.containsKey(ImportPanel.PARAM_FONTSIZE)) {
                size = Integer.parseInt((String) params.get(ImportPanel.PARAM_FONTSIZE));
            }
            this.setFont(new Font(face, Font.PLAIN, size));
            debug("Setting font to " + face + " size " + size);
        } else {
            this.setFont(new Font("SansSerif", Font.PLAIN, 11));
        }
        initialize();
    }

    public static Color getColorFromString(String color) {
        try {
            int red = Integer.parseInt(color.substring(1, 3), 16);
            int green = Integer.parseInt(color.substring(3, 5), 16);
            int blue = Integer.parseInt(color.substring(5, 7), 16);
            return new Color(red, green, blue);
        } catch (Exception e) {
            return Color.gray;
        }
    }

    public void initialize() {
        setLayout(new BorderLayout());

        Panel p = new Panel(new GridLayout(0, 2));

        Component focusComponent = null;

//        p.add(new Label("Referencenummer"));
//        final TextField referenceNumberField = new TextField(nid);
//        referenceNumberField.setColumns(16);
//        referenceNumberField.setEnabled(false);
//        p.add(referenceNumberField);

        if (pinCode == null || !AksClient.isValidPinCode(pinCode)) {
            p.add(new Label("PIN kode"));
            pinCodeField = new TextField(pinCode);
            pinCodeField.setEchoChar('*');
            pinCodeField.setColumns(AksClient.PIN_CODE_LENGTH);
            pinCodeField.addTextListener(new TextListener() {
                public void textValueChanged(TextEvent e) {
                    pinCode = pinCodeField.getText();
                    pkcs12password = null; // not valid anymore
                }
            });
            pinCodeField.addFocusListener(new FocusListener() {
                String text = "Indtast den PIN kode, du har modtaget med posten";

                public void focusGained(FocusEvent e) {
                    setResultText(text);
                }

                public void focusLost(FocusEvent e) {
                    pinCode = pinCodeField.getText();
                    try {
//                        pkcs12password = dummyGetRealPassword(pinCode, nid, ra);
//                        pkcs12password = getRealPassword(pinCode, nid, ra);
                    } catch (Exception ex) {
                        text = ex.getMessage();
                        pinCodeField.requestFocus();
                    }
                }
            });
//            pinCodeField.setFocusable(true);
            p.add(pinCodeField);
            if (focusComponent == null) {
                focusComponent = pinCodeField;
            }
        }

        if (alias == null) {
            p.add(new Label("Indtast navn p� n�glen"));
            final TextField aliasField = new TextField();
            aliasField.setColumns(20);
            alias = firstNameOwns+" digitale signatur";
            aliasField.setText(alias);
            aliasField.addTextListener(new TextListener() {
                public void textValueChanged(TextEvent e) {
                    alias = aliasField.getText();
                }
            });
            aliasField.addFocusListener(new FocusListener() {
                public void focusGained(FocusEvent e) {
                    setResultText("Find selv p� et navn til din digitale signatur eller benyt det foresl�ede. Er I flere, der skal benytte Digital Signatur fra den samme PC, kan det v�re en fordel at angive jeres fornavne her - f.eks. Lises digitale signatur.");
                }

                public void focusLost(FocusEvent e) {
//                    setResultText("");
                }
            });
            p.add(aliasField);
        }

        p.add(new Label("Opret en ny adgangskode"));
        final TextField passphraseField = new TextField();
        passphraseField.setEchoChar('*');
        passphraseField.setColumns(20);
        passphraseField.addTextListener(new TextListener() {
            public void textValueChanged(TextEvent e) {
                passphrase = passphraseField.getText();
                displayEventsInResultText = true;
                setResultText(getAccessCodeStatusText(passphrase, passphrase1));
            }
        });
        passphraseField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                setResultText(getAccessCodeStatusText(passphrase, passphrase1));
            }

            public void focusLost(FocusEvent e) {
//                setResultText("");
            }
        });
//        passphraseField.setFocusable(true);
        p.add(passphraseField);
        if (focusComponent == null) {
            focusComponent = passphraseField;
        }

        p.add(new Label("Gentag den nye adgangskode"));
        final TextField passphrase1Field = new TextField();
        passphrase1Field.setEchoChar('*');
        passphrase1Field.setColumns(20);
        passphrase1Field.addTextListener(new TextListener() {
            public void textValueChanged(TextEvent e) {
                passphrase1 = passphrase1Field.getText();
                displayEventsInResultText = true;
                setResultText(getAccessCodeStatusText(passphrase, passphrase1));
            }
        });
        passphrase1Field.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                setResultText(getAccessCodeStatusText(passphrase, passphrase1));
            }

            public void focusLost(FocusEvent e) {
//                setResultText("");
            }
        });
        p.add(passphrase1Field);

        cancelButton = new Button("Annull�r");
        cancelButton.setActionCommand("CANCEL");
        cancelButton.addActionListener(actionListener);
        okButton = new Button("Indl�s Digital Signatur");
        okButton.setActionCommand("OK");
        okButton.addActionListener(actionListener);
//        button.addActionListener(new ButtonHandler());
        p.add(new Label());
        p.add(new Label());
        p.add(cancelButton);
        p.add(okButton);

        add(p, BorderLayout.CENTER);

        Panel rp = new Panel(new FlowLayout());
        resultText = new TextArea("", 6, 50, TextArea.SCROLLBARS_NONE);
        resultText.setEditable(false);
        resultText.setBackground(this.getBackground());
        rp.add(resultText);
        add(rp, BorderLayout.SOUTH);


//        System.out.println("requesting focus: " + focusComponent.requestFocusInWindow());
        if (nid == null || nid.length() == 0){
            setResultText("Systemfejl: manglende n�gleidentifikator. Kontakt support.");
        }
    }

    private String getAccessCodeStatusText(String p, String p2) {
        if (p == null) {
            p = "";
        }
        if (p2 == null) {
            p2 = "";
        }

        boolean accessCodeIsValid = true;
        StringBuffer b = new StringBuffer("Opret adgangskode til beskyttelse af Digital Signatur.\n");
        if (p.length() < 8) {
            b.append("- Adgangskoden skal indeholde mindst 8 tegn\n");
            accessCodeIsValid = false;
        }

        if (!containsDigit(p)) {
            b.append("- Adgangskoden skal indeholde mindst eet tal\n");
            accessCodeIsValid = false;
        }

        if (p.equals(p.toUpperCase())) {
            b.append("- Adgangskoden skal indeholde mindst eet lille bogstav\n");
            accessCodeIsValid = false;
        }

        if (p.equals(p.toLowerCase())) {
            b.append("- Adgangskoden skal indeholde mindst eet stort bogstav\n");
            accessCodeIsValid = false;
        }

        if (p.equals("") || !p.equals(p2)) {
            b.append("- De to adgangskoder skal v�re ens\n");
            accessCodeIsValid = false;
        }

        if (accessCodeIsValid) {
            return "Du har oprettet og gentaget en gyldig adgangskode.";
        }
        return b.toString();

    }

    private boolean containsDigit(String s) {
        for (int i = 0; i < s.length(); i++) {
            if (Character.isDigit(s.charAt(i))) {
                return true;
            }
        }
        return false;
    }


    private static boolean passphraseValid(String passphrase) {

        if (passphrase != null) {
            // Passphrase skal v�re mindst 8 tegn
            if (passphrase.length() < 8) {
                return false;
            }
            // Passphrase skal indeholde store bogstaver
            if (passphrase.equals(passphrase.toLowerCase())) {
                return false;
            }
            // Passphrase skal indeholde sm� bogstaver
            if (passphrase.equals(passphrase.toUpperCase())) {
                return false;
            }
            // Passphrase skal indeholde et tal
            for (int i = 0; i < passphrase.toCharArray().length; i++) {
                if (passphrase.toCharArray()[i] >= '0' && passphrase.toCharArray()[i] <= '9') {
                    return true;
                }
            }
        } // end null check
        return false;
    }

    public String getKeyStoreType() {
        return keyStoreType;
    }

    public void setKeyStoreType(String keyStoreType) {
        this.keyStoreType = keyStoreType;
    }

    private boolean validateInput() {
        displayEventsInResultText = true;
//            setResultText("");

        if (nid == null || nid.equals("")) {
            setResultText("Indtast referencenummer");
            return false;
        }

        if (pinCode == null || pinCode.equals("")) {
            setResultText("Indtast PIN kode");
            return false;
        }

        if (alias == null || alias.equals("")) {
            setResultText("Indtast alias");
            return false;
        }

        if (passphrase == null) {
            setResultText("Opret adgangskode");
            return false;
        }

        if (!passphraseValid(passphrase)) {
            setResultText("Du skal indtaste et adgangskode med mindst 8 tegn, 1 tal og blandet store/sm� bogstaver");
            return false;
        }
        if (passphrase1 == null) {
            setResultText("Gentag adgangskode");
            return false;
        }
        if (!passphrase.equals(passphrase1)) {
            setResultText("De to adgangskoder skal v�re ens.");
            return false;
        }
        // passed local checks, now check with AKS:
        if (!AksClient.isValidPinCode(pinCode)) {
            setResultText("Indtast gyldig PIN kode. PIN koden skal best� af " + AksClient.PIN_CODE_LENGTH + " cifre.");
            return false;
        }
        if (pkcs12password == null){
            try {
                pkcs12password = getRealPassword(pinCode, nid, ra);
            } catch (Exception ex) {
//                pinCodeField.requestFocus();
                FileLog.info("Didn't get password from AKS: "+ex.getMessage());
                setResultText(ex.getMessage());
                return false;
            }
        }
        return true;
    }


    class NewDot extends Thread {

        boolean stop = false;

        public void stopDot() {
            stop = true;
        }

        public void run() {
            try {
                while (!stop) {
                    resultText.setText(resultText.getText() + ".");
                    Thread.sleep(500);
                }
            } catch (InterruptedException e) {

            }
        }
    }

    private void debug(String message) {
        FileLog.debug(message);
    }

    public String getRetrieveURL() {
        return retrieveURL;
    }

    public void setRetrieveURL(String retrieveURL) {
        this.retrieveURL = retrieveURL;
    }

}

