/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: AksClient.java,v 1.5 2006/08/12 19:02:33 cara Exp $ */

package org.openoces.opensign.certificate.plugin.cdcard.util;

/**
 * This class represents the AKS client
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.crypto.SSLResponse;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;

public class AksClient {
    private static final String AKS_SERVER_STUB = "http://localhost:8080/aksstub/AksStub";
    private static final String AKS_SERVER_DEV = "https://rimfakse.certifikat.dk/aks/AKS";
    private static final String AKS_SERVER_TEST = "https://test.aks.certifikat.dk/aks/AKS";
    private static final String AKS_SERVER_PROD = "https://selvbetjening.certifikat.tdc.dk/aks/AKS";
    private static String AKS_SERVER = AKS_SERVER_TEST;
    public static final int ENV_STUB = 0;
    public static final int ENV_DEV = 1;
    public static final int ENV_TEST = 2;
    public static final int ENV_PROD = 3;
    public static final int PIN_CODE_LENGTH = 4;
    public static void setEnv(int env){
        switch (env){
            case ENV_STUB:
                AKS_SERVER = AKS_SERVER_STUB;
            break;
            case ENV_DEV:
                AKS_SERVER = AKS_SERVER_DEV;
            break;
            case ENV_TEST:
                AKS_SERVER = AKS_SERVER_TEST;
            break;
            case ENV_PROD:
                AKS_SERVER = AKS_SERVER_PROD;
            break;
            default:
            break;
        }
        FileLog.info("Using "+AKS_SERVER);
    }
    public static boolean isValidPinCode(String pinCode) {
        if (pinCode.length() != AksClient.PIN_CODE_LENGTH) {
            return false;
        }
        for (int i = 0; i < pinCode.length(); i++) {
            if (!Character.isDigit(pinCode.charAt(i))) {
                return false;
            }
        }
        return true;
    }


    private static AksClient instance;

    public static AksClient getInstance(RestrictedAction ra) {
        if (instance == null) {
            instance = new AksClient(ra);
        }
        return instance;
    }

    private RestrictedAction ra;

    private AksClient(RestrictedAction ra) {
        this.ra = ra;
    }

    public AksResponse getPassword(String nid, String pin, String service) throws Exception {
        return privGetPassword(nid, pin, null, service, null);
    }

    public AksResponse getPassword(String nid, String pin, String cpr, String service) throws Exception {
        return privGetPassword(nid, pin, cpr, service, null);
    }
    public AksResponse getPasswordAndChangePin(String nid, String oldPin, String newPin, String service) throws Exception {
        return privGetPassword(nid, newPin, null, service, oldPin);
    }
    public AksResponse privGetPassword(String nid, String pin, String cpr, String service, String oldPin) throws Exception {
        FileLog.debug("About to post nid: " + nid + " pin: ***** cpr: **********" + " service: " + service);
        Hashtable ht = new Hashtable();
        if (oldPin != null){
        	ht.put("pin", oldPin);
            ht.put("newpin", pin);
        } else {
        	ht.put("pin", pin);
        }
        ht.put("nid", nid);
        if (service != null) {
            ht.put("service", service);
        }
        if (cpr != null) {
            ht.put("cpr", cpr);
        }
        String response = post(ht);
//            FileLog.debug("response: "+response);
        return new AksResponse(response);
    }

    private String post(Hashtable args) throws IOException {
        SSLResponse response = OS.getCryptoSupport().httpsPost(new URL(AKS_SERVER), args, ra);
//        FileLog.debug("SSLResponse: " + response);
        if (response == null){
            throw new IOException("Error posting to AKS");
        }
        return response.toString();
    }
}
