/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: OcesCertificateHandler.java,v 1.5 2006/10/15 20:18:43 cara Exp $ */

package org.openoces.opensign.certificate.plugin.oces;

import java.io.DataInput;
import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.security.PrivateKey;
import java.util.Properties;

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.DecryptingCertificateHandler;
import org.openoces.opensign.certificate.PwCertInfo;
import org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.bootstrap.security.SunRestrictedAction;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

/**
 * This class represents a certificate stored in a OCES specific html backup
 *
 * @author Preben Valeur  <prv@tdc.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */
public class OcesCertificateHandler extends PwCertInfo implements AbstractCertificateHandler, DecryptingCertificateHandler {
    protected OcesKeyStoreHandler handler;
 	public OcesCertificateHandler(OcesKeyStoreHandler handler, File file) {
		super(file);
		this.handler = handler;
	}
	
    private String getPkcs12FromFile(File file){
        FileLog.debug("About to load PKCS12 from OCES backup file");
        String pkcs12 = null;
      try {
          DataInput dis = new DataInputStream(handler.getRestrictedAction().getFileInputStream(file));
          for (; ;) {
              String line = dis.readLine();
              if (line == null) break;
              if ( (line.indexOf("pkcs12=")==0) || (line.indexOf("CSPUtil.ImportHTMLKey")>0) ){
            	  int lastQuote = line.lastIndexOf("\"");
            	  int secondLastQuote = line.lastIndexOf("\"",lastQuote-1);
            	  pkcs12 = line.substring(secondLastQuote+1, lastQuote);
            	  break;
              }
          }
      } catch (IOException e) {
          FileLog.error("Error reading OCES pkcs12 at " + file.getAbsolutePath());
      }
      return pkcs12;
  }
   protected synchronized Object getKeyStore (char[] password) throws IOException, Exception {
        if (keyStore == null){
            String pkcs12 = getPkcs12FromFile(file);
            keyStore = OS.getCryptoSupport().getKeyStore(pkcs12, password);
        }
        return keyStore;
    }
/*	
    public final byte[] realSign(byte[] toBeSigned, char[] realPassword) {
        FileLog.debug("signing...");
        try {
            String pkcs12 = getPkcs12FromFile(file);
            Object keyStore = OS.getCryptoSupport().getKeyStore(pkcs12, realPassword);
            PrivateKey pk = OS.getCryptoSupport().getPrivateKey(keyStore,realPassword);
            return OS.getCryptoSupport().sign(toBeSigned, pk);
        } catch (Exception e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        }
    }
    */
    public String getUserfriendlyName() {
        return getCertificateID();
    }

    public String getCertificateID() {
        return getFile().toString().replace('\\', '/');
    }

    public String getStoreName() {
        // todo: own Resources? ResourceManager?
        return ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_CERTIFICATESTORE_OCES") + " (" + this.getFile().toString() + ")";
    }

    // if you want to pass extra info about a certificate (like path to cd-card...)
    public Properties getExtraCertificateProperties() {
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }
}