/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: OcesKeyStoreHandler.java,v 1.5 2006/10/15 14:47:12 cara Exp $ */

package org.openoces.opensign.certificate.plugin.oces;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.awt.*;

import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.resources.ResourceManager;

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.plugin.pkcs12.Pkcs12CertificateHandler;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;

/**
 * This class represents an OCES html backup specific keystore
 * <p/>
 * First target is to make this support browsing to and opening an OCES backup html.
 * Later it may also allow a special directory to keep the backups.
 * And later: a new format of the backup - like cd card - where the certificate
 * can be read without opening the pkcs12 with password.
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */
public class OcesKeyStoreHandler extends KeyStoreHandler {
    public OcesKeyStoreHandler() {
    }

    public OcesKeyStoreHandler(RestrictedAction ra) {
        super(ra);
    }

    RestrictedAction getRestrictedAction() {
        return ra;
    }

    public void refreshKeystore() {
        // TODO Auto-generated method stub
    }

    public boolean isSupported(RestrictedAction ra) {
        return true;
    }

    public boolean supportsCopy() {
        // TODO: find out if - but probably not used
        return false;
    }

    public String getName() {
        return "KEYSTORE_OCES";
    }

    public String getLocalizedName() {
        return ResourceManager.getString(getName());
    }

    private FilenameFilter filenameFilter;

    public FilenameFilter getFilenameFilter() {
        if (filenameFilter == null) {
            filenameFilter = new FilenameFilter() {
                public boolean accept(File path, String file) {
                    if (file.toLowerCase().endsWith(".html"))
                        return true;
                    return false;
                }
            };
        }
        return filenameFilter;
    }

    public AbstractCertificateHandler getCertificateFromFile(File f) {
        File parentDir = new File(f.getParent());
        if (!getFilenameFilter().accept(parentDir, f.getName())) {
            return null;
        }
        AbstractCertificateHandler cert = new OcesCertificateHandler(this, f);
        ((OcesCertificateHandler) cert).setCallBackHandler(handler);
        return cert;
    }

    public void addPreferencesComponents(Panel panel, Frame frame) {
        // no preferences yet
    }

    public boolean supportsImport() {
        return false;
    }

    public boolean supportsExport() {
        return false;
    }

    public boolean supportsDelete() {
        return false;
    }

    public void execImportDialog(Frame frame) throws IOException {
    }
}
