package org.openoces.opensign.certificate.plugin.pkcs12;

/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: Pkcs12KeyStoreHandler.java,v 1.8 2006/08/12 19:02:35 cara Exp $ */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;
import org.openoces.opensign.client.applet.CallBackHandler;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.dialogs.DlgKeystoreManagement;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.io.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * This class represents the interface to a pkcs12 keystore 
 *
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Valeur  <prv@tdc.dk>
 */

public class Pkcs12KeyStoreHandler extends KeyStoreHandler{
    private File lastDirectoryName;
    private boolean initialized;
    public Pkcs12KeyStoreHandler(){
    }
    public Pkcs12KeyStoreHandler(CallBackHandler callback, RestrictedAction ra) throws IOException {
        super(callback, ra);
        init();
    }
    private void init() throws IOException {
        if (initialized) return;
        File dir = new File(ra.getUserHome()+File.separator+".oces");
        ra.mkdirs(dir);
        // todo: find out if mkdir/s makes a difference
//		dir.mkdir();
        lastDirectoryName = dir;
        setInitialPath(dir);
        initialized = true;
    }
    RestrictedAction getRestrictedAction(){
        return ra;
    }
    File getLastDirectoryName(){
        return lastDirectoryName;
    }
    public void setRestrictedAction(RestrictedAction ra){
        super.setRestrictedAction(ra);
        try {
            init();
        } catch (IOException e) {
            FileLog.error("Exception in PKCS12KeyStoreHandler: "+e.getMessage());
        }
    }
    public void refreshKeystore() {
        FileLog.debug("refreshing pkcs12 keystore. Using path: "+currentPath);
//		if (!OS.javaplugin) {
//
//			try {
//				netscape.security.PrivilegeManager.enablePrivilege("UniversalTopLevelWindow");
//				netscape.security.PrivilegeManager.enablePrivilege("UniversalFileAccess");
//			} catch (Throwable t1) {
//			};
//			try {
//				com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
//			} catch (Throwable t1) {
//			};
//		}
//		if (!currentPath.isDirectory())
//			throw new FileNotFoundException("not directory");
//		String dirlist[] = currentPath.list(new FilenameFilter() {
        String dirlist[] = ra.list(currentPath, getFilenameFilter());
        certs = new Pkcs12CertificateHandler[dirlist.length];

        for (int i = 0; i < dirlist.length; i++) {
            File f = new File(currentPath, dirlist[i]);
            FileLog.debug("pkcs12 keystore found: " + f);
            certs[i] = new Pkcs12CertificateHandler(this, f);
            ((Pkcs12CertificateHandler)certs[i]).setCallBackHandler(handler);
        }

    }
    private FilenameFilter filenameFilter;
    public FilenameFilter getFilenameFilter(){
        if (filenameFilter == null){
            filenameFilter = new FilenameFilter() {
            public boolean accept(File path, String file) {
                if (file.toLowerCase().endsWith(".p12"))
                    return true;
                if (file.toLowerCase().endsWith(".pfx"))
                    return true;
                if (file.toLowerCase().endsWith(".pkcs12"))
                    return true;
                return false;
            }
            };
        }
        return filenameFilter;
    }
    public boolean isSupported(RestrictedAction ra) {
        return hasPKCS12();
    }

    private static final String PKCS12_KEYSTORE = "PKCS12";

    /**
     * @return boolean true if PKCS12 algorithm is installed
     */
    private boolean hasPKCS12() {
        return true;
    }

    public String getName() {
        return "KEYSTORE_PKCS12";
    }

    public String getLocalizedName() {
        return ResourceManager.getString(getName());
    }

    public boolean supportsCopy(){
        return true;
    }

    public boolean supportsImport() {
        return true;
    }

    public boolean supportsExport() {
        return true;
    }

    public boolean supportsDelete() {
        return true;
    }


    // todo: change - one day
    public boolean requiresPasswordSupplied(){
        return true;
    }

    public AbstractCertificateHandler getCertificateFromFile(File f){
        File parentDir = new File(f.getParent());
        if (!getFilenameFilter().accept(parentDir, f.getName())){
            return null;
        }
        AbstractCertificateHandler cert = new Pkcs12CertificateHandler(this, f);
        ((Pkcs12CertificateHandler)cert).setCallBackHandler(handler);
        return cert;
    }

    public void importPKCS12(Frame parent) throws IOException {
        FileDialog dlg = this.getRestrictedAction().getFileDialog(parent, ResourceManager.getString("RESTORE_KEYFILE"), FileDialog.LOAD);

        dlg.setFilenameFilter(this.getFilenameFilter());
        dlg.setModal(true);
        this.getRestrictedAction().showFileDialog(dlg);

        if (dlg.getFile() == null) {
            FileLog.debug("dialog cancel");
            return;
        }

        File src = new File(dlg.getDirectory() + OS.getFileSeperator() + dlg.getFile());
        File dest = new File(this.getPath().toString() + OS.getFileSeperator() + dlg.getFile());

        prepareKeyDir();

        if (dest != null) {
            copy(src, dest);
        }

        this.refreshKeystore();
    }

    public void addPreferencesComponents(Panel panel, final Frame frame) {
        Button btnManage;
        btnManage = new Button(ResourceManager.getString("PREFERENCES"));
        final KeyStoreHandler hdl = this;

        btnManage.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DlgKeystoreManagement dlg = new DlgKeystoreManagement(ra,frame,ResourceManager.getString("PREFERENCES"),hdl);
                dlg.show();
            }
        });
        panel.add(btnManage);
    }

    public void execImportDialog(Frame parent) throws IOException {
            FileDialog dlg = this.getRestrictedAction().getFileDialog(parent, ResourceManager.getString("RESTORE_KEYFILE"), FileDialog.LOAD);

            dlg.setFilenameFilter(this.getFilenameFilter());
            dlg.setModal(true);
            this.getRestrictedAction().showFileDialog(dlg);

            if (dlg.getFile() == null) {
                FileLog.debug("dialog cancel");
                return;
            }

            File src = new File(dlg.getDirectory() + OS.getFileSeperator() + dlg.getFile());
            File dest = new File(this.getPath().toString() + OS.getFileSeperator() + dlg.getFile());

            prepareKeyDir();

            if (dest != null) {
                copy(src, dest);
            }
        }


    protected void copy(File srcFile, File destFile) throws IOException {
        BufferedInputStream in = new BufferedInputStream(this.getRestrictedAction().getFileInputStream(srcFile));
        BufferedOutputStream out = new BufferedOutputStream(this.getRestrictedAction().getFileOutputStream(destFile));
        int c;

        while ((c = in.read()) != -1) {
            out.write(c);
        }

        in.close();
        out.close();
    }

    public void prepareKeyDir() throws IOException {
        RestrictedAction ra = this.getRestrictedAction();

        File dir = this.getPath();
        if ( !ra.exists(dir)) {
            ra.mkdirs(dir);
        }

        if ( !ra.exists(dir) ) {
            FileLog.warn("Unable to construct directory: "+dir);
            throw new IOException(ResourceManager.getString("ERR_CREATE_KEY_DIR"));
        }

        if ( !ra.isDirectory(dir)) {
            FileLog.warn("Unable to construct directory: "+dir+". The directory name is used as a filename.");
            throw new IOException(ResourceManager.getString("ERR_CREATE_KEY_DIR"));
        }

    }

    public void execExportDialog(Frame frame) throws IOException {

    }
}
