/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: SunCryptoSupport.java,v 1.5 2006/08/12 19:02:35 cara Exp $ */

package org.openoces.opensign.certificate.support.suncrypto;

import org.openoces.opensign.certificate.CertificateInfo;
import org.openoces.opensign.certificate.StringPrincipal;
import org.openoces.opensign.certificate.x509.KeyUsage;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.crypto.CryptoSupport;
import org.openoces.opensign.crypto.SSLResponse;
import org.openoces.opensign.exceptions.UserCancel;
import org.openoces.opensign.utils.Base64;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.utils.X500Name;
import sun.security.x509.X509CertImpl;

import javax.net.ssl.*;
import java.io.*;
import java.math.BigInteger;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.*;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

/**
 * This class represents the interface to high-level crypto functionality on the Sun jre 1.4-1.5
 * platform
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */
public class SunCryptoSupport implements CryptoSupport {
    private static CryptoSupport instance;
    private static final String[] trustedRootCertificates = new String[]{
        //Globalsign root
            "MIIDdTCCAl2gAwIBAgILAgAAAAAA1ni3lAUwDQYJKoZIhvcNAQEEBQAwVzELMAkG" +
            "A1UEBhMCQkUxGTAXBgNVBAoTEEdsb2JhbFNpZ24gbnYtc2ExEDAOBgNVBAsTB1Jv" +
            "b3QgQ0ExGzAZBgNVBAMTEkdsb2JhbFNpZ24gUm9vdCBDQTAeFw05ODA5MDExMjAw" +
            "MDBaFw0xNDAxMjgxMjAwMDBaMFcxCzAJBgNVBAYTAkJFMRkwFwYDVQQKExBHbG9i" +
            "YWxTaWduIG52LXNhMRAwDgYDVQQLEwdSb290IENBMRswGQYDVQQDExJHbG9iYWxT" +
            "aWduIFJvb3QgQ0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDaDuaZ" +
            "jc6j40+Kfvvxi4Mla+pIH/EqsLmVEQS98GPR4mdmzxzdzxtIK+6NiY6arymAZavp" +
            "xy0Sy6scTHAHoT0KMM0VjU/43dSMUBUc71DuxC73/OlS8pF94G3VNTCOXkNz8kHp" +
            "1Wrjsok6Vjk4bwY8iGlbKk3Fp1S4bInMm/k8yuX9ifUSPJJ4ltbcdG6TRGHRjcdG" +
            "snUOhugZitVtbNV4FpWi6cgKOOvyJBNPc1STE4U6G7weNLWLBYy5d4ux2x8gkasJ" +
            "U26Qzns3dLlwR5EiUWMWea6xrkEmCMgZK9FGqkjWZCrXgzT/LCrBbBlDSgeF59N8" +
            "9iFo7+ryUp9/k5DPAgMBAAGjQjBAMA4GA1UdDwEB/wQEAwIABjAdBgNVHQ4EFgQU" +
            "YHtmGkUNl8qJUC99BM00qP/8/UswDwYDVR0TAQH/BAUwAwEB/zANBgkqhkiG9w0B" +
            "AQQFAAOCAQEArqqf/LfSyx9fOSkoGJ40yWxPbxrwZKJwSk8ThptgKJ7ogUmYfQq7" +
            "5bCdPTbbjwVR/wkxKh/diXeeDy5slQTthsu0AD+EAk2AaioteAuubyuig0SDH81Q" +
            "gkwkr733pbTIWg/050deSY43lv6aiAU62cDbKYfmGZZHpzqmjIs8d/5GY6dT2iHR" +
            "rH5Jokvmw2dZL7OKDrssvamqQnw1wdh/1acxOk5jQzmvCLBhNIzTmKlDNPYPhyk7" +
            "ncJWWJh3w/cbrPad+D6qp1RF8PX51TFl/mtYnHGzHtdS6jIX/EBgHcl5JLL2bP2o" +
            "Zg6C3ZjL2sJETy6ge/L3ayx2EYRGinij4w==",
        // Thawte Server CA
            "MIIDEzCCAnygAwIBAgIBATANBgkqhkiG9w0BAQQFADCBxDELMAkGA1UEBhMCWkEx" +
            "FTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3duMR0wGwYD" +
            "VQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlmaWNhdGlv" +
            "biBTZXJ2aWNlcyBEaXZpc2lvbjEZMBcGA1UEAxMQVGhhd3RlIFNlcnZlciBDQTEm" +
            "MCQGCSqGSIb3DQEJARYXc2VydmVyLWNlcnRzQHRoYXd0ZS5jb20wHhcNOTYwODAx" +
            "MDAwMDAwWhcNMjAxMjMxMjM1OTU5WjCBxDELMAkGA1UEBhMCWkExFTATBgNVBAgT" +
            "DFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3duMR0wGwYDVQQKExRUaGF3" +
            "dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlmaWNhdGlvbiBTZXJ2aWNl" +
            "cyBEaXZpc2lvbjEZMBcGA1UEAxMQVGhhd3RlIFNlcnZlciBDQTEmMCQGCSqGSIb3" +
            "DQEJARYXc2VydmVyLWNlcnRzQHRoYXd0ZS5jb20wgZ8wDQYJKoZIhvcNAQEBBQAD" +
            "gY0AMIGJAoGBANOkUG7I/1Zr5s9dtuoMaHVHoqrC2oQl/Kj0R1HahbUgdJSGHg91" +
            "yekIYfUGbTBuFRkC6VLAYttNmZ7iagxEOM3+vuNkCXDF/rFrKbYvScg71CcEJRCX" +
            "L+eQbcAoQpnXTEPew/UhbVSfXcNY4cDk2VuwuNy0e982OsK1ZiIS1ocNAgMBAAGj" +
            "EzARMA8GA1UdEwEB/wQFMAMBAf8wDQYJKoZIhvcNAQEEBQADgYEAB/pMaVz7lcxG" +
            "7oWDTSEwjsrZqG9JGubaUeNgcGyEYRGhGshIPllDfU+VPaGLtwtimHp1it2ITk6e" +
            "QNuozDJ0uW8NxuOzRAvZim+aKZuZGCg70eNAKJpaPNW15yAbi8qkq43pUdniTCxZ" +
            "qdq5snUb9kLy78fyGPmJvKP/iiMucEc="
    };

    public static CryptoSupport getInstance() {
        if (instance == null) {
            instance = new org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport();
        }
        return instance;
    }

    // todo: make private - singleton...but for now it is instantiated by BootApplet...
//    private SunCryptoSupport(){}
    public SunCryptoSupport() {
    }

    public byte[] sign(byte[] signText, PrivateKey privateKey) {
        try {
            return JCETools.sign(getSignature(), signText, privateKey);
        } catch (NoSuchProviderException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        } catch (SignatureException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        } catch (InvalidKeyException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        }
    }
    public PrivateKey getPrivateKey(Object keyStore, char[] password) throws Exception {
        String alias = JCETools.getKeyAlias((KeyStore)keyStore);
        return JCETools.getPrivateKey((KeyStore)keyStore, alias, password);
    }

    protected static class SunCertificateInfo implements CertificateInfo {
        private X509Certificate cert;

        SunCertificateInfo(X509Certificate cert) {
            this.cert = cert;
        }

        public boolean isInfoAvailable() {
            return true;
        }

        public String getUserfriendlyName() {
            X500Name xname = new X500Name(cert.getSubjectDN().getName());
            return xname.getCN();
        }

        public java.security.Principal getSubjectDN() throws Exception {
            return new StringPrincipal(cert.getSubjectDN().getName());
        }

        public Principal getIssuerDN() throws Exception {
            return new StringPrincipal(cert.getIssuerDN().getName());
        }

        public BigInteger getSerialNumber() throws Exception {
            return cert.getSerialNumber();
        }

        public Date getNotBefore() throws Exception {
            return cert.getNotBefore();
        }

        public Date getNotAfter() throws Exception {
            return cert.getNotAfter();
        }

        public int getVersion() throws Exception {
            return cert.getVersion();
        }

        public String getKeyUsage() throws Exception {
            return JCETools.getUsage(cert);
        }

        public boolean canSign() throws IOException, UserCancel {
            return JCETools.canSign(cert);
        }

        public KeyUsage getIntendedKeyUsage() throws Exception, IOException {
            return JCETools.getIntendedKeyUsage(cert);
        }

        public byte [] getCertificate() throws Exception {
            return cert.getEncoded();
        }
    }

    private static X509Certificate getCertificate(String b64clientCert) throws CertificateException {
        byte[] decoded = Base64.decode(b64clientCert.getBytes());
        ByteArrayInputStream bis = new ByteArrayInputStream(decoded);
        CertificateFactory cf = CertificateFactory.getInstance("X.509");
        return (X509Certificate) cf.generateCertificate(bis);
    }

    public byte [][] getCertificateChain(Object keyStore) throws Exception {
        Certificate[] certs = JCETools.getSigningCertificateChain((KeyStore)keyStore);
        byte[][] bchain = new byte[certs.length][0];
        for (int i = 0; i < certs.length; i++) {
            bchain[i] = certs[i].getEncoded();
        }
        return bchain;
    }

    public CertificateInfo getCertInfo(Object keyStore) {
        try {
            Certificate[] certChain = JCETools.getSigningCertificateChain((KeyStore)keyStore);
            return new org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport.SunCertificateInfo((X509Certificate)certChain[0]);
        } catch (KeyStoreException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        } catch (Exception e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        }
    }
    public CertificateInfo getCertInfo(String b64clientCert) {
        try {
            return new org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport.SunCertificateInfo(getCertificate(b64clientCert));
        } catch (CertificateException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        }
    }

    protected Signature getSignature() throws NoSuchAlgorithmException {
        return Signature.getInstance("SHA1withRSA");
    }

    public boolean verify(String modulus, String publicExp, byte[] signedValue, byte[] signatureToVerify) {
        RSAPublicKeySpec publicKeySpec = new RSAPublicKeySpec(new BigInteger(1, hexDecode(modulus)), new BigInteger(1, hexDecode(publicExp)));
        try {
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            RSAPublicKey publicKey = (RSAPublicKey) keyFactory.generatePublic(publicKeySpec);
            Signature s  = getSignature();
            s.initVerify(publicKey);
            s.update(signedValue);
            return s.verify(signatureToVerify);
        } catch (InvalidKeySpecException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (InvalidKeyException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        } catch (SignatureException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        FileLog.info("Failed with SUN crypto");
        return false;
    }

    // todo: move to common util-class - also used by IaikTools
    protected byte[] hexDecode(String hexString) {
        StringTokenizer st = new StringTokenizer(hexString, ":");
        byte[] bytes = new byte[st.countTokens()];
        int i = 0;
        while (st.hasMoreTokens()) {
            bytes[i] = (byte) Integer.parseInt(st.nextToken(), 16);
            i++;
        }
        return bytes;
    }

    public byte[] sha1(byte[] bytes) throws Exception {
        return JCETools.sha1(bytes);
    }

    public SSLResponse httpsPost(URL url, Hashtable params, RestrictedAction ra) {
        String USER_AGENT = "OpenSign v1.x.y (http://www.openoces.org)";


        if (!"https".equals(url.getProtocol())) {
            FileLog.warn("Url does not indicate the use of https. Indicate protocol: " + url.getProtocol());
            return null;
        }
        SSLSocketFactory originalSSLSocketFactory = null;
        try {
            ra.addSSLProvider();
            SSLContext context = SSLContext.getInstance("TLS");
            context.init(null, //No Key manager needed
                    new TrustManager[]{new SunTrustManager()},
                    new java.security.SecureRandom());
            FileLog.debug("Ok, got the context, now doing the setDefaultSSLFactory thing!");
            originalSSLSocketFactory = HttpsURLConnection.getDefaultSSLSocketFactory();
            ra.setDefaultSSLSocketFactory(context.getSocketFactory());
//            HttpsURLConnection.setDefaultSSLSocketFactory(context.getSocketFactory());
            URLConnection con = null;
            con = url.openConnection();
            con.setRequestProperty("User-Agent", USER_AGENT);
            con.setDoOutput(true);
            con.setDoInput(true);
            OutputStream os = ra.getOutputStream(con);
            if (os == null) {
                FileLog.warn("Did not get an output stream");
            }
            PrintStream ps = new PrintStream(os);
            ps.write(generateContents(params));
            ps.flush();
            con.getOutputStream().close();

            /* read response */
            BufferedReader br = new BufferedReader(new InputStreamReader(con.getInputStream()));
            String line;
            StringBuffer response = new StringBuffer();
            while ((line = br.readLine()) != null) {
                response.append(line + "\n");
//                FileLog.debug("received line: " + line);
            }
            return new SSLResponse(response);

        } catch (IOException e) {
            FileLog.warn("An IO exception occurred");
            e.printStackTrace();
            return null;
        } catch (NoSuchAlgorithmException e) {
            FileLog.warn("NoSuchAlgorithmn exception occurred");
            e.printStackTrace();
            return null;
        } catch (KeyManagementException e) {
            FileLog.warn("KeyManagement exception occurred");
            e.printStackTrace();
            return null;
        } finally {
            // reset to the original value
            if (originalSSLSocketFactory != null){
                ra.setDefaultSSLSocketFactory(originalSSLSocketFactory);
            }
        }

    }

    private byte[] generateContents(Hashtable params) {
        StringBuffer sb = new StringBuffer();
        boolean first = true;

        for (Enumeration keys = params.keys(); keys.hasMoreElements();) {
            if (!first) sb.append("&");
            String key = (String) keys.nextElement();
            String value = (String) params.get(key);
            sb.append(URLEncoder.encode(key));

            sb.append("=");
            sb.append(URLEncoder.encode(value));
            first = false;
        }
        return sb.toString().getBytes();
    }

    // could be in JCETools or here:
    private static final String KEYSTORE_TYPE = "PKCS12";

    public Object getKeyStore(File file, char [] pw) throws Exception {
        return JCETools.getKeyStore(file, pw);
    }

    public Object getKeyStore(String pkcs12string, char[] password) throws KeyStoreException, NoSuchAlgorithmException, IOException, CertificateException {
        byte[] decoded = Base64.decode(pkcs12string.getBytes("UTF8"));
        ByteArrayInputStream bis = new ByteArrayInputStream(decoded);
        KeyStore store = KeyStore.getInstance(KEYSTORE_TYPE);
        try {
            store.load(bis, password);
        } finally {
            try {
                bis.close();
            } catch (IOException e) {
            }
        }
        return store;
    }

    class SunTrustManager implements X509TrustManager {

        private Certificate[] trustedRoots = null;

        public SunTrustManager() {
            trustedRoots = new Certificate[trustedRootCertificates.length];
            for (int i = 0; i < trustedRoots.length; i++) {
                byte[] decoded = Base64.decode(trustedRootCertificates[i].getBytes());
                try {
                    CertificateFactory cf = CertificateFactory.getInstance("X.509");
                    InputStream bis = new ByteArrayInputStream(decoded);
                    trustedRoots[i] = cf.generateCertificate(bis);
                } catch (CertificateException e) {
                    FileLog.error("CertificateException in SunTrustManager: " + e.getMessage());
                }
            }
        }

        public boolean isClientTrusted(X509Certificate[] cert) {
            return true;
        }

        public boolean isServerTrusted(X509Certificate[] certChain) {
            boolean trusted = false;
            FileLog.debug("Checking trust for certChain");
            for (int i = 0; i < certChain.length; i++) {
                X509Certificate x509Certificate = certChain[i];
                FileLog.debug("Certchain element " + i + " : " + x509Certificate.getSubjectDN().getName() + " issued by : " + x509Certificate.getIssuerDN());
            }

            //Check chain consistency
            int verifyCert = 0;
            if (certChain.length > 1) {
                for (int i = 1; i < certChain.length; i++) {
                    try {
                        certChain[i - 1].verify(certChain[i].getPublicKey());
                    } catch (Exception e) {
                        return false;
                    }
                }
                verifyCert = certChain.length - 1;
                if (certChain[verifyCert].getIssuerDN().equals(certChain[verifyCert].getSubjectDN())) {
                    verifyCert--;
                }
            }


            for (int i = 0; i < trustedRoots.length; i++) {
                Certificate trustedRoot = trustedRoots[i];
                try {
                    certChain[verifyCert].verify(trustedRoot.getPublicKey());
                    trusted = true;
                    break;
                } catch (Exception e) {
                    //Not signed by trustedRoot
                }
            }
            FileLog.debug("isServerTrusted returning "+trusted);
            return trusted;

        }

        public X509Certificate[] getAcceptedIssuers() {
            X509Certificate[] trusted = new X509Certificate[trustedRoots.length];
            try {
                for (int i = 0; i < trustedRoots.length; i++) {
                    Certificate trustedRoot = trustedRoots[i];
                    trusted[i] = new X509CertImpl(trustedRoot.getEncoded());
                }
                return trusted;
            } catch (CertificateEncodingException e) {
                e.printStackTrace();
            } catch (CertificateException e) {
                e.printStackTrace();
            }
            return null;
        }

        public void checkClientTrusted(X509Certificate[] x509Certificates, String s) throws CertificateException {

        }

        public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
            if (authType == null || "".equals(authType)) {
                throw new IllegalArgumentException("Null or empty authType string supplied");
            }
            if (chain == null || chain.length == 0) {
                throw new IllegalArgumentException("Empty certChain supplied");
            }
            boolean isTrusted = isServerTrusted(chain);
            if (!isTrusted) {
                throw new CertificateException("Certificate chain is not trusted");
            }
        }
    }

}
