/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: KeyStoreHandler.java,v 1.10 2006/04/02 08:52:35 cara Exp $ */

package org.openoces.opensign.certificate.x509;

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.client.applet.CallBackHandler;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;

import java.applet.Applet;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.awt.*;

/**
 * This class represents the interface to keystores
 *
 * @author Jens Bo Friis  <jbf@it-practice.dk>
 * @author Preben Valeur  <prv@tdc.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk> 
 */
public abstract class KeyStoreHandler {

    protected AbstractCertificateHandler[] certs = null;
    protected File currentPath;
    protected File initialPath;
    protected CallBackHandler handler;
    protected RestrictedAction ra;

    // this happens to be a RestrictedActor too - probably
    public KeyStoreHandler() {

    }
	public KeyStoreHandler(RestrictedAction ra){
		this.ra = ra;
	}
    public void setCallBackHandler(CallBackHandler handler) {
        this.handler = handler;
    }

    public CallBackHandler getCallBackHandler() {
        return handler;
    }

    public void setRestrictedAction(RestrictedAction ra) {
        this.ra = ra;
        init(ra);
    }

    // override if something should happen when ra is set
    protected void init(RestrictedAction ra) {
    }
    /**
     * Microsoft keystore
     *
     * @throws IOException
     */
    public KeyStoreHandler(CallBackHandler handler, RestrictedAction ra) throws IOException {
        this.handler = handler;
        this.ra = ra;
    }

    /**
     * Directory of PKCS12 files
     *
     * @param path
     * @throws IOException
     */
    public void setInitialPath(File path) throws IOException {
        //		this.passwordHandler = passwordHandler;
        this.currentPath = path;
        this.initialPath = path;
    }

    public abstract void refreshKeystore();

    /**
     * @return if this keyStore is supported in this environment (JVM, OS...)
     */
    public abstract boolean isSupported(RestrictedAction ra);

    public abstract boolean supportsCopy();

    public abstract boolean supportsImport();
    public abstract boolean supportsExport();
    public abstract boolean supportsDelete();

    public abstract String getName();

    public abstract String getLocalizedName();

    // enable the password field in the applet - we don't like this pt
    public boolean requiresPasswordSupplied() {
        return false;
    }

    public AbstractCertificateHandler[] getCertificates() {
        return certs;
    }

    public AbstractCertificateHandler getCertificateFromFile(File file) {
        return null;
    }

    public FilenameFilter getFilenameFilter() {
        return null;
    }

    public boolean isEmpty() {
        // todo: refresh or?
        refreshKeystore();
        return (certs == null || certs.length == 0);
    }

    public AbstractCertificateHandler getCertificate(String certificateID) {
        if (certificateID == null)
            return null;
        for (int i = 0; i < certs.length; i++) {
            if (certificateID.equals(certs[i].getUserfriendlyName()))
                return certs[i];
        }
        return null;
    }

    public File getPath() {
        return currentPath;
    }

    public void setPath(File path) {
        if (path != null) {
            // change, temporarily, to specified path
            this.currentPath = path;
        } else {
            // if null'ed, return to initial default path
            this.currentPath = initialPath;
        }
        refreshKeystore();
    }

    // override if you want to provide a service
    public void service(Applet applet, String serviceName, String serviceParam) {
    }

    public abstract void addPreferencesComponents(Panel panel, Frame frame);

    public abstract void execImportDialog(Frame frame) throws IOException; 
}