/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: KeyUsage.java,v 1.4 2006/04/02 08:52:35 cara Exp $ */

package org.openoces.opensign.certificate.x509;

/**
 * This class represents the keyusage element as specified by the X.509 format
 *
 * @author Jens Bo Friis  <jbf@it-practice.dk>
 */
public class KeyUsage {
    private boolean digitalSignature;
    private boolean nonRepudiation;
    private boolean keyEncipherment;
    private boolean dataEncipherment;
    private boolean keyAgreement;
    private boolean keyCertSign;
    private boolean keyCRLSign;

    public KeyUsage(boolean digitalSignature,
                    boolean nonRepudiation,
                    boolean keyEncipherment,
                    boolean dataEncipherment,
                    boolean keyAgreement,
                    boolean keyCertSign,
                    boolean keyCRLSign) {
        this.digitalSignature = digitalSignature;
        this.nonRepudiation = nonRepudiation;
        this.keyEncipherment = keyEncipherment;
        this.dataEncipherment = dataEncipherment;
        this.keyAgreement = keyAgreement;
        this.keyCertSign = keyCertSign;
        this.keyCRLSign = keyCRLSign;
    }

    public boolean includesDigitalSignature() {
        return digitalSignature;
    }

    public boolean includesNonRepudiation() {
        return nonRepudiation;
    }

    public boolean includesKeyEncipherment() {
        return keyEncipherment;
    }

    public boolean includesDataEncipherment() {
        return dataEncipherment;
    }

    public boolean includesKeyAgreement() {
        return keyAgreement;
    }

    public boolean includesKeyCertSign() {
        return keyCertSign;
    }

    public boolean includesKeyCRLSign() {
        return keyCRLSign;
    }
}
