/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: AbstractApplet.java,v 1.42 2006/10/15 16:08:14 cara Exp $ */

package org.openoces.opensign.client.applet;

/**
 * This class implements the general applet functionality.
 *
 * @author Jens Bo Friis  <jbf@it-practice.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.Copyright;
import org.openoces.opensign.appletsupport.AppletSupport;
import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.plugin.oces.OcesKeyStoreHandler;
import org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;
import org.openoces.opensign.client.applet.bootstrap.interfaces.Plugaware;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedActor;
import org.openoces.opensign.client.applet.bootstrap.security.AbstractRestrictedAction;
import org.openoces.opensign.client.applet.dialogs.DlgAbout;
import org.openoces.opensign.client.applet.dialogs.DlgInformation;
import org.openoces.opensign.client.applet.dialogs.DlgPassword;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.crypto.CryptoSupport;
import org.openoces.opensign.exceptions.UserCancel;
import org.openoces.opensign.utils.Base64;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.utils.Version;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.*;

public abstract class AbstractApplet extends Applet implements Copyright, CallBackHandler, RestrictedActor, Plugaware, CertificateListView {
    public AppletArguments arguments;

    protected Button bCancel;
    protected Button bFunctions;
    protected Button bOk;

    protected TextField tfPassword;
    protected TextField tfSocialSecurityNumber; // social security number, drivers license, passport number
    protected TextField tfOptionalEntry; // customer specific entry. customer id/number, agreement number etc.
    protected Label lCertList;
    protected Label lPassword;
    protected Label lSocialSecurityNumber;
    protected Label lOptionalEntry;

    protected Label lLogonTo;
    protected Label lHost;

    protected RestrictedAction ra;

    private Properties signProperties;

    private String preSelectedCertificate;// name of certificate to select initially

    protected boolean bootstrapped = true;

    static String[] validOptionalIds = {"CUSTOMERID", "USERID"};
    static final int DEFAULT_MAX_CERT_LIST_WIDTH = 28;

    protected ParamReader paramReader;

    protected abstract ParamReader createParamReader();


    private int maxCertListWidth = DEFAULT_MAX_CERT_LIST_WIDTH; // or more dynamic if we like - like depending on the actual applet size maybe if possible (width/height)

    protected void setNoBootstrap() {
        bootstrapped = false;
        FileLog.debug("We are running without bootstrap applet");
        if (org.openoces.opensign.client.applet.bootstrap.OS.isMsJVM()) {
            this.setRestrictedAction(new org.openoces.opensign.client.applet.bootstrap.security.MsRestrictedAction());
            this.addPlugin(new org.openoces.opensign.certificate.support.mscrypto.MsCryptoSupport());
            OS.setCryptoSupport((new org.openoces.opensign.certificate.support.mscrypto.MsCryptoSupport()));
        } else {
            this.addPlugin(new org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport());
            OS.setCryptoSupport((new SunCryptoSupport()));
        }
        getRestrictedAction().setBaseUrl(org.openoces.opensign.client.applet.bootstrap.Utils.generateBaseUrl(getCodeBase(),getParameter("ZIP_BASE_URL")));
        this.addPlugin(new org.openoces.opensign.certificate.plugin.capi.CapiKeyStoreHandler());
        FileLog.debug("Skipping CdCard for now when single jar/cab");// todo: fix
        this.addPlugin(new org.openoces.opensign.certificate.plugin.cdcard.CdCardKeyStoreHandler());
        if (OS.isMsJVM()) {
            FileLog.debug("Skipping PKCS12 for now when MS JVM"); //todo: fix
        } else {
            this.addPlugin(new org.openoces.opensign.certificate.plugin.pkcs12.Pkcs12KeyStoreHandler());
        }
        this.addPlugin(new OcesKeyStoreHandler());
        // if it is OpenMail...
//        this.addPlugin(new SmimeUtil());
    }

    protected CertificateListController listCtrl = null;
    private AbstractCertificateHandler[] certList = null;
    protected Choice chCertList = null;
    protected boolean emptyCertificateList = true;
    private boolean certsLoaded = false; // true when certs fully loaded
    private boolean certsLoading = false; // true when the certs are in the process of loading

    public void setCertificateList(AbstractCertificateHandler[] list) {
        FileLog.debug("Timing: setCertificateList with "+list.length +" certs: "+ new Date());
        certList = list;
        chCertList.removeAll();
        if (certList == null || certList.length == 0) {
            emptyCertificateList = true;
            chCertList.add(ResourceManager.getString("NO_CERTIFICATES"));
        } else {
            emptyCertificateList = false;
            for (int i = 0; i < certList.length; i++) {
                chCertList.add(listCtrl.getCertificateDisplayName(certList[i], maxCertListWidth));
            }
        }
        if (preSelectedCertificate != null){
            chCertList.select(preSelectedCertificate.replace('\\', '/'));
        }
        refreshGUI();
    }

    public void selectCertificate(AbstractCertificateHandler cert) {
        int index = 0;
        for (int i = 0; i < certList.length; i++) {
            if (certList[i].equals(cert)) {
                index = i;
            }
        }
        chCertList.select(index);
    }

    public AbstractCertificateHandler getSelectedCertificate() {
        return certList[chCertList.getSelectedIndex()];
    }

    private void about() {
        try {
            DlgAbout dlg = new DlgAbout(ra, getFrame());
            dlg.show();
        } catch (Exception ex) {
            FileLog.error("error occurred during show of about dialog", ex);
        }
    }

    /**
     * @see Applet#destroy()
     */
    public void destroy() {
        FileLog.debug("AbstractApplet.destroy() invoked " + this.getClass().getClassLoader());
        try {
            super.destroy();
        } catch (Throwable t) {
            FileLog.error("unexpected error occured", t);

        }
    }

    public abstract void enableDialog(boolean enable);

    public abstract String getAppletInfo();

    public Frame getFrame() {
        try {
            Container c = this;
            FileLog.debug(c.toString());
            try {
                while (c != null && !(c instanceof Frame))
                    c = c.getParent();
                if (c != null)
                    return (Frame) c;
            } catch (Throwable t) {
                FileLog.error("Unable to find browser frame, creating new one");
            }
            return new Frame();
        } catch (Throwable t) {
            FileLog.error("unexpected error occured", t);
            return null;
        }

    }

    public Properties getSignProperties() {
        return signProperties;
    }

    public void alert(String s) {
        try {
            javascript("alert", new String[]{s});
        } catch (RuntimeException e) {
            FileLog.error("ALERT " + s);
        }
    }

    public boolean confirm(String question) {
        try {
            Boolean bool = (Boolean) javascript("confirm", new String[]{question});
            return bool.booleanValue();
        } catch (Exception e) {
            FileLog.error("CONFIRM " + question);
            return false;
        }
    }

    public String prompt(String question) {
        try {
            String answer = (String) javascript("confirm", new String[]{question});
            return answer;
        } catch (Exception e) {
            FileLog.error("PROMPT " + question);
            return null;
        }
    }

    private class CertificateLoader implements Runnable {
        private CertificateListView view;

        public CertificateLoader(CertificateListView view) {
            this.view = view;
        }

        public void run() {
            for (int i = 0; i < keyStoreHandlers.size(); i++) {
                KeyStoreHandler keyStoreHandler = (KeyStoreHandler) keyStoreHandlers.elementAt(i);
                listCtrl.addKeyStoreHandler(keyStoreHandler);
                FileLog.debug("added keystoreHandler to certificate view controller: "+keyStoreHandler.getName());
            }
            listCtrl.init(view, bFunctions, ra);
            whenCertificatesLoaded();
        }
    }

    private void clearWaitMsg(){
        if (emptyCertificateList && certsLoaded && userMessageLabel != null){
            // todo: find a nicer way - Hack - to get rid of the "vent venligst.." msg...
            FileLog.debug("clearing vent venligst!");
            userMessageLabel.setText("");
        }
    }

    private void whenCertificatesLoaded() {
        certsLoading = false;
        certsLoaded = true;
        FileLog.debug("whenCertificatesLoaded certsLoaded:"+certsLoaded);
        bFunctions.setEnabled(certsLoaded);
        clearWaitMsg();
    }

    private Thread certLoaderThread;

    protected void setCertificateListController() {
        String certListWidth = paramReader.getParameter("certlistwidth");
        if (certListWidth != null){
            maxCertListWidth = Integer.parseInt(certListWidth);
        }
        if (classicStyle()) {
            listCtrl = new ClassicCertificateListController();
        } else {
            listCtrl = new ModernCertificateListController();
        }
        bOk.setEnabled(false);
        chCertList.removeAll();
        // pad in relation to width of applet somehow
        String pad = "                                                       ";
        String msg = ResourceManager.getString("LOADING_CERTIFICATES") + pad;
        chCertList.setFont(new Font("Monospaced", Font.PLAIN, 12));//todo: find out
        chCertList.add(msg.substring(0, Math.min(maxCertListWidth, msg.length())));
//        chCertList.getFont().getSize()
        chCertList.setEnabled(false);
        certLoaderThread = ra.createThread(new CertificateLoader(this));
    }

    public Applet getApplet() {
        return this;
    }

    protected Label userMessageLabel;
    private boolean isErrorMsg; // can be info or error
    public synchronized void  userMessage(String header, String msg, boolean error) {
        if (classicStyle()) {
            String ok = ResourceManager.getString("DLG_INFO_BUTTON_OK");
            DlgInformation dlgInfo = new DlgInformation(ra, getFrame(), header, msg, ok);
            dlgInfo.show();
        } else {
//            FileLog.debug("userMessage("+msg+" ,"+error+") "+isErrorMsg);
            isErrorMsg = error;
            if (isErrorMsg){
                userMessageLabel.setForeground(Color.red);
            } else {
                userMessageLabel.setForeground(Color.black);
            }
            userMessageLabel.setText(msg);
            userMessageLabel.invalidate();
        }

    }

    public synchronized void clearUserMessage(boolean error) {
        if (classicStyle()) {
            // nothing to do
        } else {
            FileLog.debug("clearUserMessage("+error+") "+isErrorMsg);
            if (error == isErrorMsg){
                userMessageLabel.setText("");
            } // else cannot erase it
        }
    }

    public void confirmedUserMessage(String header, String msg) {
        String ok = ResourceManager.getString("DLG_INFO_BUTTON_OK");
        DlgInformation dlgInfo = new DlgInformation(ra, getFrame(), header, msg, ok);
        dlgInfo.show();
    }

    protected void commonInit() {
        if (ra == null) {
            setNoBootstrap();
        }
        String locale = paramReader.getParameter("locale");
        if (locale != null) {
            boolean validatedLocale = false;

            String lang = locale.substring(0, locale.indexOf(",")).trim();
            String country = locale.substring(locale.indexOf(",") + 1, locale.length()).trim();

            if (lang == null || country == null) {
                validatedLocale = false;
            } else if (lang.equals("da") && country.equals("DK")) {
                validatedLocale = true;
            } else if (lang.equals("en") && country.equals("US")) {
                validatedLocale = true;
            }  else if ( lang.equals("ca") && country.equals("ES") ) {
                validatedLocale = true;
            }  else if ( lang.equals("es") && country.equals("ES") ) {
                validatedLocale = true;
            }  else if ( lang.equals("nl") && country.equals("NL") ) {
                validatedLocale = true;
            } else {
                validatedLocale = false;
            }

            if (validatedLocale) {
                ResourceManager.setLocale(lang, country);
            } else {
                ResourceManager.setLocale("da", "DK");
            }
        } else {
            ResourceManager.setLocale("da", "DK");
        }

        String loglevel = paramReader.getParameter("loglevel");
        if (loglevel == null) {
            FileLog.setVerbose(FileLog.DEBUG);
        } else if (loglevel.equalsIgnoreCase("debug")) {
            FileLog.setVerbose(FileLog.DEBUG);
        } else if (loglevel.equalsIgnoreCase("info")) {
            FileLog.setVerbose(FileLog.INFO);
        } else if (loglevel.equalsIgnoreCase("warn")) {
            FileLog.setVerbose(FileLog.WARN);
        } else if (loglevel.equalsIgnoreCase("error")) {
            FileLog.setVerbose(FileLog.ERROR);
        } else if (loglevel.equalsIgnoreCase("fatal")) {
            FileLog.setVerbose(FileLog.FATAL);
        } else {
            FileLog.setVerbose(FileLog.DEBUG);
        }

// always output version info no matter what:
        FileLog.info("log-level set to: " + FileLog.getVerbose());
        FileLog.fatal(Version.oneliner());

        FileLog.info(Copyright.name);
        FileLog.info(Copyright.copyright);
        FileLog.info(Copyright.author);
        FileLog.info(Version.getBuildVersion());
        FileLog.info(Version.getBuildTimeStamp());
        FileLog.debug(this.getClass() + ".init() invoked");
    }

    /*
     * @param signtext String BASE64, UTF8
     * @param signproperties String BASE64, UTF8
     * @param certificateId String BASE64, UTF8
     * @param cpr String BASE64, UTF8
     */
    public void initialize() throws Exception {
        FileLog.debug("Timing: AbstractApplet.initialize():"+ new Date());
        paramReader = createParamReader();
        commonInit();
        OcesAppletUtils.setApplet(this);
        OcesAppletUtils.printInitInformation();


        String background = paramReader.getParameter("background");
        if (background != null) {
            StringTokenizer st = new StringTokenizer(background, ",;");
            String red = st.nextToken();
            String green = st.nextToken();
            String blue = st.nextToken();
            setBackground(new Color(Integer.parseInt(red), Integer.parseInt(green), Integer.parseInt(blue)));
        }

        //			keystoredir = getParameter("key.store.directory");
        if (FileLog.isDebugEnabled()) {
// todo: put back in prv
//			Properties props = OS.getProperties();
//			for (Enumeration enum = props.keys(); enum.hasMoreElements();) {
//				String key = (String) enum.nextElement();
//				FileLog.debug(key + "=" + OS.getProperty(key));
//			}
        }
//		if (!OS.javaplugin) {
//
//			try {
//				netscape.security.PrivilegeManager.enablePrivilege("UniversalFileAccess");
//				netscape.security.PrivilegeManager.enablePrivilege("UniversalTopLevelWindow");
//			} catch (Throwable t) {
//			};

        // signproperties is base64 encoded.
        String signProperties = paramReader.getParameter("signproperties");
        this.signProperties = new Properties();
        if (signProperties != null) {
            //			byte[] ba = Base64.decode(signProperties.getBytes());
            //			signProperties = new String(ba, "UTF8");
            StringTokenizer st = new StringTokenizer(signProperties, ",;");
            while (st.hasMoreTokens()) {
                String namevalue = st.nextToken();
                String name = namevalue.substring(0, namevalue.indexOf("=")).trim();
                String value = namevalue.substring(namevalue.indexOf("=") + 1, namevalue.length()).trim();
                this.signProperties.put(name, value);
            }
        }

        // subject is base64 encoded.
//todo: find out where to put
        String certificateId = paramReader.getParameter("certificateId");
        if (certificateId != null) {
            byte[] ba = Base64.decode(certificateId.getBytes());
            this.preSelectedCertificate = new String(ba, "UTF8");
        }

        String socialSecurityNumber = paramReader.getParameter("socialsecuritynumber");
        if (socialSecurityNumber != null && socialSecurityNumber.equalsIgnoreCase("yes")) {
            lSocialSecurityNumber = new Label(ResourceManager.getString("SOCIAL_SECURITY_NUMBER"));
            tfSocialSecurityNumber = new TextField();
            tfSocialSecurityNumber.setColumns(16);
        }

        String optionalEntry = paramReader.getParameter("optionalid");
        if ((optionalEntry != null) && (optionalEntryIsValid(optionalEntry))) {

            lOptionalEntry = new Label(ResourceManager.getString(optionalEntry));
            String optionalEntryLen = paramReader.getParameter("optionalentrylen");
            tfOptionalEntry = new TextField();
            if (optionalEntryLen != null) {
                tfOptionalEntry.setColumns(Integer.parseInt(optionalEntryLen));
            } else {
                tfOptionalEntry.setColumns(16);
            }
        }

        String host = paramReader.getParameter("logonto");
        if (host == null || host.length() == 0)
            host = getCodeBase().getProtocol() + "://" + getCodeBase().getHost();
        lLogonTo = new Label(ResourceManager.getString("LOGONTO"));
        lHost = new Label(host);

        lPassword = new Label(ResourceManager.getString("PASSWORD"));
        tfPassword = new TextField();
        tfPassword.setColumns(16);
        tfPassword.setEchoChar('*');

        chCertList = new Choice();
        chCertList.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                clearUserMessage(true);
            }
        });

        lCertList = new Label(ResourceManager.getString("CERTIFICATE_LIST"));
        bOk = new Button(ResourceManager.getString("OK"));
        bCancel = new Button(ResourceManager.getString("CANCEL"));
        bFunctions = new Button(ResourceManager.getString("FUNCTIONS"));

        setCertificateListController();

        refreshGUI();

        super.init();
    }

    protected boolean classicStyle() {
        String gui = paramReader.getParameter("gui");
        // if defined and set to "modern" - use that
//        FileLog.debug("parameter gui was set to: " + gui);
        boolean modernGui = (gui != null && gui.equalsIgnoreCase("modern"));
        return !modernGui;
    }

    private boolean optionalEntryIsValid(String optionalEntry) {
        for (int i = 0; i < validOptionalIds.length; i++) {
            if (validOptionalIds[i].equals(optionalEntry)) {
                return true;
            }
        }

        return false;
    }


    /*
     * @see ICallbackHandler#callback(String)
     */
    protected Object javascript(String function, String args[]) {
        try {
            String result = OcesAppletUtils.getFunctionResult(function, args);
            if (result != null) {
                if (args == null || args.length == 0)
                    args = new String[]{""};
                OcesAppletUtils.doAppletRequest(ra, args[0], result);
                return null;
            }

            Vector list = new Vector();
            if (args != null)
                for (int i = 0; i < args.length; i++)
                    list.addElement(args[i]);
            if (FileLog.isDebugEnabled())
                FileLog.debug("callback: function=" + function + " with args:" + list);
            if (list.size() > 0) {
                Object[] objs = new Object[list.size()];
                list.copyInto(objs);
                return netscape.javascript.JSObject.getWindow(this).call(function, objs);
            } else {
                netscape.javascript.JSObject.getWindow(this).call(function, new Object[]{
                });
                return null;
            }
            //		} catch (JSException e) {
            //			FileLog.error("error executing javascript",e);
            //			throw new RuntimeException("no javascript available: " + e.getMessage());
        } catch (UnsatisfiedLinkError e) {
            FileLog.error("no javascript available: " + e.getMessage());
            throw new RuntimeException("no javascript available: " + e.getMessage());
        } catch (NoClassDefFoundError e) {
            FileLog.error("no javascript available: " + e.getMessage());
            throw new RuntimeException("no javascript available: " + e.getMessage());
        }
    }

    protected void refreshCertificates(String selectedCertificate) {
//        updateCertificateDropDown(selectedCertificate);
        updateButtons();
        clearWaitMsg();// todo: find out if it is needed
//        update ();
        doLayout();
        validate();
    }

    /**
     * @see Applet#start()
     */
    public void start() {
        FileLog.info("Timing: AbstractApplet.start():"+ new Date());
        FileLog.debug("xxx AbstractApplet start() " + this.getClass().getClassLoader());

//		try {
//			if (FileLog.isDebugEnabled)
//				FileLog.debug(this.getClass() + ".start()");
//			javascript("appletStarted", null);
//		} catch (Throwable t) {
//			FileLog.error("unexpected error occured", t);
//		}
        checkLoadCerts();
    }

    private void checkLoadCerts() {
        if (!certsLoading && !certsLoaded) {
            // todo: hack! layout if userMesageLabel
            if (userMessageLabel != null) {
                clearUserMessage(false);
            }
            if (certLoaderThread != null) {
                certLoaderThread.start();
                certsLoading = true;
            }
        }
    }

    /**
     * @see Applet#stop()
     */
    public void stop() {
        FileLog.debug("xxx AbstractApplet stop() " + this.getClass().getClassLoader());
        try {

            if (FileLog.isDebugEnabled())
                FileLog.debug(this.getClass() + ".stop()");

        } catch (Throwable t) {
            FileLog.error("unexpected error occured", t);
        }
    }

    /**
     * wipe the password entry field, call garbage collection
     */
    public void clearPassword() {
        tfPassword.setText("");
        // sometimes two garbage collect requests are needed to really make things work
        System.gc();
        System.gc();
    }

    protected void refreshGUI() {
        updatePasswordField();
        refreshCertificates(null);
    }

    protected void updateButtons() {
        chCertList.setEnabled(!emptyCertificateList);
        bOk.setEnabled(!emptyCertificateList);
        FileLog.debug("updateButtons setEnabled cerstLoaded:"+certsLoaded);
        bFunctions.setEnabled(certsLoaded);
        enableDialog(!emptyCertificateList);
    }

    protected void updatePasswordField() {
        // todo: ask selected cert about this
//        if (keyStoreHandler.requiresPasswordSupplied()){
        if (false) {
            // disable password entry
            tfPassword.setVisible(true);
            lPassword.setVisible(true);
            tfPassword.setEnabled(true);
        } else {
            // disable password entry
            tfPassword.setVisible(false);
            lPassword.setVisible(false);
            tfPassword.setEnabled(false);
        }
    }

    public String promptPassword(String message) throws UserCancel {
        DlgPassword dlg = new DlgPassword(ra, getFrame(), message);
        dlg.show();
        if (!dlg.isCancelled()) {
            return dlg.getPassword();
        } else {
            throw new UserCancel();
//            return null;
        }
    }

    protected void addLayoutSettings(Properties p) {
        /* add properties concerning the applet layout.
           if the visible signed text is somehow unreadable we may be able to detect
           it using these values
        */
        p.put("openoces_opensign_layout_color_background", this.getBackground().getRed() + "," + this.getBackground().getGreen() + "," + this.getBackground().getBlue());
        try {
            p.put("openoces_opensign_layout_size_width", "" + this.getWidth());
            p.put("openoces_opensign_layout_size_height", "" + this.getHeight());
        } catch (java.lang.NoSuchMethodError e) {
            FileLog.debug("Applet dimensions not available");
        }
    }
    protected void addEnvironmentProperties(Properties p) {
        /* add properties concerning the applet environment.
            - opensign version - and signature?
            - os name&version
            - jvm name&version
            - browser
            - local time & locale? (to compare with locale parameter...)
        */

        // inspired by http://gunther.web66.com/crossjava/General/platformcheck/
        /*os.name = Windows XP
        browser = sun.plugin
        browser.vendor = Sun Microsystems, Inc.
        browser.version = 1.1
        java.vendor = Sun Microsystems Inc.
        java.version = 1.5.0_06 */
        // maybe use digest of OpenSignDownloads from MANIFEST.MF:
        /*
        Name: org/openoces/opensign/client/applet/bootstrap/downloads/OpenSignDownloads.class
        SHA1-Digest: qAHYJCh3Ryp4jT8qdBG+q1DWDG4=
        */
        // and then something else if it is in a cab...
        // todo: find out how to do this.
        // maybe: let bootstrap part provide digest of OpenSignDownloads!

        String ss[][] = {
                {"openoces_opensign_environment_applet_version",Version.oneliner()},
                {"openoces_opensign_environment_applet_digest", "todo: implement"},
                {"openoces_opensign_environment_os_name", OS.getProperty("os.name")},
                {"openoces_opensign_environment_browser_name", OS.getProperty("browser")},
                {"openoces_opensign_environment_browser_vendor", OS.getProperty("browser.vendor")},
                {"openoces_opensign_environment_browser_version", OS.getProperty("browser.version")},
                {"openoces_opensign_environment_java_vendor", OS.getProperty("java.vendor")},
                {"openoces_opensign_environment_java_version", OS.getProperty("java.version")},
                {"openoces_opensign_environment_applet_context", this.getAppletContext().toString()},
                {"openoces_opensign_environment_local_time", new Date().toString()},
                {"openoces_opensign_environment_locale", Locale.getDefault().toString()},
            };
        for (int i = 0; i < ss.length; i++) {
            String[] s = ss[i];
            String k = ss[i][0];
            String v = ss[i][1];
            if ( v != null ) {
                p.put(k,v);
            } else {
                /* not all properties are defined in all java-plugins, so we expect this */
                FileLog.debug("value for key: "+k+" is not available");
            }
        }
    }

    /**
     * RestrictedActor
     *
     * @param ra
     */
    public void setRestrictedAction(RestrictedAction ra) {
        this.ra = ra;
        OS.setRestrictedAction(ra);
    }

    /**
     * Returns the restricted-action class
     * @return the restricted-action class
     */
    public RestrictedAction getRestrictedAction() {
        return ra;

    }

    /**
     * Plugaware
     *
     * @param plugin
     */
    public void addPlugin(Object plugin) {
        FileLog.debug("RealApplet adding plugin " + plugin);
        if (plugin instanceof KeyStoreHandler) {
            KeyStoreHandler keyStoreHandler = (KeyStoreHandler) plugin;
            FileLog.debug("It is a KeyStoreHandler: " + keyStoreHandler + " ...");
            if (keyStoreHandler.isSupported(ra)) {
                FileLog.debug("AND it is supported on this client");
                keyStoreHandler.setCallBackHandler(this);
                keyStoreHandler.setRestrictedAction(ra);
                keyStoreHandlers.addElement(keyStoreHandler);
            } else {
                FileLog.debug("BUT it is not supported so we ignore it!");
            }
        }
        if (plugin instanceof CryptoSupport) {
            OS.setCryptoSupport((CryptoSupport) plugin);
        }
        if (plugin instanceof AppletSupport) {
            OS.setAppletSupport((AppletSupport) plugin);
        }
    }

    protected Vector keyStoreHandlers = new Vector();
}