/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: AppletArguments.java,v 1.6 2006/08/12 19:02:38 cara Exp $ */

package org.openoces.opensign.client.applet;

/**
 * This class implements the general applet functionality.
 *
 * @author Jens Bo Friis  <jbf@it-practice.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 * @author rh
 * @author Michael Motet
 */

import org.openoces.opensign.utils.Base64;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.xml.nanoxml.XMLElement;
import org.openoces.opensign.xml.nanoxml.XMLParseException;
import org.openoces.opensign.xml.xmldsig.SignatureGenerator;
import org.openoces.opensign.xml.xmldsig.VirkSignatureGenerator;

import java.applet.Applet;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.Hashtable;

public class AppletArguments implements ParamReader {

    protected XMLElement _doc; // this will hold the top element in the XML
    private String xml;
    private Applet applet;
    private Hashtable ht = new Hashtable();

    private XMLElement dataContainerElement;

//	AppletLayoutInfo layoutInfo;
//	AttachmentInfoCollection attachments;

    public AppletArguments(Applet applet) {
        this.xml = applet.getParameter("signtext");
        if (xml != null){
            getDocument();
        }
        this.applet = applet;
    }

    protected String getDefaultXML() {
        return "<?xml version=\"1.0\" encoding=\"utf8\" ?>";
    }

    /**
     * Returns a dom containing xml parameter given to the applet.
     *
     * @return
     */
    // todo: make private when done testing
    public XMLElement getDocument() {
        if (xml == null){
            xml = getDefaultXML();
        }
        XMLElement doc = new XMLElement();
        FileLog.info("xml:\r\n\"" + xml.substring(0, Math.min(300, xml.length())) + "\"");

        //TODO set try catch here
        doc.parseString(xml);

        _doc = doc; // set doc element value in class var
        return _doc;
    }

    // todo: make private when done testing
    public XMLElement getNode(XMLElement node, String xmlTag) {
        XMLElement lNode = null;
        FileLog.debug("Assign local node.");
        XMLElement tstNode = node;
        FileLog.debug("Test tstnode...");
        FileLog.debug("tstnode" + tstNode.toString());
        try {
            if (node == null)
                FileLog.debug("node is NULL");
            else
                FileLog.debug("node is :" + node.getName() + "\n" + node.toString());
            FileLog.debug("1. Calling Enum on the node");
            Enumeration enumeration = node.enumerateChildren();
            FileLog.debug("2. Calling while on the enum");
            while (enumeration.hasMoreElements()) {
                FileLog.debug("3. Entered the while loop");
                XMLElement child = (XMLElement) enumeration.nextElement();
                FileLog.debug("4. Getting next child element in while loop");
                if (child.getName().compareTo(xmlTag) == 0) {
                    FileLog.debug("5. Child found.");
                    lNode = child;
                    break;
                }
            }
        } catch (XMLParseException ex) {
            // not good
            ex.printStackTrace();
        }
        FileLog.debug("6. Returning the node.");
        return lNode;
    }


    private XMLElement getNodeWithAttribute(XMLElement node, String xmlTag, String Attribute, String AttribParam) {
        XMLElement lNode = null;

        try {
            Enumeration enumeration = node.enumerateChildren();
            while (enumeration.hasMoreElements()) {
                XMLElement child = (XMLElement) enumeration.nextElement();

                if ((child.getName().compareTo(xmlTag) == 0) && (getNodeAttribute(child, Attribute))) {
                    if (((String) child.getAttribute(Attribute)).equalsIgnoreCase(AttribParam)) {
                        lNode = child;
                        break;
                    }
                }
            }
        } catch (XMLParseException ex) {
            // not good
            ex.printStackTrace();
        }
        return lNode;
    }


    private boolean getNodeAttribute(XMLElement node, String Attribute) {
        String attributeString = null;
        boolean attributeFound = false;

        try {

            Enumeration enumeration = node.enumerateAttributeNames();
            while (enumeration.hasMoreElements()) {

                attributeString = (String) enumeration.nextElement();

                if (attributeString.equalsIgnoreCase(Attribute)) {
                    attributeFound = true;
                    break;
                } else {
                    attributeString = null;
                }
            }
        } catch (XMLParseException ex) {
            // not good
            attributeString = null;
        }

        return attributeFound;
    }

    private XMLElement getDataContainerElement() {
        XMLElement dataContainerElement = null;
        try {
//			 _doc name is virk:SignInputStructure
            XMLElement firstChild = getNode(_doc, "virk:DataContainer");
            dataContainerElement = firstChild;
        } catch (XMLParseException ex) {
            // not good
            ex.printStackTrace();
        }

        return dataContainerElement;
    }

    /**
     * Returns the input xml attachment element containing the sign text.
     *
     * @return Sign text (html).
     */
    private XMLElement getSignTextAttachmentElement() {
        XMLElement el = null;
        String xpath;

        FileLog.debug("inside getsigntextattchmentElement - going for the try catch.");

        try {
            //XMLElement rootNode = getNode(_doc, "virk:SignInputStructure");
            FileLog.debug("Calling _doc.GetName...");
            FileLog.debug("Doc is :" + _doc.getName() + "\n" + _doc.toString());
            XMLElement firstChild = getNode(_doc, "virk:DataContainer");
            XMLElement secondChild = getNode(firstChild, "virk:DataObjectCollection");
            XMLElement finalchild = getNodeWithAttribute(secondChild, "virk:DataObject", "fileContentType", "SignText");

            Enumeration enumeration = finalchild.enumerateAttributeNames();
            while (enumeration.hasMoreElements()) {

                String attributeString = (String) enumeration.nextElement();
                if (attributeString.equalsIgnoreCase("fileContentType")) {
                    String attStr = (String) finalchild.getAttribute(attributeString);
                    if (attStr.compareTo("SignText") == 0) {
                        el = finalchild;
                        break;
                    }
                }
            }


        } catch (XMLParseException ex) {
            FileLog.error("An error occurred when locating sign text attachment structure.");
            return null;
        }

        if (el == null) {
            FileLog.error("Could not locate sign text attachment structure.");
        }

        return el;
    }

    /**
     * Returns the text or html that should be presented to the user.
     *
     * @return Text or html.
     */
    // todo: make private when done testing
    public String getSignText() {
        String xpath;
        String rawSignText, signText;
        XMLElement el;

        FileLog.debug("inside getsigntext - calling getsigntextAttachmentElement");
        el = getSignTextAttachmentElement();

        rawSignText = el.getContent();
        signText = "";
        try {
            byte[] ba = Base64.decode(rawSignText.getBytes());
            signText = new String(ba, "UTF8");
        } catch (UnsupportedEncodingException ex) {
        }

        return signText;
    }

    public String getParameter(String name) {
        if ("signtext".equals(name)){
            return getSignText();
        } else {
            String v;
            if ( !ht.containsKey(name)) {
                v = applet.getParameter(name);
                ht.put(name,v);
            } else {
                v = (String)ht.get(name);
            }
            return v;
        }
    }

    public SignatureGenerator createSignatureGenerator() {
        if (xml == null){
            return new VirkSignatureGenerator();
        } else {
            return new VirkSignatureGenerator(getDocument());
        }
    }

    public StringBuffer getSigntext() {
        return new StringBuffer(getParameter("signtext"));
    }
}