/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: AppletLayout.java,v 1.14 2006/10/21 14:00:10 cara Exp $ */

package org.openoces.opensign.client.applet;

/**
 * This class implements  common code for Sign and Logon
 * - but not in AbstractApplet (for historical reasons?)
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

import org.openoces.opensign.certificate.filter.*;
import org.openoces.opensign.client.applet.bootstrap.interfaces.PollingApplet;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.exceptions.InputDataError;
import org.openoces.opensign.utils.Base64;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.awt.*;
import java.io.UnsupportedEncodingException;

public abstract class AppletLayout extends AbstractApplet implements PollingApplet  {
    // RM polling code
    private StringBuffer outputData = new StringBuffer("");
    private String appletState = "waiting";

    public String getAppletState() {
        return appletState;
    }

    public String getOutputData() {
        String data;
        try {
            data = new String(Base64.encode(outputData.toString().getBytes("UTF8")), "UTF8");
        } catch (UnsupportedEncodingException ex) {
            data = null;
        }
        return data;
    }

    private void setAppletState(String state) {
        appletState = state;
    }
    private void setOutputData(StringBuffer data) {
        outputData = data;
    }

    /**
     * Reports that the signing process has been canceled and clears any old output data.
     */
    private void reportCancel() {
        setOutputData(new StringBuffer(""));
        setAppletState("signingcanceled");
    }

    private void reportDataSigned() {
        setAppletState("datasigned");
    }

    /**
     * Reports an error and clears the output data.
     */
    private void reportError(String errorDescription) {
        setOutputData(new StringBuffer(errorDescription));
        setAppletState("error");
    }

    // end RM polling code
    // general handling of result of signing/logging on
    protected void handleOk(String javaScriptFunction, StringBuffer signature){
        setOutputData(signature);
        reportDataSigned();
        try {
            javascript(javaScriptFunction, new String[]{new String(Base64.encode(signature.toString().getBytes("UTF8")), "UTF8")});
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }
    protected void handleError(String javaScriptFunction, String msg){
        javascript(javaScriptFunction, new String[]{msg});
        reportError("could not create signature");
    }
    protected void handleCancel(){
        FileLog.debug("cancel pressed");
        javascript("onSignCancel", null);
        reportCancel();
    }


    public void init() {
        FileLog.debug("xxx Sign/Logon init() " + this.getClass().getClassLoader());
        try {
            super.initialize();
            initFilters();
            initialize();
        } catch (Exception e) {
            FileLog.error("error during initialize", e);
            javascript("onSignError", new String[]{e.toString()});
            reportError("error during initialize");
        }
    }

    private void initFilter(String parameterName, Class filterClass) throws InstantiationException, IllegalAccessException {
        String filterParameter = "mandatory";
        if (parameterName != null) {
            filterParameter = paramReader.getParameter(parameterName);
        }
        if (filterParameter != null && !filterParameter.equals("")) {
            CertificateFilter f = (CertificateFilter) filterClass.newInstance();
            f.setParameter(filterParameter);
            CertificateFilter.add(f);
        }
    }

    protected void initFilters() {
        // check if the filtering should be repeated - this takes some time!
        if (CertificateFilter.isUsed()) {
            // maybe we don't have to read & filter the certs again
            String refreshcerts = paramReader.getParameter("refreshcerts");
            if (refreshcerts != null && refreshcerts.equals("no")) {
                // use the old list
                FileLog.debug("Refreshcerts is \"no\", Reusing the old certificate list");
                return;
            } else {
//                MicrosoftWindowsCertificateCache.getInstance().clearCache();
            }
        }
        CertificateFilter.reset();
        // KeyUsage-filter (which is currently a canSign()-filter) is always automatically added as it is a signing applet.
        CertificateFilter.add(new KeyUsageFilter());
        // for backwards compatibility we only include company certs if "includecompanycerts" is set to "yes"
        String includeCompanyCerts = getParameter("includecompanycerts");
        boolean doIncludeCompanyCerts = (includeCompanyCerts != null && includeCompanyCerts.equals("yes"));
        if (!doIncludeCompanyCerts){
            // we also remove "virksomhedscertifikater" as they are not applicable
            CertificateFilter.add(DenySubjectDnCertificateFilter.denyCompanyFilter);
        }
        try {
            initFilter("subjectdnfilter", SubjectDnCertificateFilter.class);
            initFilter("sernofilter", SerialNoCertificateFilter.class);
            initFilter("issuerdnfilter", IssuerDnCertificateFilter.class);
        } catch (InstantiationException e) {
            FileLog.error(e.getMessage());
        } catch (IllegalAccessException e) {
            FileLog.error(e.getMessage());
        }
    }

    protected boolean getExternalCertificateEnabledIndicator() {
        // todo: check KeyStoreHandlers - CAPI doesn't make sense with "Load"-button
        return true;
    }

    protected boolean getCancelButtonEnabledIndicator() {
        return true;
    }

    protected Button getCancelButton() {
        bCancel.setLabel(ResourceManager.getString("CANCEL_MODERN"));
        return bCancel;
    }

    abstract protected Button getOkButton();

    protected Button getFunctionsButton() {
        bFunctions.setLabel(ResourceManager.getString("FUNCTIONS_MODERN"));
        return bFunctions;
    }

    protected GridBagConstraints getGridBagConstraints() {
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.fill = GridBagConstraints.NONE;
        c.weightx = 0;
        c.weighty = 0;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(3, 3, 3, 3);
        return c;
    }

    abstract protected Panel getHeaderPanel();

    protected Panel getAttachmentPanel() {
        return null;
    }

    protected void doModernLayout(TextArea ta, boolean fixedTextHeight, boolean fixedTextWidth) throws InputDataError {
        // arrange the components:
        // taSignText
        // (lPassword, tfPassword)
        // bOk, bCancel, bFunctions,
        setLayout(new GridBagLayout());

        GridBagConstraints c = getGridBagConstraints();

        add(getHeaderPanel(), c);

        c.gridy++;
        if (ta != null) {
            GridBagConstraints textConstraints = (GridBagConstraints) c.clone();
            textConstraints.gridwidth = 2;

            if (!fixedTextWidth && !fixedTextHeight) {
                textConstraints.fill = GridBagConstraints.BOTH;
                textConstraints.weightx = 1;
                textConstraints.weighty = 1;
            } else if (!fixedTextWidth && fixedTextHeight) {
                textConstraints.fill = GridBagConstraints.HORIZONTAL;
                textConstraints.weightx = 1;
                textConstraints.weighty = 0;
            } else if (fixedTextWidth && !fixedTextHeight) {
                textConstraints.fill = GridBagConstraints.VERTICAL;
                textConstraints.weightx = 0;
                textConstraints.weighty = 1;
            } else if (fixedTextWidth && fixedTextHeight) {
                textConstraints.fill = GridBagConstraints.NONE;
                textConstraints.weightx = 0;
                textConstraints.weighty = 0;
            }
            add(ta, textConstraints);
        }
        c.gridy++;
        Panel attachmentPanel = getAttachmentPanel();
        if (attachmentPanel != null){
            add(attachmentPanel, c);
            c.gridy++;
        }
        add(getButtonPanel(), c);
    }

    protected Panel buttonPanel;
    // Part of Virk layout:
    // if we determine to have it - finish the code related to this
    protected static final boolean USE_DETAILS_BUTTON = false;

    protected Panel getButtonPanel() {
        if (buttonPanel != null)
            return buttonPanel;

        buttonPanel = new Panel();

//        buttonPanel.setBackground(layoutInfo.getBackgroundColor());
        buttonPanel.setLayout(new BorderLayout());

        // Certificate label, listbox and load button.
        Panel upperButtonPanel = new Panel();
//            upperButtonPanel.setBackground(layoutInfo.getBackgroundColor());

        upperButtonPanel.add(lCertList, BorderLayout.WEST);
        upperButtonPanel.add(chCertList, BorderLayout.CENTER);

        // could add details button:
        // todo: finish this:
        if (USE_DETAILS_BUTTON) {
            Panel certButtonPanel = new Panel();
            certButtonPanel.setLayout(new FlowLayout());
            certButtonPanel.add(new Button("Detaljer..."));
            certButtonPanel.add(getFunctionsButton());
            upperButtonPanel.add(certButtonPanel, BorderLayout.EAST);
        } else {
            if (getExternalCertificateEnabledIndicator()) {
                upperButtonPanel.add(getFunctionsButton(), BorderLayout.EAST);
            }
        }

        Panel lowerButtonPanel = new Panel();
        lowerButtonPanel.setLayout(new BorderLayout());

        userMessageLabel = new Label();
        userMessageLabel.setForeground(Color.red);
        lowerButtonPanel.add(userMessageLabel, BorderLayout.CENTER);

        Panel realButtonPanel; // The *real* panel ;)
        realButtonPanel = new Panel();
        realButtonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

        if (getCancelButtonEnabledIndicator()) {
            realButtonPanel.add(getCancelButton());
        }
        realButtonPanel.add(getOkButton());

        lowerButtonPanel.add(realButtonPanel, BorderLayout.EAST);

        buttonPanel.add(upperButtonPanel, BorderLayout.NORTH);
        buttonPanel.add(lowerButtonPanel, BorderLayout.SOUTH);

        return buttonPanel;
    }
}
