/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: ClassicCertificateListController.java,v 1.9 2006/10/14 21:43:53 cara Exp $ */

package org.openoces.opensign.client.applet;

/**
 * This class implements a classical view of the list of certificates
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.CertificateInfo;
import org.openoces.opensign.certificate.CopyableItem;
import org.openoces.opensign.certificate.RestoreableItem;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.dialogs.DlgAbout;
import org.openoces.opensign.client.applet.dialogs.DlgCertificateDetails;
import org.openoces.opensign.client.applet.dialogs.DlgKeystoreChooser;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Vector;

public class ClassicCertificateListController implements CertificateListController {
    private CertificateListView view;
    private String preselectedCertificate;

    public void addKeyStoreHandler(KeyStoreHandler handler) {
        FileLog.debug("Adding keystore handler to classic certificatelist controller: " + handler.getName());
        keyStoreHandlers.addElement(handler);
    }

    public void init(CertificateListView view, Button btn, RestrictedAction ra) {
        // compile a list of certificates and:
        this.view = view;
        this.ra = ra;
        initKeystores();
        view.setCertificateList(keyStoreHandler.getCertificates());
        initFunctionsPopup();
        ((Component) view).add(functionsPopup); //must have a parent
        ActionListener showFunctions = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                functionsPopup.show((Button) e.getSource(), 40, 10);
            }
        };
        btn.addActionListener(showFunctions);
    }

    public void action(AbstractCertificateHandler cert) {
        // show popup
    }

    public String getCertificateDisplayName(CertificateInfo info, int maxWidth) {
        try {
            return info.getUserfriendlyName();
        } catch (Exception e) {
            return "error";
        }
    }

    private Frame getFrame() {
        // todo: make nicer
        return ((AbstractApplet) view).getFrame();
    }

    protected Object javascript(String function, String args[]) {
        // todo: make nicer
        return ((AbstractApplet) view).javascript(function, args);
    }

    protected void initKeystores() {
        keyStoreHandler = null;
        FileLog.debug("certificatelist controller is initializing keystores");
        if (keyStoreHandlers.isEmpty()) {
            FileLog.error("No keystore type available");
            ((CallBackHandler) view).userMessage(ResourceManager.getString("DLG_INFO_HEADER_NO_KEYSTORES_AVAILABLE"), ResourceManager.getString("DLG_INFO_TEXT_NO_KEYSTORES_AVAILABLE"), true);
            // FIXME: We should probably exit the applet with an appropriate error result here
        }
        for (int i = 0; i < keyStoreHandlers.size(); i++) {
            KeyStoreHandler storeHandler = (KeyStoreHandler) keyStoreHandlers.elementAt(i);
            // select the first non-empty keyStoreHandler
            if (!storeHandler.isEmpty()) {
                keyStoreHandler = storeHandler;
                break;
            }
        }
        if (keyStoreHandler == null) {
            // select the first one if all are empty
            keyStoreHandler = (KeyStoreHandler) keyStoreHandlers.elementAt(0);
        }

        keyStoreHandler.refreshKeystore();
    }

    RestrictedAction ra;
    private File lastDirectoryName = null;

    protected PopupMenu functionsPopup;
    private KeyStoreHandler keyStoreHandler;
    protected MenuItem miAbout;
    protected MenuItem miBackupKeyfile;
    protected MenuItem miDeleteKeyfile;
    protected MenuItem miCertDetails;
    protected MenuItem miRestoreKeyfile;
    protected MenuItem miSelectKeystore;
    protected CheckboxMenuItem cbmiKeyfileFromPath;
    private final static int KEYSTORE_UNDEFINED = 0;
    private final static int KEYSTORE_CDCARD = 3;
    private final static int KEYSTORE_COUNT = KEYSTORE_CDCARD + 1;
    // same sequence below:
    private CheckboxMenuItem [] keystoreMenuItems = new CheckboxMenuItem[KEYSTORE_COUNT];
    private int selectedKeyStore = KEYSTORE_UNDEFINED;
    private Vector keyStoreHandlers = new Vector();
    protected boolean pathSelected = false;

    protected void initFunctionsPopup() {
        functionsPopup = new PopupMenu(ResourceManager.getString("FUNCTIONS"));
//		add(functionsPopup);
        // prv:
        // CD CARD skal overvejes n�je - om det skal dukke op dynamisk eller statisk - m�ske konfigurerbart
        // - hvis brugeren ikke lige har sat sit kort i drevet, vil det ikke dukke op som mulighed hvis
//        ItemListener il = new ItemListener() {
//            public void itemStateChanged(ItemEvent e) {
//                if (FileLog.isDebugEnabled)
//                    FileLog.debug(e.toString());
//                selectKeystore((CheckboxMenuItem)e.getSource());
//                keyStoreHandler.refreshKeystore();
//                view.setCertificateList(keyStoreHandler.getCertificates());
//            }
//        };

//        for (int i = 0; i < keyStoreHandlers.size(); i++) {
//            KeyStoreHandler storeHandlerIter = (KeyStoreHandler) keyStoreHandlers.elementAt(i);
//            CheckboxMenuItem checkboxMenuItem;
//            String name = storeHandlerIter.getName();
//            String localizedName = storeHandlerIter.getLocalizedName();
//            checkboxMenuItem = new CheckboxMenuItem(localizedName, keyStoreHandler == storeHandlerIter );
//            checkboxMenuItem.setName(name);
//            functionsPopup.add(checkboxMenuItem);
//            checkboxMenuItem.addItemListener(il);
//            keystoreMenuItems[i] = checkboxMenuItem;
//        }

//        functionsPopup.add("-------------");
//		miRestoreKeyfile = new MenuItem(ResourceManager.getString("RESTORE_KEYFILE"));
//		functionsPopup.add(miRestoreKeyfile);
//		miDeleteKeyfile = new MenuItem(ResourceManager.getString("DELETE_KEYFILE"));
//		functionsPopup.add(miDeleteKeyfile);
//		miBackupKeyfile = new MenuItem(ResourceManager.getString("BACKUP_KEYFILE"));
//		functionsPopup.add(miBackupKeyfile);

        miSelectKeystore = new MenuItem(ResourceManager.getString("SELECT_KEYSTORE"));
        functionsPopup.add(miSelectKeystore);
        miCertDetails = new MenuItem(ResourceManager.getString("CERTIFICATE_DETAILS"));
        functionsPopup.add(miCertDetails);
//		cbmiKeyfileFromPath = new CheckboxMenuItem(ResourceManager.getString("PATH"));
//		cbmiKeyfileFromPath.setState(false);
//		functionsPopup.add(cbmiKeyfileFromPath);

//		cbmiKeyfileFromPath.addItemListener(new ItemListener() {
//			public void itemStateChanged(ItemEvent e) {
//				FileLog.debug("item listener for using a certificate from path got an event: "+e.toString());
//                try {
//                    selectPathToggled();
//                } catch (Throwable e1) {
//                    FileLog.error("An exception occurred",e1);
//                }
//                FileLog.debug("setting state of locate-certificate menu item to "+pathSelected);
//                pathSelected = !pathSelected;
//                cbmiKeyfileFromPath.setState(pathSelected);
//			}
//		});

        miAbout = new MenuItem(ResourceManager.getString("ABOUT"));
        functionsPopup.add(miAbout);

        functionsPopup.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
//				try {
                FileLog.debug("popup listener got event:" + e.toString());

//					if (e.getActionCommand().equals(ResourceManager.getString("BACKUP_KEYFILE"))) {
//						backupSelectedCertificate();
//						return;
//					}
//					if (e.getActionCommand().equals(ResourceManager.getString("RESTORE_KEYFILE"))) {
//						restoreCertificate();
//						return;
//					}
//					if (e.getActionCommand().equals(ResourceManager.getString("DELETE_KEYFILE"))) {
//						deleteSelectedCertificate();
//						return;
//					}
                if (e.getActionCommand().equals(ResourceManager.getString("CERTIFICATE_DETAILS"))) {
                    AbstractCertificateHandler[] certs = keyStoreHandler.getCertificates();
                    AbstractCertificateHandler certificate = view.getSelectedCertificate();
//                                certs[chCertList.getSelectedIndex()];
                    try {
                        DlgCertificateDetails dlg = new DlgCertificateDetails(ra, getFrame(), ResourceManager.getString("CERTIFICATE_DETAILS"), certificate);
                        dlg.show();
                    } catch (Exception ex) {
                        FileLog.error("error occurred during show of details dialog", ex);
                    }
                    return;
                }
                if (e.getActionCommand().equals(ResourceManager.getString("ABOUT"))) {
                    about();
                    return;
                }
                if (e.getActionCommand().equals(ResourceManager.getString("SELECT_KEYSTORE"))) {
                    keystorechooser();
                    return;
                }
//				} catch (IOException ex) {
//					FileLog.error("IO error occurred", ex);
//				}
            }

        });
//		if (!keyStoreHandler.supportsCopy()) {
//			cbPKCS12.setEnabled(false);
//			miBackupKeyfile.setEnabled(false);
//			miDeleteKeyfile.setEnabled(false);
//			miRestoreKeyfile.setEnabled(false);
//		}
    }

    private void selectKeystore(CheckboxMenuItem selectedItem) {
        if (!selectedItem.getState()) {
            // it is allready selected and a deselect is attempted - not allowed
            selectedItem.setState(true);
            return;
        } else {
            // deselect the old one:
            if (keystoreMenuItems[selectedKeyStore] != null) {
                keystoreMenuItems[selectedKeyStore].setState(false);
            }
        }
        // find out which one it was
        for (int i = 0; i < keystoreMenuItems.length; i++) {
            if (selectedItem.equals(keystoreMenuItems[i])) {
                selectedKeyStore = i;
                keyStoreHandler = (KeyStoreHandler) keyStoreHandlers.elementAt(i);
            }
        }
    }

    private void keystorechooser() {
        DlgKeystoreChooser dlg = new DlgKeystoreChooser(ra, getFrame(), ResourceManager.getString("SELECT_KEYSTORE"), keyStoreHandlers, keyStoreHandler);
        dlg.show();
        if ( !dlg.wasCancelled() ) {
            keyStoreHandler = dlg.getSelectedKeystoreHandler();
            refreshCertificates(null);
        }
    }

    private void about() {
        try {
            DlgAbout dlg = new DlgAbout(ra, getFrame());
            dlg.show();
        } catch (Exception ex) {
            FileLog.error("error occurred during show of about dialog", ex);
        }
    }

    protected void backupSelectedCertificate() throws IOException {
        AbstractCertificateHandler selectedCert = view.getSelectedCertificate();
        if (selectedCert instanceof CopyableItem) {
            ((CopyableItem) selectedCert).backup(getFrame());
        }
    }

    public boolean confirm(String question) {
        try {
            Boolean bool = (Boolean) javascript("confirm", new String[]{question});
            return bool.booleanValue();
        } catch (Exception e) {
            FileLog.error("CONFIRM " + question);
            return false;
        }
    }

    protected void deleteSelectedCertificate() {
        FileLog.debug("About to delete selected certificate");
        String message = ResourceManager.getString("CONFIRM_DELETE");
        if (confirm(message)) {
            FileLog.debug("client accepted deletion of certificate");
            AbstractCertificateHandler selectedCert = view.getSelectedCertificate();
            FileLog.debug("selected cert: " + selectedCert);
            FileLog.debug("it is CopyableItem : " + (selectedCert instanceof CopyableItem));
            if (selectedCert instanceof CopyableItem) {
                FileLog.debug("certificate to be deleted is copyable");
                ((CopyableItem) selectedCert).delete(getFrame());
            } else {
                FileLog.debug("certificate to be deleted is not coptable - aborting");
            }
            refreshCertificates(null);
        } else {
            FileLog.debug("client cancelled deletion of pkcs12");
        }
    }

    protected void refreshCertificates(String selectedCertificate) {
//        updateCertificateDropDown(selectedCertificate);
//        updateButtons();
        keyStoreHandler.refreshKeystore();
        updateFunctionsPopup();
        view.setCertificateList(keyStoreHandler.getCertificates());
//        doLayout();
//        validate();
    }

    protected void restoreCertificate() throws IOException {
        AbstractCertificateHandler selectedCert = view.getSelectedCertificate();
        if (selectedCert instanceof RestoreableItem) {
            ((RestoreableItem) selectedCert).restore(getFrame());
            keyStoreHandler.refreshKeystore();
            refreshCertificates(selectedCert.getUserfriendlyName());
        }
    }

    /**
     * The path select was toggled (if toggled to true then we open a file dialog)
     *
     * @throws IOException
     */
    protected void selectPathToggled() throws IOException {
        pathSelected = !pathSelected;
        if (pathSelected) {
            if (FileLog.isDebugEnabled()) {
                FileLog.debug("path selected");
            }
            FileDialog dlg = ra.getFileDialog(getFrame(), ResourceManager.getString("PATH"), FileDialog.LOAD);
            dlg.setFilenameFilter(new FilenameFilter() {
                public boolean accept(File dir, String n) {
                    return n.toLowerCase().endsWith("pfx") || n.toLowerCase().endsWith("p12") || n.toLowerCase().endsWith("pkcs12");
                }
            });
            dlg.setModal(true);
            ra.showFileDialog(dlg);

            if (dlg.getFile() == null) {
                FileLog.debug("dialog cancel");
                pathSelected = !pathSelected;
                return;
            }
            File selectedFile = new File(dlg.getFile());
            lastDirectoryName = new File(dlg.getDirectory());

            refreshCertificates(new File(lastDirectoryName, selectedFile.toString()).toString());
            FileLog.debug("path " + selectedFile + " selected");

        } else
            refreshCertificates(null);
    }

    protected void updateFunctionsPopup() {
        FileLog.debug("updating functions-popup menu");
        // update it just before it is shown - depending on which cert is selected - not just keystoretype
        if (keyStoreHandler.getCertificates().length > 0) {
            miCertDetails.setEnabled(true);
//            cbmiKeyfileFromPath.setEnabled(false);
        } else {
            miCertDetails.setEnabled(false);
//            cbmiKeyfileFromPath.setEnabled(false);
        }
    }
//    protected void updateCertificateDropDown(String selectedCertificate){
//        if (selectedCertificate == null)
//            selectedCertificate = preSelectedCertificate;
//        keyStoreHandler.refreshKeystore();
//        AbstractCertificateHandler[] certs = getKeyStoreHandler().getCertificates();
//        chCertList.removeAll();
//        for (int i = 0; i < certs.length; i++) {
//            chCertList.add(certs[i].getUserfriendlyName());
//        }
//        if (selectedCertificate != null){
//            chCertList.select(selectedCertificate.replace('\\', '/'));
//        }
//    }


}
