/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DefaultParamReader.java,v 1.7 2006/08/12 19:02:39 cara Exp $ */

package org.openoces.opensign.client.applet;

import org.openoces.opensign.xml.xmldsig.BasicSignatureGenerator;
import org.openoces.opensign.xml.xmldsig.SignatureGenerator;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.utils.Base64;

import java.util.Hashtable;
import java.applet.Applet;
import java.io.UnsupportedEncodingException;

/**
 * This class implements the default parameters reader
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

public class DefaultParamReader implements ParamReader {

    protected Hashtable ht = new Hashtable();
    protected Applet applet;
    protected StringBuffer signtext = null;

    public DefaultParamReader(Applet applet) {
        this.applet = applet;
    }

    public String getParameter(String key) {
        String v;
        if ( !ht.containsKey(key)) {
            v = applet.getParameter(key);
            if ( v != null ) {
                ht.put(key,v);
            }
        } else {
            v = (String)ht.get(key);
        }

        return v;
    }

    public SignatureGenerator createSignatureGenerator() {
        return new BasicSignatureGenerator();
    }

    /**
     * Returns the text to be signed
     *
     * Note that we keep a separate getter for the signtext, as it can be large and immutable strings therefore should
     * be avoided.
     *
     * @return the text to be signed
     */
    public StringBuffer getSigntext() {
        if ( signtext == null ) {
            byte[] ba = Base64.decode(applet.getParameter("signtext").getBytes());
            try {
                signtext = new StringBuffer(new String(ba, "UTF8"));
            } catch (UnsupportedEncodingException e) {
                FileLog.fatal("utf-8 not supported");
                // FIXME: handle error situation
            }
        }
        return signtext;
    }
}