/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: ModernCertificateListController.java,v 1.7 2006/08/12 19:02:39 cara Exp $ */

package org.openoces.opensign.client.applet;

/**
 * This class implements a modern controller for the certificate list
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.CertificateInfo;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Vector;

public class ModernCertificateListController implements CertificateListController {
    private Vector keyStores = new Vector();
    private Vector keyStoreCerts = new Vector();
    private Vector foundCerts = new Vector();
    private RestrictedAction ra;
    private CertificateListView view;

    public void addKeyStoreHandler(KeyStoreHandler handler) {
        keyStores.addElement(handler);
        handler.refreshKeystore();
        if (!handler.isEmpty()){
	        AbstractCertificateHandler[] certs = handler.getCertificates();
	        for (int i = 0; i < certs.length; i++) {
	            AbstractCertificateHandler cert = certs[i];
	            keyStoreCerts.addElement(cert);
	        }
        }
    }

    public void init(CertificateListView view, Button btn, RestrictedAction ra) {
        // compile a list of certificates and:
        this.view = view;
        this.ra = ra;
        // check if any keystorehandlers supports browsing:
        boolean supportBrowsing = false;
        for (int i = 0; i < keyStores.size(); i++) {
            KeyStoreHandler keyStore = (KeyStoreHandler) keyStores.elementAt(i);
            FilenameFilter filter = keyStore.getFilenameFilter();
            if (filter != null) {
                supportBrowsing = true;
                break;
            }
        }
        btn.setEnabled(supportBrowsing);
        if (supportBrowsing) {
            btn.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    btnPushed();
                }
            });
        }
        setCertificates();
    }

    private void setCertificates() {
        // first put the foundCerts in reverse order
        AbstractCertificateHandler[] certArray = null;
        if (foundCerts.size() > 0) {
            certArray = new AbstractCertificateHandler[foundCerts.size() + keyStoreCerts.size()];
            for (int i = 0; i < foundCerts.size(); i++) {
                certArray[foundCerts.size() - i - 1] = (AbstractCertificateHandler) foundCerts.elementAt(i);
            }
            AbstractCertificateHandler[] fromKeyStores = new AbstractCertificateHandler[keyStoreCerts.size()];
            keyStoreCerts.copyInto(fromKeyStores);
            System.arraycopy(fromKeyStores, 0, certArray, foundCerts.size(), keyStoreCerts.size());
        } else {
            certArray = new AbstractCertificateHandler[keyStoreCerts.size()];
            keyStoreCerts.copyInto(certArray);
            // not 1.1
//            certArray = (AbstractCertificateHandler[]) (keyStoreCerts.toArray(new AbstractCertificateHandler[0]));
        }
        view.setCertificateList(certArray);
    }

    private void btnPushed() {
        ((CallBackHandler) view).clearUserMessage(true);
        // "hent"-knappen blev trykket
        FileDialog dlg = ra.getFileDialog(getFrame(), ResourceManager.getString("FIND_CERTIFICATE"), FileDialog.LOAD);
        dlg.setFilenameFilter(new FilenameFilter() {
            public boolean accept(File dir, String n) {
                for (int i = 0; i < keyStores.size(); i++) {
                    KeyStoreHandler keyStore = (KeyStoreHandler) keyStores.elementAt(i);
                    FilenameFilter filter = keyStore.getFilenameFilter();
                    if (filter != null && filter.accept(dir, n)) {
                        return true;
                    }
                }
                return false;
            }
        });
        dlg.setModal(true);
        ra.showFileDialog(dlg);
//        dlg.show();
        String certFileName = dlg.getFile();
        String certDirName = dlg.getDirectory();
        if (dlg.getFile() == null) {
            if (FileLog.isDebugEnabled())
                FileLog.debug("dialog cancel");
            return;
        }
        // we have a file - check who can handle it - if anybody
        File certFile = new File(certDirName, certFileName);
        AbstractCertificateHandler cert = null;
        for (int i = 0; i < keyStores.size(); i++) {
            KeyStoreHandler handler = (KeyStoreHandler) keyStores.elementAt(i);
            cert = handler.getCertificateFromFile(certFile);
            if (cert != null) {
                break;
            }
        }
        if (cert != null) {
            // check if it is equal to any of the old ones
            // in that case: replace the old one as the user may want an overwritten one.
            boolean replaces = false;
            // check keystoreCerts first
            for (int i = 0; i < keyStoreCerts.size(); i++) {
                AbstractCertificateHandler anotherCert = (AbstractCertificateHandler) keyStoreCerts.elementAt(i);
                if (anotherCert.equals(cert)){
                    keyStoreCerts.setElementAt(cert, i);
                    replaces = true;
                    FileLog.debug("Cert replaces another keystoreCert at index "+i);
                    break;
                }
            }
            if (!replaces){
                // check also foundCerts
                for (int i = 0; i < foundCerts.size(); i++) {
                    AbstractCertificateHandler anotherCert = (AbstractCertificateHandler) foundCerts.elementAt(i);
                    if (anotherCert.equals(cert)){
                        foundCerts.setElementAt(cert, i);
                        replaces = true;
                        FileLog.debug("Cert replaces another foundCert at index "+i);
                        break;
                    }
                }
            }
            if (!replaces){
                FileLog.debug("Cert doesn't replace");
                foundCerts.addElement(cert);
            }
            setCertificates();
        } else {
            // tell user it didn't work - nobody can read it
            ((CallBackHandler) view).userMessage(certFile.getName(), ResourceManager.getString("DLG_ERROR_TEXT_GENERAL_LOAD_FROM_FILE_FAILURE"), true);
        }
    }

    private Frame getFrame() {
        // todo: make nicer
        return ((AbstractApplet) view).getFrame();
    }

    public void action(AbstractCertificateHandler cert) {
        // show the file-dialog relevant
        //To change body of implemented methods use File | Settings | File Templates.
    }

    private int getSecondaryNameLength(){
        String s1 = ResourceManager.getString("CERT_EMPLOYEE");
        String s2 = ResourceManager.getString("CERT_PERSONAL");
        return Math.max(s1.length(), s2.length());
    }
    private String getStringBetween(String prefix, String postfix, String source){
        int start = source.indexOf(prefix);
        if (start == -1){
            FileLog.error("Error finding prefix "+prefix+" in source "+source);
            return "?";
        }
        String rest = source.substring(start+prefix.length());
        int end = rest.indexOf(postfix);
        if (end == -1){
            FileLog.error("Error finding postfix "+postfix+" in rest "+rest);
            return "?";
        }
        return rest.substring(0, end);
    }
    /**
     * Tries to make a friendly name from the certificate.
     * If it fails to do so, it will attempt to generate a name.
     *
     * @param info
     * @param maxWidth
     * @return the constructed name
     */
    public String getCertificateDisplayName(CertificateInfo info, int maxWidth) {
        String name = "";
        String primaryName = "", secondaryName = "";

        if (info == null)
            throw new IllegalArgumentException("cert");

        if (!info.isInfoAvailable())
            return info.getUserfriendlyName();

        // todo: re-insert the expression evaluator etc
        primaryName = info.getUserfriendlyName();
        try {
            // check if it is personal or employee cert
            String subjectDN = info.getSubjectDN().getName();

            if (subjectDN.indexOf("-RID:") != -1){
                // new version: make it "cvr:12341234"
                secondaryName = "cvr:"+getStringBetween("=CVR:","-RID:",info.getSubjectDN().getName());
            } else if (subjectDN.indexOf("-UID:") != -1){
                secondaryName = "vcert:"+getStringBetween("=CVR:","-UID:",info.getSubjectDN().getName());
            } else if (subjectDN.indexOf("PID:") != -1){
                secondaryName = ResourceManager.getString("CERT_PERSONAL");
            } else {
                secondaryName = info.getSerialNumber().toString(16);
                // cut it down if longer than others:
                if (secondaryName.length() > getSecondaryNameLength()){
                    secondaryName = secondaryName.substring(0, getSecondaryNameLength()-2)+"..";
                }
            }
        } catch (Exception e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        name = primaryName;
        if (name.equals("")) {
            name = secondaryName;

            if (name.equals("")) {
                try {
                    name = info.getSubjectDN().toString();
                } catch (Exception e2) {
                    e2.printStackTrace();
                    name = "(unknown)";
                }
            }

            return name;
        }

        if (!secondaryName.equals("")){
            // Abbreviate the name if needed
            if (name.length() + secondaryName.length() + 3 > maxWidth){
                // abbrev name first:
                int maxNameLength = maxWidth - (secondaryName.length() + 3);
                name = name.substring(0, Math.min(name.length(), Math.max(0, maxNameLength - 2)))+"..";
            }
            int spaces = maxWidth - (name.length() + secondaryName.length() + 3);
            String spaceStringSrc = "                                                                          ";
            String spaceString = spaceStringSrc.substring(0, Math.min(spaceStringSrc.length(), Math.max(0, spaces)));
            name += spaceString +" (" + secondaryName + ")";
        }

        return name;
    }
}
