/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: OS.java,v 1.5 2006/08/12 19:02:39 cara Exp $ */

package org.openoces.opensign.client.applet;

/**
 * This class implements OS specifics
 *
 * @author Jens Bo Friis  <jbf@it-practice.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.appletsupport.AppletSupport;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.crypto.CryptoSupport;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.IOException;

public class OS {
    private static CryptoSupport cryptoSupport;
    private static AppletSupport appletSupport;
    public static AppletSupport getAppletSupport(){
        return appletSupport;
    }
    public static void setAppletSupport(AppletSupport support){
    	appletSupport = support;
    }
    public static CryptoSupport getCryptoSupport(){
        return cryptoSupport;
    }
    public static void setCryptoSupport(CryptoSupport support){
        cryptoSupport = support;
    }
    private static RestrictedAction ra;
    // todo: deprecate! possible securityrisk
    public static RestrictedAction getRestrictedAction(){
        return ra;
    }
    // todo: unmake public - only for debugging
    public static void setRestrictedAction(RestrictedAction ra){
        OS.ra = ra;
        init();
    }

    public static boolean isMsJVM(){
        return (getProperty("java.vendor").indexOf("Microsoft") == 0);
    }

	public static boolean javaplugin = false;
	private static void init() {
		String pluginVersion = getProperty("javaplugin.version"); // prv: to make ms jvm dyn load work!
		if (pluginVersion != null)
			javaplugin = true;
		FileLog.debug((javaplugin) ? ("Using Java2 security") : ("Using Java1 secrity"));
	}

	public static boolean isOSLinux() {
		if (getProperty("os.name").toLowerCase().startsWith("linux"))
			return true;
		return false;
	}

	public static boolean isOSWindows() {
		if (getProperty("os.name").toLowerCase().startsWith("win"))
			return true;
		return false;
	}

//	public static String getUserHome() {
//		String home = getProperty("user.home");
//		if (isOSWindows()) {
//			// find install drive
//			String path = home.substring(0, home.indexOf("\\"));
//			return path.replace('\\', '/');
//		} else
//			return home;
//	}
//
	public static String getUserName() {
		return getProperty("user.name");
	}

	public static String getFileSeperator() {
		if (isOSWindows())
			return getProperty("file.separator").replace('\\', '/');
		return getProperty("file.separator");
	}
	public static String getTempDir() {
		return getProperty("tmp.dir");
	}


	/**
	 * Read system property secured by a security manager. The method mandles the 3 different security managers:
	 * <ol>
	 * <li>Microsoft native JVM using com. ms.security.*
	 * <li>Netscape or Java 1 security manager using netscape.security.*
	 * <li>Plugin or Java 2 security managers using AccessController.doPriviledge() 
	 * </ol> 
	 * @param key
	 * @return
	 */

    public static String getProperty(final String key) {
        return ra.getProperty(key);
//        if (javaplugin) {
//			try {
//				java.security.AccessController.doPrivileged(new java.security.PrivilegedAction() {
//					public Object run() {
//						return System.getProperty(key);
//					}
//				});
//			} catch (Throwable t) {
//			}
//
//        } else {
//            try {
//                netscape.security.PrivilegeManager.enablePrivilege("UniversalPropertyRead");
//            } catch (Throwable t) {
//            };
//            try {
//                com. ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PROPERTY);
//            } catch (Throwable t) {
//            };
//        }
//        return System.getProperty(key);
    }

//	public static Properties getProperties() {
//		if (javaplugin) {
////			try {
////				java.security.AccessController.doPrivileged(new java.security.PrivilegedAction() {
////					public Object run() {
////						return System.getProperties();
////					}
////				});
////			} catch (Throwable t) {
////			}
//            // todo: put back in prv
//		} else {
//			try {
//				netscape.security.PrivilegeManager.enablePrivilege("UniversalPropertyRead");
//			} catch (Throwable t) {
//			};
//			try {
//				com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PROPERTY);
//			} catch (Throwable t) {
//			};
//		}
//		return System.getProperties();
//	}

//    private static void permit(){
//        if (!OS.javaplugin) {
//            try {
//                netscape.security.PrivilegeManager.enablePrivilege("UniversalFileAccess");
//                netscape.security.PrivilegeManager.enablePrivilege("UniversalTopLevelWindow");
//            } catch (Throwable t) {
//            };
//            try {
//                com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
//            } catch (Throwable t) {
//            };
//        }
//    }

    public static void copyFile(File srcFile, File destFile) throws IOException {
        BufferedInputStream in = new BufferedInputStream(ra.getFileInputStream(srcFile));
        BufferedOutputStream out = new BufferedOutputStream(ra.getFileOutputStream(destFile));
        int c;

        while ((c = in.read()) != -1)
            out.write(c);

        in.close();
        out.close();
    }

}
