/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: OcesAppletUtils.java,v 1.9 2006/08/12 19:02:39 cara Exp $ */

package org.openoces.opensign.client.applet;

/**
 * This class contains code for handling logon through
 * a simple URLConnection to the server instead of logging
 * on through JS (LiveConnect)
 *
 * @author Jesper Nielsen  <?@it-practice.dk>
 */

import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.applet.Applet;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

/**
 * This class contains code for handling logon through
 * a simple URLConnection to the server instead of logging
 * on through JS (LiveConnect)
 *
 *
 * @version $Revision: 1.9 $
 *
 * <pre>
 * OCES certificate logon and digital signature
 * Copyright(c) 2002,2003, IT Practice A/S
 *
 * web page: http://it-practice.dk
 *
 * This is open source software, placed under the terms of the
 * GNU LESSER GENERAL PUBLIC LICENSE.
 * Please see: http://www.gnu.org/licenses/lgpl.txt or
 * opensign.license for details.
 *
 * Permission to use, copy, modify, and distribute this software
 * for any purpose and without fee is hereby granted, provided
 * that the above copyright notices appear in all copies and that
 * both the copyright notice and this permission notice appear in
 * supporting documentation.
 * </pre>
 */
public class OcesAppletUtils {
    static Applet applet;

    public static void setApplet(Applet app) {
        applet = app;
    }

    public static void printInitInformation() {
        FileLog.debug(
                "opensign.doappletrequestonmac: "
                + applet.getParameter("opensign.doappletrequestonmac"));
        FileLog.debug(
                "opensign.doappletrequest: "
                + applet.getParameter("opensign.doappletrequest"));
        FileLog.debug(
                "opensign.verifieruri: "
                + applet.getParameter("opensign.verifieruri"));
        FileLog.debug(
                "opensign.alerturi: " + applet.getParameter("opensign.alerturi"));
        FileLog.debug(
                "opensign.erroruri: " + applet.getParameter("opensign.erroruri"));
        FileLog.debug(
                "opensign.canceluri: " + applet.getParameter("opensign.canceluri"));
        FileLog.debug(
                "opensign.verifiedokuri: "
                + applet.getParameter("opensign.verifiedokuri"));
        FileLog.debug(
                "opensign.verifiederroruri: "
                + applet.getParameter("opensign.verifiederroruri"));
        FileLog.debug(
                "opensign.cookiecount: "
                + applet.getParameter("opensign.cookiecount"));
        FileLog.debug(
                "opensign.formdata.count: "
                + applet.getParameter("opensign.formdata.count"));
    }

    /**
     *
     * @return In indicator
     */
    public static boolean doAppletRequest() {
        String doAutoMac = applet.getParameter("opensign.doappletrequestonmac");
        if (doAutoMac != null
                && doAutoMac.equalsIgnoreCase("true")
                && isOSMac()) {
            FileLog.debug(
                    "doAppletRequest() called - returning true (auto on mac)");
            return true;
        }

        String doAppletRequest =
                applet.getParameter("opensign.doappletrequest");
        if (doAppletRequest != null
                && doAppletRequest.equalsIgnoreCase("true")) {
            FileLog.debug(
                    "doAppletRequest() called - returning true (doappletrequest flag set)");
            return true;
        }

        FileLog.debug("doAppletRequest() called - returning false");
        return false;
    }

    public static String getFunctionResult(String function, String args[]) {
        if (OcesAppletUtils.doAppletRequest()) {
            function = function.toLowerCase();

            if (function.indexOf("alert") >= 0)
                return "alert";
            else if (function.indexOf("error") >= 0)
                return "error";
            else if (function.indexOf("cancel") >= 0)
                return "cancel";
            else if (function.indexOf("ok") >= 0)
                return "ok";
        }

        return null;
    }

    public static void doAppletRequest(String message, String result) {
        FileLog.error("not used anymore");
    }
    private static final int chunkSize = 1000;
    private static void writeChunked(OutputStream os, byte [] bytes) throws IOException {
        int off = 0;
        while (off < bytes.length){
            int len = Math.min(chunkSize, bytes.length-off);
            os.write(bytes, off, len);
            off += len;
            os.flush();
        }
    }

    public static void doAppletRequest(RestrictedAction ra, String message, String result) {
        try {
            FileLog.debug(
                    "doAppletRequest called with result '"
                    + result
                    + "' and message: '"
                    + message
                    + "'");
            String vurl = getDocBase();
            String uri = applet.getParameter("opensign.verifieruri");

            if (result.equalsIgnoreCase("ok") ||
                    result.equalsIgnoreCase("error") ||
                    result.equalsIgnoreCase("alert")) {
                if (uri == null)
                    uri = "";
                if (!uri.startsWith("/"))
                    vurl = vurl + "/";
                vurl = vurl + uri;

                FileLog.debug("Sending sign/logon request to: " + vurl);

                URLConnection con = new URL(vurl).openConnection();

                FileLog.debug("Url connection class: " + con.getClass());

                ByteArrayOutputStream bo = new ByteArrayOutputStream();
                StringBuffer output = new StringBuffer(2048);

                String mname = applet.getParameter("opensign.message.name");
                if (mname == null || mname.length() <= 0)
                    mname = "message";

                String rname = applet.getParameter("opensign.result.name");
                if (rname == null || rname.length() <= 0)
                    rname = "result";

                if (message != null) {
                    output.append(mname);
                    output.append("=");

                    char[] chars = message.toCharArray();

                    for (int i = 0; i < chars.length; i++) {
                        if (chars[i] >= 'a'
                                && chars[i] <= 'z'
                                || chars[i] >= 'A'
                                && chars[i] <= 'Z'
                                || chars[i] >= '0'
                                && chars[i] <= '9') {
                            output.append(chars[i]);
                        } else {
                            output.append("%");
                            String s = Integer.toHexString((int) chars[i]);
                            s = s.toUpperCase();
                            if (s.length() < 2)
                                output.append("0");
                            output.append(s);
                        }
                    }
                }

                if (result != null) {
                    if (message != null)
                        output.append("&");
                    output.append(rname);
                    output.append("=");
                    output.append(result);
                }

                int ffcount = getCustomFormFieldCount();
                for (int i = 0; i < ffcount; i++) {
                    output.append("&");
                    output.append(getCustomFormField(i + 1));
                }

                bo.write(output.toString().getBytes());
                byte[] bytes = bo.toByteArray();

                int responsecode;

                /* java vm specific branch
                   unfortunately not all vms return an object of the same class, so we have to deal with all
                   possibilities
                */
                if (con instanceof HttpURLConnection) {
                    FileLog.debug("Using HttpUrlConnection to do the POST");
                    HttpURLConnection httpcon = (HttpURLConnection) con;
                    httpcon.setRequestMethod("POST");
                    httpcon.setDoOutput(true);
                    httpcon.setUseCaches(false);
                    if (!HttpURLConnection.getFollowRedirects()){
                        FileLog.debug("getFollowRedirects is false - trying to set it then..............");
                        // todo: put back in - if necessary
                        try {
                            HttpURLConnection.setFollowRedirects(true);
                        } catch (SecurityException e){
                            FileLog.info("Couldn't setFollowRedirects - but is it needed? (if so: implement in RestrictedAction");
                        }
                    }
                    httpcon.setRequestProperty("Content-Length", "" + bytes.length);
                    if (getCookieString() != null)
                        httpcon.setRequestProperty("Cookie", getCookieString());
                    FileLog.debug(">>>>>>>>>>>>>>>>>>>> using RestrictedAction connect:");
                    ra.connect(httpcon);
                    writeChunked(httpcon.getOutputStream(), bytes);
                    httpcon.getOutputStream().close();
                    responsecode = ra.getResponseCode(httpcon);
                    FileLog.debug("got response code:"+responsecode);
//                    FileLog.debug(
//                            "Got the following respons to the sign/logn request: "
//                            + httpcon.getResponseCode()
//                            + " "
//                            + httpcon.getResponseMessage());
                } else {
                    FileLog.debug("Using SOMETHING ELSE to do the POST");
                    con.setDoOutput(true);
                    con.setDoInput(true);
                    con.setUseCaches(false);

                    con.setRequestProperty("Content-Length", "" + bytes.length);
                    if (getCookieString() != null)
                        con.setRequestProperty("Cookie", getCookieString());

                    ra.connect(con);

                    OutputStream outStream = con.getOutputStream();
                    writeChunked(outStream, bytes);
                    outStream.close();

                    String httpStatusLine = con.getHeaderField(0);

                    FileLog.debug(
                            "Got the following status line to the sign/logon request: "
                            + httpStatusLine);

                    responsecode = getHttpStatusCode(httpStatusLine);
                }

                if (responsecode == 200) {
                    uri = applet.getParameter("opensign.verifiedokuri");
                } else {
                    uri = applet.getParameter("opensign.verifiederroruri");
                }
            }

            if (result.equalsIgnoreCase("error"))
                uri = applet.getParameter("opensign.erroruri");
            if (result.equalsIgnoreCase("alert"))
                uri = applet.getParameter("opensign.alerturi");
            if (result.equalsIgnoreCase("cancel"))
                uri = applet.getParameter("opensign.canceluri");

            vurl = getDocBase();
            if (uri == null)
                uri = "";
            if (!uri.startsWith("/"))
                vurl = vurl + "/";
            vurl = vurl + uri;
            if (uri.startsWith("https://")){
                vurl = uri;
            }

            FileLog.debug("Setting new document base: " + vurl);
            applet.getAppletContext().showDocument(new URL(vurl));
        } catch (MalformedURLException urle) {
            FileLog.error("malformed applet request url ", urle);
        } catch (IOException ioe) {
            FileLog.error("io errror occured ", ioe);
        }
    }

    /**
     * Returns the http status code given a http status line
     * @param statusLine
     * @return The status code or -1 if the status line is not valid
     */
    private static int getHttpStatusCode(String statusLine) {
        int sp0, sp1, httpStatusCode = -1;

        sp0 = statusLine.indexOf(' ');
        sp1 = statusLine.indexOf(' ', sp0 + 1);


        if (sp0 != -1) {
            try {
                httpStatusCode = (sp1 == -1) ? new Integer(statusLine.substring(sp0 + 1)).intValue() : new Integer(statusLine.substring(sp0 + 1, sp1)).intValue();
            } catch (Exception e) {
            }
        }

        return httpStatusCode;
    }

    /**
     * Returns the count of the custom form entries given as
     * applet parametes.
     * @return -1 if count is not defined or not a number. The number in other cases
     */
    private static int getCustomFormFieldCount() {
        String scount = applet.getParameter("opensign.formdata.count");
        int count = 0;

        FileLog.debug("Looked up this formdata count: " + scount);

        if (scount == null)
            return -1;

        try {
            count = Integer.parseInt(scount.trim());
        } catch (Exception e) {
            FileLog.error(
                    "The argument opensign.formdata.count is not a number '"
                    + scount
                    + "'");
            return -1;
        }

        return count;
    }

    /**
     * combines parameters to form a custom form field name/value pair
     * @param index The index of the form field value in the applet parameters
     * @return The form field string
     */
    private static String getCustomFormField(int index) {
        String name =
                applet.getParameter("opensign.formdata." + index + ".name");
        String value =
                applet.getParameter("opensign.formdata." + index + ".value");
        FileLog.debug(
                "Looked up formfield for index "
                + index
                + " with name/value: "
                + name
                + "="
                + value);
        return name + "=" + value;
    }

    /**
     * Generates a complete cookie string from a the applet paramets.
     * The method will return <code>null</code> if the cookie count is
     * less or equal to 0, not a number or not existing.
     * @return The generated cookie string
     */
    private static String getCookieString() {
        String scount = applet.getParameter("opensign.cookiecount");
        int count = 0;

        FileLog.debug("Looked this cookiecount: " + scount);

        if (scount == null)
            return null;

        try {
            count = Integer.parseInt(scount.trim());
        } catch (Exception e) {
            FileLog.error(
                    "The argument opensign.cookiecount is not a number '"
                    + scount
                    + "'");
            return null;
        }

        if (count == 0)
            return null;

        String cookieString = "";
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                cookieString = cookieString + "; ";
            }
            cookieString = cookieString + getClientCookie(i + 1);
        }

        return cookieString;
    }

    /**
     * combines parameters to form a client cookie value
     * @param index The index of the client cookie value in the applet parameters
     * @return The client cookie value
     */
    private static String getClientCookie(int index) {
        String name = applet.getParameter("opensign.cookie." + index + ".name");
        String value =
                applet.getParameter("opensign.cookie." + index + ".value");
        FileLog.debug(
                "Looked up cookie on index "
                + index
                + " with name/value: "
                + name
                + "="
                + value);
        return name + "=" + value;
    }

    public static boolean isOSMac() {
        if (OS.getProperty("os.name").toLowerCase().indexOf("mac") >= 0)
            return true;
        return false;
    }

    public static String getDocBase() {
        URL docbase = applet.getDocumentBase();

        FileLog.debug("document base elements reported by applet:");
        FileLog.debug("host:"+docbase.getHost());
        FileLog.debug("protocol:"+docbase.getProtocol());
        FileLog.debug("port:"+docbase.getPort());

        String vurl = docbase.getProtocol() + "://" + docbase.getHost();
        if (docbase.getProtocol().equalsIgnoreCase("http")
                && docbase.getPort() != 80
                && docbase.getPort() != -1
                || docbase.getProtocol().equalsIgnoreCase("https")
                && docbase.getPort() != 443
                && docbase.getPort() != -1) {
            vurl = vurl + ":";
            vurl = vurl + docbase.getPort();
        }
        return vurl;
    }
}
