/*
<<<<<<< Sign.java
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: Sign.java,v 1.34 2006/10/21 13:57:59 cara Exp $ */

package org.openoces.opensign.client.applet;

/**
 * This class implements the sign applet
 *
 * @author Jens Bo Friis  <jbf@it-practice.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.exceptions.InputDataError;
import org.openoces.opensign.exceptions.UserCancel;
import org.openoces.opensign.utils.Base64;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.utils.Version;
import org.openoces.opensign.xml.xmldsig.SignatureGenerator;
import org.openoces.opensign.appletsupport.AppletSupport;
import org.openoces.opensign.appletsupport.AttachmentSupport;
import org.openoces.opensign.appletsupport.Attachment;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.*;
import java.util.Date;
import java.util.Properties;
import java.util.StringTokenizer;

public class Sign extends AppletLayout {
    private TextArea taSignText;
    private String attachmentParam;

    public StringBuffer getSignText() {
        return paramReader.getSigntext();
    }

    // motet to here
    public Sign() {
        super();
        FileLog.debug("Sign constructor invoked " + this.getClass().getClassLoader());
        this.setName(ResourceManager.getString("TITLE_SIGN") + " v." + Version.getBuildVersion());
    }

    public String getAppletInfo() {
        return "OpenSign";
    }

    protected Button getOkButton() {
        bOk.setLabel(ResourceManager.getString("OK_SIGN_MODERN"));
        return bOk;
    }

    protected Panel getHeaderPanel() {
        Panel p = new Panel();
        p.add(new Label(ResourceManager.getString("HEADER_SIGNTEXT")));
        return p;
    }

    private AttachmentSupport attachmentSupport;

    private void createAttachmentServer(String b64param) {
        AppletSupport sup = OS.getAppletSupport();
        if (sup != null && sup instanceof AttachmentSupport) {
            attachmentSupport = (AttachmentSupport) sup;
            attachmentSupport.init(ra, this, b64param);
        }
    }

    protected void showAttachments() {
        if (attachmentSupport != null) {
            attachmentSupport.show(ra, getFrame());
        }
    }

    protected Panel getAttachmentPanel() {
        if (attachmentSupport == null) return null;
        Button viewAttachmentButton = new Button(ResourceManager.getString("ATTACHMENT_LIST"));
        viewAttachmentButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                showAttachments();
            }
        });
        Panel p = new Panel();
        p.add(viewAttachmentButton);


        return p;
    }

    protected void doClassicLayout(boolean fixedTextHeight, boolean fixedTextWidth) throws InputDataError {
        setLayout(new GridBagLayout());
        GridBagConstraints c = getGridBagConstraints();

        GridBagConstraints textConstraints = (GridBagConstraints) c.clone();
        textConstraints.gridwidth = 2;

        if (!fixedTextWidth && !fixedTextHeight) {
            textConstraints.fill = GridBagConstraints.BOTH;
            textConstraints.weightx = 1;
            textConstraints.weighty = 1;
        } else if (!fixedTextWidth && fixedTextHeight) {
            textConstraints.fill = GridBagConstraints.HORIZONTAL;
            textConstraints.weightx = 1;
            textConstraints.weighty = 0;
        } else if (fixedTextWidth && !fixedTextHeight) {
            textConstraints.fill = GridBagConstraints.VERTICAL;
            textConstraints.weightx = 0;
            textConstraints.weighty = 1;
        } else if (fixedTextWidth && fixedTextHeight) {
            textConstraints.fill = GridBagConstraints.NONE;
            textConstraints.weightx = 0;
            textConstraints.weighty = 0;
        }
        add(taSignText, textConstraints);

        c.gridy++;
        add(lCertList, c);
        c.gridx++;
        add(chCertList, c);

        if (tfSocialSecurityNumber != null) {
            c.gridx = 0;
            c.gridy++;
            add(lSocialSecurityNumber, c);
            c.gridx++;
            add(tfSocialSecurityNumber, c);
        }

        if (tfOptionalEntry != null) {
            c.gridx = 0;
            c.gridy++;
            add(lOptionalEntry, c);
            c.gridx++;
            add(tfOptionalEntry, c);
        }

        c.gridx = 0;
        c.gridy++;
        add(lPassword, c);
        c.gridx++;
        add(tfPassword, c);
        tfPassword.requestFocus();

        c.gridx = 0;
        c.gridy++;
        c.gridwidth = 2;

        Panel pnl = new Panel();
        pnl.add(bOk);
        pnl.add(bCancel);
        Panel attachmentPanel = getAttachmentPanel();
        if (attachmentPanel != null){
            pnl.add(attachmentPanel);
        }
        pnl.add(bFunctions);

        add(pnl, c);
    }

    protected ParamReader createParamReader() {
        // find out if we have xml input in "signtext" and set paramreader if so

        String b64signText = getParameter("signtext");
        String explicitInputStyle = getParameter("inputstyle");

        if (b64signText != null) {
            String signText;
            byte[] ba = Base64.decode(b64signText.getBytes());
            try {
                signText = new String(ba, "UTF8");
            } catch (UnsupportedEncodingException e) {
                FileLog.fatal(e.toString());
                return null;
            }

            if (explicitInputStyle != null) {
                if ("default".equals(explicitInputStyle)) {
                    return new DefaultParamReader(this);
                } else if ("virk".equals(explicitInputStyle)) {
                    return new AppletArguments(this);
                } else {
                    return new DefaultParamReader(this);
                }
            } else {
                if (signText.indexOf("<?xml") == 0) {
                    return new AppletArguments(this);// could be named XmlParamReader..
                } else {
                    return new DefaultParamReader(this);
                }
            }
        } else {
            return new DefaultParamReader(this);
        }
    }

    public void initialize() throws IOException, InputDataError {
        attachmentParam = paramReader.getParameter("attachments");
        if (attachmentParam != null) {
            this.createAttachmentServer(attachmentParam);
        }

        String wrap = paramReader.getParameter("wrap");
        if ("true".equals(wrap)) {
            taSignText = new TextArea(null, 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);
        } else {
            taSignText = new TextArea();
        }

        String fontname = paramReader.getParameter("signtextfontname");
        if (fontname == null)
            fontname = "Courier";

        String fontsize = paramReader.getParameter("signtextfontsize");
        if (fontsize == null) {
            fontsize = "12";
        }

        int i = Integer.parseInt(fontsize);
        if (i >= 6 && i <= 32) {
            Font font = new Font(fontname, Font.PLAIN, Integer.parseInt(fontsize));
            taSignText.setFont(font);
        } else {
            FileLog.fatal("Illegal font size specified: " + i);
            throw new org.openoces.opensign.exceptions.InputDataError();
        }

        String signtextcols = paramReader.getParameter("signtextcols");
        if (signtextcols == null)
            signtextcols = "80";
        int signtextcolsValue = Integer.parseInt(signtextcols);
        boolean fixedTextWidth = (signtextcolsValue >= 5 && signtextcolsValue <= 120);
        if (fixedTextWidth) {
            taSignText.setColumns(signtextcolsValue);
        }

        String signtextrows = paramReader.getParameter("signtextrows");
        if (signtextrows == null)
            signtextrows = "15";
        int signtextrowsValue = Integer.parseInt(signtextrows);

        boolean fixedTextHeight = (signtextrowsValue >= 5 && signtextrowsValue <= 50);
        if (fixedTextHeight) {
            taSignText.setRows(signtextrowsValue);
        }

        taSignText.setEditable(false);
        taSignText.setText(getSignText().toString());

        checkBkgColor(taSignText);

        if (classicStyle()) {
            FileLog.debug("initialize Sign to classic style");
            doClassicLayout(fixedTextHeight, fixedTextWidth);
        } else {
            FileLog.debug("initialize Sign to modern style");
            doModernLayout(taSignText, fixedTextHeight, fixedTextWidth);
        }

        ActionListener okAction = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                enableDialog(false);

                FileLog.info("Timing: OK clicked: " + new Date());

                // check for a valid state of attachments if they exists ..
                if (attachmentSupport == null || attachmentSupport.isSigningOK()) {
                    FileLog.info("Timing: OK clicked: " + new Date());
                    char[] password = tfPassword.getText().trim().toCharArray();
                    clearPassword();

                    FileLog.debug("ok pressed");
                    AbstractCertificateHandler selected = getSelectedCertificate();
                    if (selected != null) {
                        try {

/* check if the certificate is filtered */
// todo: find out what to do here
//                        if (selected instanceof Pkcs12CertificateHandler){
//                            // check password - indirectly - and throw exception if it doesn't check out
//                            selected.getCertificateChain(password);
//                            if (!CertificateFilter.acceptByAll(selected)){
//                                FileLog.error("Certificate filtered out");
//                                alert(ResourceManager.getString("ERR_WRONG_CERTIFICATE"));
//                                return;
//                            }
//                        }
/* end check certificate */

                            Properties p = getSignProperties();
                            if (!OS.isMsJVM()) {
                                addSunLayoutSettings(p);
                            }
                            addCommonLayoutSettings(p);

                            String socialSecurityNumber = null;
                            String optionalEntry = null;

                            if (tfSocialSecurityNumber != null) {
                                socialSecurityNumber = tfSocialSecurityNumber.getText().trim();
                                if (socialSecurityNumber.equals(""))
                                    socialSecurityNumber = null;
                            }

                            if (tfOptionalEntry != null && (!tfOptionalEntry.getText().trim().equals(""))) {
                                optionalEntry = tfOptionalEntry.getText().trim();
                            }
                            FileLog.info("Timing: before generating signature: " + new Date());

                            // new signature generation:
                            Properties visibleProps = new Properties();
                            if (socialSecurityNumber != null) {
                                visibleProps.put(SignatureGenerator.SOCIALSECURITYNUMBER, socialSecurityNumber);
                            }
                            if (optionalEntry != null) {
                                visibleProps.put(SignatureGenerator.OPTIONALFIELD2, optionalEntry);
                            }
                            addEnvironmentProperties(p);

                            Attachment[] attachments = null;
                            if (attachmentSupport != null) {
                                // todo: find out if all input parameters should be added here - and various other methods maybe changed accordingly.
                                addInputAttachmentProperties(p);
                                attachments = attachmentSupport.getAttachments();
                            }
                            SignatureGenerator sigGen = paramReader.createSignatureGenerator();
                            StringBuffer signature = sigGen.sign(selected, password, getSignText(), visibleProps, p, attachments);
                            FileLog.debug(signature.toString());

                            handleOk("onSignOK", signature);
                        } catch (InputDataError ex) {
                            // Tell the user but let the user try again
                            userMessage("", ex.getMessage(), true);
                        } catch (UserCancel ex) {
                            userMessage("", ex.getMessage(), true);
                        } catch (IOException ex) {
                            FileLog.error("could not create signature, possibly wrong password", ex);
                            handleError("onSignError", "possibly wrong password");
                        } catch (Exception ex) {
                            FileLog.error("could not create signature", ex);
                            handleError("onSignError", "possibly wrong password");
                        }
                    }
                }
                enableDialog(true);
            }
        };

        bOk.addActionListener(okAction);
        tfPassword.addActionListener(okAction);

        if (tfOptionalEntry != null) {
            tfOptionalEntry.addActionListener(okAction);
        }

        if (tfSocialSecurityNumber != null) {
            tfSocialSecurityNumber.addActionListener(okAction);
        }

//		ActionListener showFunctions = new ActionListener() {
//			public void actionPerformed(ActionEvent e) {
//                updateFunctionsPopup();
//				functionsPopup.show((Button) e.getSource(), 40, 10);
//			}
//		};
//
//		bFunctions.addActionListener(showFunctions);

        bCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                handleCancel();
            }
        });

        KeyListener kl = new KeyListener() {
            public void keyTyped(KeyEvent evt) {
            }

            public void keyPressed(KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
                    clearPassword();
                    handleCancel();
                }
            }

            public void keyReleased(KeyEvent evt) {
            }

        };

        bOk.addKeyListener(kl);
        bCancel.addKeyListener(kl);
        bFunctions.addKeyListener(kl);
        chCertList.addKeyListener(kl);
        tfPassword.addKeyListener(kl);

        if (tfOptionalEntry != null) {
            tfOptionalEntry.addKeyListener(kl);
        }

        if (tfSocialSecurityNumber != null) {
            tfSocialSecurityNumber.addKeyListener(kl);
        }
        // Add data
        refreshCertificates(null);

        tfPassword.requestFocus();
    }

    /**
     * As the MS and SUN JVM differs for this we set it explicit. For MS the background color is the same as that of the
     * applet where it for SUN has another background color.
     * <p/>
     * If
     *
     * @param taSignText
     */
    private void checkBkgColor(TextArea taSignText) {
        String textBgColor = paramReader.getParameter("signtextbgcolor");
        if (textBgColor == null) {
            taSignText.setBackground(this.getBackground());
        } else {
            // refactor, same code as in AbstractApplet
            StringTokenizer st = new StringTokenizer(textBgColor, ",;");
            String red = st.nextToken();
            String green = st.nextToken();
            String blue = st.nextToken();
            taSignText.setBackground(new Color(Integer.parseInt(red), Integer.parseInt(green), Integer.parseInt(blue)));
        }
    }

    public void enableDialog(boolean enable) {
        if (!classicStyle() && userMessageLabel != null) {
            if (enable) {
                clearUserMessage(false);
            } else {
                userMessage("", ResourceManager.getString("PLEASE_WAIT"), false);
            }
        }
        bOk.setEnabled(enable);
        tfPassword.setEnabled(enable);
    }

    protected void addSunLayoutSettings(Properties p) {
        try {
            p.put("openoces_opensign_layout_signtext_fontname", taSignText.getFont().getFontName());
        } catch (java.lang.NoSuchMethodError e) {
            FileLog.debug("Applet signtext font name not available");
        }
    }

    protected void addCommonLayoutSettings(Properties p) {
        /* add properties concerning the applet layout.
           if the visible signed text is somehow unreadable we may be able to detect
           it using these values
        */
        super.addLayoutSettings(p);
        try {
            p.put("openoces_opensign_layout_signtext_fontsize", "" + taSignText.getFont().getSize());
        } catch (java.lang.NoSuchMethodError e) {
            FileLog.debug("Applet signtext font size not available");
        }
        try {
            p.put("openoces_opensign_layout_signtext_columns", "" + taSignText.getColumns());
        } catch (java.lang.NoSuchMethodError e) {
            FileLog.debug("Applet signtext number of columns not available");
        }
        try {
            p.put("openoces_opensign_layout_signtext_rows", "" + taSignText.getRows());
        } catch (java.lang.NoSuchMethodError e) {
            FileLog.debug("Applet signtext number of rows not available");
        }
    }

    protected void addInputAttachmentProperties(Properties p) {
        p.put("opensign_openoces_attachment_input", attachmentParam);
    }

    protected void addToBeSignedAttachmentProperties(Properties p) {
        // add something to identify the attachments seen along with a digest to sign
        if (attachmentSupport == null) return;
        Attachment [] attachments = attachmentSupport.getAttachments();
        for (int i = 0; i < attachments.length; i++) {
            Attachment attachment = attachments[i];
            if (attachment.isHasSeen()) {
                p.put("opensign_openoces_attachment_signed", attachment.getHashValue());
            }
        }
    }
}