/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: AttachmentListViewDialog.java,v 1.10 2006/10/21 22:55:20 cara Exp $ */

package org.openoces.opensign.client.applet.attach;

/**
 * This class implements a dialog style attachment-list view
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.dialogs.AbstractDialog;
import org.openoces.opensign.client.applet.attach.resources.AttachmentResourceManager;

import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Observer;
import java.util.Observable;

public class AttachmentListViewDialog extends AbstractDialog implements AttachmentListView {
    // GridBagConstraints.LINE_START/LINE_END is not supported in JDK 1.1.4
    // but as long as we are writing text left->right this will do:
    private int LINE_START = GridBagConstraints.WEST;
    private int LINE_END = GridBagConstraints.EAST;

    private AttachmentImpl [] attachments;
    private Frame dialogOwner;

    public AttachmentListViewDialog(RestrictedAction ra, Frame owner) {
        super(ra, owner, AttachmentResourceManager.getString("ATTACHMENTS"));
        this.dialogOwner = owner;
    }

    public void setAttachments(AttachmentImpl[] attachments) {
        this.attachments = attachments;
        buildDialog();
    }

    public void showAttachments() {
        show();
    }

    private void buildDialog() {
        Panel eastPnl = new Panel();
        Panel buttonsPnl = new Panel();

        buttonsPnl.setLayout(new BorderLayout());

/*
        eastPnl.add(new Button("Add attachment..."));
        eastPnl.add(new Button("Cancel"));
*/
        Button okBtn = new Button("OK");
        okBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okClicked();
            }
        });
        eastPnl.add(okBtn);

        buttonsPnl.add(eastPnl, BorderLayout.EAST);


        Panel tblPnl = new Panel();
        tblPnl.setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = LINE_END;
        c.fill = GridBagConstraints.NONE;
        c.gridx = 0;
        c.gridy = 0;
        // add head lines
        c.anchor = LINE_START;
        tblPnl.add(new Label(AttachmentResourceManager.getString("TITLE")), c);
        c.gridx++;
        tblPnl.add(new Label(AttachmentResourceManager.getString("SIZE")), c);
        c.gridx++;
        c.gridx++; // past "save" button
        c.gridx++; // past "view" button
        c.gridy++; // past status indicator

        for (int i = 0; i < attachments.length; i++) {
            c.gridx = 0;
            AttachmentImpl a = attachments[i];
            a.setDialogOwner(dialogOwner);
            c.anchor = LINE_START;
            tblPnl.add(new Label(a.getTitle()), c);
            c.gridx++;
            c.anchor = LINE_END;
            tblPnl.add(new Label(Long.toString(a.getSize()) + " bytes"), c);
            c.gridx++;
            Button saveButton = new Button(AttachmentResourceManager.getString("SAVE"));
            tblPnl.add(saveButton, c);
            saveButton.addActionListener(a);
            c.gridx++;

/*
            tblPnl.add(new Button("Remove"), c);
            c.gridx++;
*/
            Button viewButton = new Button(AttachmentResourceManager.getString("VIEW"));
            viewButton.setEnabled(a.isViewable());
            tblPnl.add(viewButton, c);
            viewButton.addActionListener(a);
            c.gridx++;
            c.anchor = LINE_START;
            Label statusIndicator = new Label();
            if (a.isHasSeen()) {
                statusIndicator.setText(AttachmentResourceManager.getString("FETCHED"));
            } else {
                if (a.isOptional()) {
                    statusIndicator.setText(AttachmentResourceManager.getString("NOT_FETCHED_IS_OPTIONAL"));
                } else {
                    statusIndicator.setText(AttachmentResourceManager.getString("NOT_FETCHED_IS_MANDATORY"));
                }
            }
            new SeenObserver(a, statusIndicator);
            tblPnl.add(statusIndicator, c);

            c.gridx++;
            c.gridy++;
        }
        dialog.setLayout(new BorderLayout());
        // add a ScrollPane:
        ScrollPane scrollPane = new ScrollPane();
        scrollPane.add(tblPnl);
        dialog.add(scrollPane, BorderLayout.CENTER);
        dialog.add(buttonsPnl, BorderLayout.SOUTH);

        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                okClicked();
            }
        });
        dialog.pack();
        // check again:
        Dimension tableSize = tblPnl.getSize();
        Insets insets = scrollPane.getInsets();
        tableSize.height += insets.top + insets.bottom;
        tableSize.width += insets.left + insets.right;
        // allow space for scrollbar:
        tableSize.width += scrollPane.getVScrollbarWidth();
        // set to with of table and height of this - if not too big
        if (tableSize.height > 500) {
            tableSize.height = 500;
        }
        scrollPane.setSize(tableSize);
        dialog.pack();
    }

    private static class SeenObserver implements Observer {
        private Label statusIndicator;

        SeenObserver(AttachmentImpl a, Label statusIndicator) {
            this.statusIndicator = statusIndicator;
            a.addObserver(this);
        }

        public void update(Observable o, Object arg) {
            AttachmentImpl a = (AttachmentImpl) o;
            if (a.isHasSeen()) {
                statusIndicator.setText(AttachmentResourceManager.getString("FETCHED"));
            }
        }
    }
}