/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: AttachmentSupportImpl.java,v 1.5 2006/07/23 21:35:04 cara Exp $ */

package org.openoces.opensign.client.applet.attach;

import org.openoces.opensign.appletsupport.AttachmentSupport;
import org.openoces.opensign.appletsupport.Attachment;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.attach.resources.AttachmentResourceManager;
import org.openoces.opensign.client.applet.dialogs.DlgInformation;
import org.openoces.opensign.client.applet.CallBackHandler;

import java.applet.Applet;
import java.awt.*;

/**
 * This class implements a facade between the applet and attachment support classes
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

public class AttachmentSupportImpl implements AttachmentSupport {
    private AttachmentServer attachmentServer;
    private AttachmentListView attachmentListView;
    private Applet applet;

    public void init(RestrictedAction ra, Applet applet, String attachmentParam) {
        this.applet = applet;
        attachmentServer = new AttachmentServer(ra, applet, attachmentParam);
        AttachmentViewerManager.registerViewers(ra);
    }

    public void show(RestrictedAction ra, Frame parent) {
        AttachmentImpl[] attachments = (AttachmentImpl [])attachmentServer.getAttachments();
        attachmentListView = new AttachmentListViewDialog(ra, parent);
        attachmentListView.setAttachments(attachments);
        attachmentListView.showAttachments();
    }
    public Attachment [] getAttachments(){
        return attachmentServer.getAttachments();
    }

    public boolean isSigningOK() {
        Attachment[] attachments = getAttachments();
        boolean allMandatorySeen = true;
        for (int i=0; i<attachments.length; i++){
            allMandatorySeen &= (attachments[i].isOptional() || attachments[i].isHasSeen());
        }

        if (!allMandatorySeen){
            String ok = ResourceManager.getString("DLG_INFO_BUTTON_OK");
            String header = AttachmentResourceManager.getString("NOT_ALL_MANDATORY_ATTACHMENTS_DOWNLOADED_HEADER");
            String msg = AttachmentResourceManager.getString("NOT_ALL_MANDATORY_ATTACHMENTS_DOWNLOADED_HEADER");
            DlgInformation dlgInfo = new DlgInformation(attachmentServer.getRestrictedAction(),
                    ((CallBackHandler) applet).getFrame(), header, msg, ok);
            dlgInfo.show();
            return false;
        }
        return true;
    }
}
