/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: HtmlViewer.java,v 1.4 2006/08/12 19:02:36 cara Exp $ */

package org.openoces.opensign.client.applet.attach;

/**
 * This class implements a html attachment viewer
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.text.EditorKit;
import javax.swing.text.ChangedCharSetException;
import javax.swing.text.Document;
import javax.swing.text.BadLocationException;
import java.awt.*;
import java.io.UnsupportedEncodingException;
import java.io.StringReader;
import java.io.Reader;
import java.io.IOException;
import java.util.StringTokenizer;

public class HtmlViewer extends AbstractAttachmentViewer {
    public static boolean isSupported(){
        return !OS.isMsJVM();
    }
    HtmlViewer(RestrictedAction ra) {
        super(ra);
        JEditorPane.registerEditorKitForContentType("text/html",
                  "org.openoces.opensign.client.applet.attach.html.OpenSignHTMLEditorKit",
                this.getClass().getClassLoader());

    }

    protected String getContentType() {
        return "text/html";
    }

    protected String getErrorText() {
        return getErrorText("UNSPECIFIED_ERROR");
    }

    protected String getErrorText(Exception e) {
        return getErrorText(e.getMessage());
    }

    protected String getErrorText(String s) {
        return "<html><body><h1>" + s + "</h1></body></html>";
    }
    // make it scroll up again  -  here could be a nicer way!
    private JScrollPane displayScroller;
    private boolean doneUpscrolling;
    void whenStateChanged(){
        if (doneUpscrolling) return;
        if (!displayScroller.isValid()) return;
        Point before = displayScroller.getViewport().getViewPosition();
        if (before.y == 0) return;
        doneUpscrolling = true;
        displayScroller.getViewport().setViewPosition(new Point());
    }
    Component getAttachmentComponent(byte [] data) {
        FileLog.debug("htmlViewer getAttachmentComponent called");
        JEditorPane display;
        //...
        display = new JEditorPane();
        display.setContentType(getContentType());
        display.setEditable(false);
/*        display.setBackground( Color.LIGHT_GRAY );
        display.setFont(new Font( "Serif", Font.PLAIN, 12 ));
        display.setForeground( Color.BLACK );
        */
/*        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 4;
        */
        // ...
        FileLog.debug("Creating JScrollPane");
        /*JScrollPane*/ displayScroller = new JScrollPane(display);
        displayScroller.getViewport().addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e) {
                whenStateChanged();
            }
        });
        try {
            // determine charset
            String t = new String(data, "UTF8");
            // check for a BOM (Byte Order Mark)(http://www.unicode.org/faq/utf_bom.html#22)
            // and remove if present
            if (t.charAt(0) == 0xFEFF){
                FileLog.debug("skipping BOM");
                t = t.substring(1);
            }
            FileLog.debug("Content: "+t);
            Document doc = display.getDocument();
            doc.remove(0, doc.getLength());
            if (t == null || t.equals("")) {
                display.setText(getErrorText("EMPTY"));
            }
            Reader r = new StringReader(t);
            EditorKit kit = display.getEditorKit();
            kit.read(r, doc, 0);
        } catch (ChangedCharSetException e) {
            FileLog.debug("ChangedCharSetException: "+e.getMessage());
            // try new encoding
            String charset = getNewCharSet(e);
            try {
                String t = new String(data, charset);
                Reader r = new StringReader(t);
                EditorKit kit = display.getEditorKit();
                FileLog.debug("Got editorkit (again): "+kit);
                Document doc = display.getDocument();
                doc.remove(0, doc.getLength());
                doc.putProperty("IgnoreCharsetDirective", Boolean.TRUE);
                kit.read(r, doc, 0);
            } catch (IOException ee) {
                display.setText(getErrorText(e));
            } catch (BadLocationException ee) {
                display.setText(getErrorText(e));
            }
        } catch (BadLocationException e) {
            display.setText(getErrorText(e));
        } catch (UnsupportedEncodingException e) {
            display.setText(getErrorText(e));
        } catch (IOException e) {
            display.setText(getErrorText(e));
        }
        return displayScroller;
    }

    // from:
    // http://www.java2s.com/ExampleCode/Swing-JFC/JEditorPaneExample.htm
    protected String getNewCharSet(ChangedCharSetException e) {
        String spec = e.getCharSetSpec();
        if (e.keyEqualsCharSet()) {
            // The event contains the new CharSet
            return spec;
        }

        // The event contains the content type
        // plus ";" plus qualifiers which may
        // contain a "charset" directive. First
        // remove the content type.
        int index = spec.indexOf(";");
        if (index != -1) {
            spec = spec.substring(index + 1);
        }

        // Force the string to lower case
        spec = spec.toLowerCase();

        StringTokenizer st = new StringTokenizer(spec, " \t=", true);
        boolean foundCharSet = false;
        boolean foundEquals = false;
        while (st.hasMoreTokens()) {
            String token = st.nextToken();
            if (token.equals(" ") || token.equals("\t")) {
                continue;
            }
            if (foundCharSet == false && foundEquals == false
                    && token.equals("charset")) {
                foundCharSet = true;
                continue;
            } else if (foundEquals == false && token.equals("=")) {
                foundEquals = true;
                continue;
            } else if (foundEquals == true && foundCharSet == true) {
                return token;
            }

            // Not recognized
            foundCharSet = false;
            foundEquals = false;
        }

        // No charset found - return a guess
        return "8859_1";
    }
}
