/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: BootApplet.java,v 1.23 2006/10/15 16:05:34 cara Exp $ */

package org.openoces.opensign.client.applet.bootstrap;

/**
 * This class implements the initial bootstrapping part of OpenSign
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.client.applet.bootstrap.classloader.MultiClassLoader;
import org.openoces.opensign.client.applet.bootstrap.classloader.ZipClassLoader;
import org.openoces.opensign.client.applet.bootstrap.downloads.OpenSignDownloads;
import org.openoces.opensign.client.applet.bootstrap.interfaces.Plugaware;
import org.openoces.opensign.client.applet.bootstrap.interfaces.PollingApplet;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedActor;
import org.openoces.opensign.client.applet.bootstrap.security.AbstractRestrictedAction;
import org.openoces.opensign.client.Configuration;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

public class BootApplet extends Applet {

    private class InstallerThread extends Thread {
        DownloadItem item;
        Vector pluginItems;

        public InstallerThread(DownloadItem item, Vector pluginItems) {
            super("InstallerThread");
            this.item = item;
            this.pluginItems = pluginItems;
        }

        public void run() {
            try {
                loadItems(item, pluginItems);
                whenDone();
            } catch (Exception e) {
                e.printStackTrace();
                FileLog.fatal("An exception occured " + e);
                whenFailed(e);
            }
        }
    }

    protected Applet realApplet = null;
    private boolean realAppletInitialized = false;
    private CardLayout layoutMgr;
    private static final String ZIP_FILE_ALIAS_PARAM = "ZIP_FILE_ALIAS";

    // list of zip-files separated by ; in the form:
    // bla.zip,dk.oces.Register;bla2.zip,bla.bla.Init
    private static final String MS_SUPPORT_PARAM = "MS_SUPPORT";
    private static final String SUN_SUPPORT_PARAM = "SUN_SUPPORT";
    private static final String APPLET_SUPPORT_PARAM = "APPLET_SUPPORT";// depending on the Applet - currently we do it for OpenMail
    private static final String EXTRA_FILE_NAME_PARAM = "EXTRA_ZIP_FILE_NAMES";
    private static final String ZIP_BASE_URL_PARAM = "ZIP_BASE_URL";
    private static final String LOG_LEVEL_PARAM = "LOG_LEVEL";
    private static final String STRIP_ZIP_PARAM = "STRIP_ZIP";

    private Hashtable downloadTable = new Hashtable();
    private Vector pluginItems = new Vector();
    private DownloadItem mainItem = null;
    private AbstractRestrictedAction ra; // the one suitable on the runtime platform
    /**
     * This is really what needs to be signed! The description of the potential files with checksums and classnames. The
     * names in the applet-tag will be the ones to download.
     */
    public static final int DEBUG = 0;
    public static final int INFO = 1;
    public static final int ERROR = 2;


    private void initDownloadItems() {
        Vector downloadItems = OpenSignDownloads.getDownloadItems();
        for (int i = 0; i < downloadItems.size(); i++) {
            DownloadItem downloadItem = (DownloadItem) downloadItems.elementAt(i);
            downloadTable.put(downloadItem.getAlias(), downloadItem);
        }
    }

    public BootApplet() throws Exception {
        layoutMgr = new CardLayout();
        this.setLayout(layoutMgr);
    }

    private boolean isSigned() {
        // check if we are part of a signed applet and tell the user if this is not the case
        try {
            ra.getProperty("user.name");
            FileLog.debug("The boot applet is signed");
            return true;
        } catch (SecurityException e) {
            FileLog.debug("The boot applet is not signed!!");
            return false;
        }
    }

    private void showUnsignedMessage() {
        Panel msgPanel = new Panel();
        msgPanel.add(new Label("This applet was not allowed to run as a signed applet."));
        this.add(msgPanel, "msg");
        layoutMgr.show(this, "msg");
    }

    public void init() {
        String logLevel = getParameter(LOG_LEVEL_PARAM);
        if ("INFO".equalsIgnoreCase(logLevel)) {
            FileLog.setVerbose(FileLog.INFO);
        } else if ("DEBUG".equalsIgnoreCase(logLevel)) {
            FileLog.setVerbose(FileLog.DEBUG);
        } else if ("ERROR".equalsIgnoreCase(logLevel)) {
            FileLog.setVerbose(FileLog.ERROR);
        }

        FileLog.info("selected loglevel: " + FileLog.getLoglevelString());
        FileLog.info("Timing: BootApplet.init():" + new Date());
        if (realApplet != null) {
            FileLog.info("BootApplet.init - realApplet exists so just passing init()");
            realApplet.init();
            return;
        }
        String stripZip = getParameter(STRIP_ZIP_PARAM);
        Configuration.setDoStrip(stripZip != null && stripZip.length() > 0);

        initDownloadItems();
        try {
            checkEnvironment();
            if (!isSigned()) {
                showUnsignedMessage();
                return;
            }
            if (Configuration.is_beta()) {
                ZipClassLoader.setRestrictedAction(ra);
                this.showBetaDialog();
            }
            prepareDownload();
            initGui();
            Thread t = new InstallerThread(mainItem, pluginItems);
            t.start();
        } catch (Exception e) {
            // show error msg
            FileLog.error("Failed to prepare download :" + e.getMessage());
            whenFailed(e);
        }
    }

    private void checkEnvironment() throws Exception {
        // determine which OS and JVM etc
        FileLog.info("This browser is a " + OS.getBrowser(this));
        Class raClass;
        if (OS.isMsJVM()) {
            raClass = Class.forName("org.openoces.opensign.client.applet.bootstrap.security.MsRestrictedAction");
        } else if (OS.isJavaPlugin()) {
            String version = OS.getJavaPluginVersion();
            if (version.indexOf("1.3") != -1) {
                throw new Exception("Java plugin 1.3 is not supported");
            } else {
                raClass = Class.forName("org.openoces.opensign.client.applet.bootstrap.security.SunRestrictedAction");
            }
        } else if (OS.getBrowser(this).equals("OPERA")) {
            // wonder if it runs - and if it supports pluginversion:
            String version = OS.getJavaPluginVersion();
            FileLog.info("This is an OPERA with java plugin version: " + version);
            raClass = Class.forName("org.openoces.opensign.client.applet.bootstrap.security.SunRestrictedAction");
        } else if (OS.getBrowser(this).equals("APPLETVIEWER")) {
            // not needed?
            raClass = Class.forName("org.openoces.opensign.client.applet.bootstrap.security.SunRestrictedAction");
        } else {
            throw new Exception("Unknown or unsupported Java environment");
        }
        ra = (AbstractRestrictedAction) raClass.newInstance();
    }


    /**
     * Check if any of the zip-files requires OS support lib to be loaded
     *
     * @return whether OS support plugin is needed
     */
    private boolean getRequiresOSSupport() {
        // check what support is needed by the extra zip files (dependencies)
        String extraZipFiles = getParameter(EXTRA_FILE_NAME_PARAM);
        if (extraZipFiles != null) {
            StringTokenizer strTok = new StringTokenizer(extraZipFiles, ",");
            while (strTok.hasMoreTokens()) {
                String pluginFileName = strTok.nextToken();
                DownloadItem pluginItem = (DownloadItem) downloadTable.get(pluginFileName);
                if (pluginItem.getRequiresOSSupport()) {
                    return true;
                }
            }
        }
        return false;
    }


    private void prepareDownload() throws Exception {
        String zipFileAlias = getParameter(ZIP_FILE_ALIAS_PARAM);
        // check if relative or absolute:
        String zipBaseUrl = getParameter(ZIP_BASE_URL_PARAM);
        if (zipBaseUrl == null) {
            // if not set: assume relative to here codeBase or documentBase
            zipBaseUrl = Utils.getCodeBaseWithoutMinusOnePort(getCodeBase(),true);
            FileLog.info("ZIP BASE not set - relative to codeBase - set to " + zipBaseUrl);
        } else if (zipBaseUrl.startsWith("/")) {
            // relative to this server root
            String prefix = Utils.getCodeBaseWithoutMinusOnePort(getCodeBase(),false);
            zipBaseUrl = prefix + zipBaseUrl;
            FileLog.info("ZIP BASE server root relative - set to " + zipBaseUrl);

        } else if (zipBaseUrl.startsWith("http")) {
            // absolute
            FileLog.info("ZIP BASE absolute - unchanged " + zipBaseUrl);
        } else {
            // assume relative to here codeBase or documentBase
            zipBaseUrl = Utils.getCodeBaseWithoutMinusOnePort(getCodeBase(),true) + zipBaseUrl;
            FileLog.info("ZIP BASE relative to codeBase - set to " + zipBaseUrl);
        }

        if (!DownloadItem.setZipCacheDir(getPluginCacheDir(), ra)) {
            throw new Exception("Unable to create zip cache dir " + getPluginCacheDir());
        }
        ra.setBaseUrl(zipBaseUrl);
        DownloadItem.setBaseUrl(zipBaseUrl);
        mainItem = (DownloadItem) downloadTable.get(zipFileAlias);
        if (mainItem == null) {
            throw new Exception("Failed to get main item from alias: " + zipFileAlias);
        }

        if (getRequiresOSSupport()) {
            if (OS.isMsJVM()) {
                String msSupportZip = getParameter(MS_SUPPORT_PARAM);
                if (msSupportZip != null && msSupportZip.length() > 0) {
                    DownloadItem supportItem = (DownloadItem) downloadTable.get(msSupportZip);
                    if (supportItem == null) {
                        throw new Exception("Unable to find support item " + msSupportZip);
                    }
                    pluginItems.addElement(supportItem);
                }
            } else if (OS.isJavaPlugin() || OS.getBrowser(this).equals("APPLETVIEWER")) {
                String sunSupportZip = getParameter(SUN_SUPPORT_PARAM);
                if (sunSupportZip != null && sunSupportZip.length() > 0) {
                    DownloadItem supportItem = (DownloadItem) downloadTable.get(sunSupportZip);
                    if (supportItem == null) {
                        throw new Exception("Unable to find support item " + sunSupportZip);
                    }
                    pluginItems.addElement(supportItem);
                }
            }
        }
        String extraZipFiles = getParameter(EXTRA_FILE_NAME_PARAM);
        if (extraZipFiles != null) {
            StringTokenizer strTok = new StringTokenizer(extraZipFiles, ",");
            while (strTok.hasMoreTokens()) {
                String pluginFileName = strTok.nextToken();
                DownloadItem pluginItem = (DownloadItem) downloadTable.get(pluginFileName);
                if (pluginItem == null) {
                    throw new Exception("Failed to get main item from alias: " + zipFileAlias);
                }
                pluginItems.addElement(pluginItem);
            }
        }
        // see if any extra support is needed
        String appletSupportZip = getParameter(APPLET_SUPPORT_PARAM);
        if (appletSupportZip != null && appletSupportZip.length() > 0) {
            DownloadItem supportItem = (DownloadItem) downloadTable.get(appletSupportZip);
            if (supportItem == null) {
                throw new Exception("Unable to find support item " + appletSupportZip);
            }
            pluginItems.addElement(supportItem);
        }
    }

    private void showBetaDialog() {
        String disclaimer = "This is a beta version of an OpenOCES applet. It should only be used for testing purposes.";
        final Dialog dlg = ra.createModalDialog(getFrame(), "Important note");
        dlg.add(new Label(disclaimer));
        Button okBtn = new Button("OK");
        okBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dlg.hide();
            }
        });
        Panel btnPanel = new Panel();
        btnPanel.add(okBtn);
        dlg.add(btnPanel, BorderLayout.SOUTH);
        dlg.pack();
        Dimension dim = dlg.getToolkit().getScreenSize();
        Rectangle abounds = dlg.getBounds();
        dlg.setLocation((dim.width - abounds.width) / 2,
                (dim.height - abounds.height) / 2);
        dlg.show();
    }

    private Frame getFrame() {
        try {
            Container c = this;
            FileLog.debug(c.toString());
            try {
                while (c != null && !(c instanceof Frame)) c = c.getParent();
                if (c != null)
                    return (Frame) c;
            } catch (Throwable t) {
                FileLog.error("Unable to find browser frame, creating new one");
            }
            return new Frame();
        } catch (Throwable t) {
            FileLog.error("unexpected error occured " + t.getMessage());
            return null;
        }

    }

    private void initGui() {
        Panel progressPanel = new Panel();
        // calculate sizes:
        progressPanel.setLayout(new GridLayout(1, 1));
        progressPanel.add(mainItem.getProgressBar());
        mainItem.addProgressItem();
        for (int i = 0; i < pluginItems.size(); i++) {
            DownloadItem downloadItem = (DownloadItem) pluginItems.elementAt(i);
            downloadItem.addProgressItem();
//            progressPanel.add(downloadItem.getProgressBar());
        }
        this.add(progressPanel, "boot");
        layoutMgr.show(this, "boot");

    }

    private boolean loadItems(DownloadItem item, Vector pluginItems) throws IOException, NoSuchAlgorithmException {
//        String appletClassName = getParameter(APPLET_CLASS_NAME_PARAM);

        MultiClassLoader appletLoader = new ZipClassLoader(item, this.getClass().getClassLoader());

        Class appletClass;
        try {
//            appletClass = appletLoader.loadClass(appletClassName);
            appletClass = appletLoader.loadClass(item.getClassName());
            FileLog.debug("Loaded class " + appletClass.getName());
        } catch (Exception ex) {
            FileLog.debug("Load failed");
            ex.printStackTrace();
            return false;
        }
        try {
            realApplet = (Applet) appletClass.newInstance();
        } catch (Exception ex) {
            FileLog.debug("Failed to instantiate");
            ex.printStackTrace();
            return false;
        }

        if (realApplet != null) {
            if (realApplet instanceof RestrictedActor) {
                FileLog.debug("Applet is RestrictedActor as expected");
                ((RestrictedActor) realApplet).setRestrictedAction(ra);
            } else {
                FileLog.debug(">>>>>>>>>>>>>>>>>>>>>>> The applet is NOT RestrictedActor, pity: " + realApplet);
            }
            realApplet.setStub(new DelegationAppletStub(this));

            loadPlugins(realApplet, pluginItems);
            return true;
        }
        return false;
    }

    protected void whenDone() {
        this.add(realApplet, "real");
        realApplet.init();
        realAppletInitialized = true;
        layoutMgr.show(this, "real");
        realApplet.start();
        FileLog.debug("After BootApplet calling realApplet.start()");
    }

    private void whenFailed(Exception e) {
        FileLog.error("Failed downloading: " + e.getMessage());
        Label label = new Label(e.getMessage());
        this.add(label, "error");
        layoutMgr.show(this, "error");
    }

    private boolean loadPlugins(Applet applet, Vector pluginItems) throws IOException, NoSuchAlgorithmException {
        // check extra plugins:
        ClassLoader parentLoader = applet.getClass().getClassLoader();
        for (int i = 0; i < pluginItems.size(); i++) {
            DownloadItem downloadItem = (DownloadItem) pluginItems.elementAt(i);
            ZipClassLoader extraLoader = new ZipClassLoader(downloadItem, parentLoader);
            try {
                Class bootClass = extraLoader.loadClass(downloadItem.getClassName());
                try {
                    Object plugin = bootClass.newInstance();
                    if (applet instanceof Plugaware) {
                        FileLog.debug("Applet is Plugaware as expected - adding plugin " + plugin);
                        ((Plugaware) applet).addPlugin(plugin);
                    } else {
                        FileLog.debug("The applet is NOT Plugaware, pity " + applet);
                    }
                } catch (InstantiationException e) {
                    e.printStackTrace();
                    return false;
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                    return false;
                }
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
                return false;
            }
        }
        return true;
    }

    public void start() {
//        whenStarted();
        if ((realApplet != null) && realAppletInitialized) {
            realApplet.start();
        }
    }

    public void stop() {
        if (realApplet != null) {
            realApplet.stop();
        }
    }

    public void destroy() {
        if (realApplet != null) {
            realApplet.destroy();
            realApplet = null;
        }
    }

    private File getPluginCacheDir() {
        String relativePath;
        String OCES_INSTALL_DIRECTORY = ".oces";
        String OPENSIGN_INSTALL_DIRECTORY = "opensign";
        String LIBRARY_INSTALL_DIRECTORY = "plugins";

        relativePath = OCES_INSTALL_DIRECTORY + File.separator + OPENSIGN_INSTALL_DIRECTORY +
                File.separator + LIBRARY_INSTALL_DIRECTORY;

        return new File(ra.getUserHome() + File.separator + relativePath);
    }

    public String getAppletState() {
        if (realApplet != null){
        	if (realApplet instanceof PollingApplet){
                return ((PollingApplet) realApplet).getAppletState();
        	} else {
            	FileLog.error("Applet is not pollable");
                return "";
        	}
        } else {
            return "";
        }
    }

    public String getOutputData() {
        if (realApplet != null){
        	if (realApplet instanceof PollingApplet){
                return ((PollingApplet) realApplet).getOutputData();
        	} else {
            	FileLog.error("Applet is not pollable");
                return "";
        	}
        } else {
            return "";
        }
    }
}
