/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DownloadItem.java,v 1.5 2006/04/17 19:35:43 cara Exp $ */

package org.openoces.opensign.client.applet.bootstrap;

/**
 * This class defines a downloadable item
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.client.applet.bootstrap.classloader.Checksum;
import org.openoces.opensign.client.applet.bootstrap.classloader.MultiClassLoader;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.Configuration;

import java.awt.*;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;

public class DownloadItem {
    private String alias; // the name that distinguishes it - if more than one zip-file is relevant (useful for OpenSign.zip - Logon & Sign)
    private String zipFileName;
    private String pluginClassName;
    private String friendlyName;
    private Checksum checksum;
    private long size;
    private boolean localFile = true;
    private File dirForUsedClasses;// when classloader is used for stripping libraries
    private static String baseUrl = "";

    private static File zipCacheDir;
    private static RestrictedAction ra;
    private static final int BUFFER_SIZE = 2048;
    private static Component commonProgressBar = new ProgressBar(0);
    private static long commonSizeDone = 0;
    private static long commonSizeDoneAcc = 0;
    private boolean requiresOSSupport = false;

    public DownloadItem(String alias, String zipFileName, String pluginClassName, String friendlyName, Checksum checksum, long size, File dirForUsedClasses) {
        this(alias, zipFileName, pluginClassName, friendlyName, checksum, size, null, dirForUsedClasses);
    }

    public DownloadItem(String alias, String zipFileName, String pluginClassName, String friendlyName, Checksum checksum, long size) {
        this(alias, zipFileName, pluginClassName, friendlyName, checksum, size, null, null);
    }

    public DownloadItem(String zipFileName, String pluginClassName, String friendlyName, Checksum checksum, long size) {
        this(zipFileName, pluginClassName, friendlyName, checksum, size, (File) null);
    }

    public DownloadItem(String zipFileName, String pluginClassName, String friendlyName, Checksum checksum, long size, File dirForUsedClasses) {
        this(zipFileName, zipFileName, pluginClassName, friendlyName, checksum, size, null, dirForUsedClasses);
    }

    public DownloadItem(String zipFileName, String pluginClassName, String friendlyName, Checksum checksum, long size, String trustedClass) {
        this(zipFileName, zipFileName, pluginClassName, friendlyName, checksum, size, trustedClass, null);
    }

    public DownloadItem(String alias, String zipFileName, String pluginClassName, String friendlyName, Checksum checksum, long size, String trustedClass, File dirForUsedClasses) {
        MultiClassLoader.addTrustedClass(trustedClass);
        this.alias = alias;
        this.zipFileName = zipFileName;
        this.pluginClassName = pluginClassName;
        this.friendlyName = friendlyName;
        this.checksum = checksum;
        this.size = size;
        this.dirForUsedClasses = dirForUsedClasses;
        if (dirForUsedClasses != null) {
            FileLog.info("dirForUsedClasses set for: " + zipFileName + " to " + dirForUsedClasses);
        }
    }

    public boolean getRequiresOSSupport(){
        return requiresOSSupport;
    }
    public void setRequiresOSSupport(boolean requiresOSSupport){
        this.requiresOSSupport = requiresOSSupport;
    }
    public void addProgressItem(){
        FileLog.debug("About to add "+alias+" of file "+zipFileName);
        ((ProgressBar)commonProgressBar).addItem((int) size, baseUrl, zipCacheDir.toString());
    }

    public Component getProgressBar() {
        return commonProgressBar;
    }

    public void noteProgress(long progress) {
//        FileLog.debug("noteProgress "+progress);
//        try {
//            Thread.currentThread().sleep(1000);
//        } catch (InterruptedException e) {
//            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
//        }
        commonSizeDone += progress;
        String src = localFile ? "Local" : "Remote";
        String msg = friendlyName + " " + src + " progress: " + commonSizeDone + " of total " + size;
        FileLog.debug( msg);
    }

    public void noteDone() {
        String src = localFile ? "Local" : "Remote";
        String msg = friendlyName + " " + src + " fully read : " + size;
        FileLog.info(msg);
        commonSizeDoneAcc += size;
        ((ProgressBar) commonProgressBar).setLevel((int) commonSizeDoneAcc);
        localFile = false;
    }

    public String getAlias() {
        return alias;// what is used as key in a hashtable...
    }

    public String getName() {
        return zipFileName;
    }

    /**
     * Returns the filename containig the plugin. No parts of the path is returned.
     *
     * @return the filename
     */
    public String getFileName() {
        return getName() + ".zip";
    }

    public Checksum getChecksum() {
        return checksum;
    }

    public String getClassName() {
        return pluginClassName;
    }

    public File getDirForUsedClasses() {
        if (Configuration.doStrip()) {
            return dirForUsedClasses;
        } else {
            return null;
        }
    }

    public void download() throws IOException {
        URLConnection urlCon = new URL(baseUrl + "/" + getFileName()).openConnection();
        InputStream is = urlCon.getInputStream();
        int count;
        BufferedOutputStream dest = null;
        byte data[] = new byte[BUFFER_SIZE];
        File cacheDir = new File(getChecksumCacheDir());
        if (!ra.exists(cacheDir)) {
        	if (!ra.mkdirs(cacheDir)) {
        		throw new IOException("Unable to make directory "+cacheDir);
        	}
        }
        FileOutputStream fos = new FileOutputStream(this.getCachedAbsoluteFileName());
        dest = new BufferedOutputStream(fos, BUFFER_SIZE);
        while ((count = is.read(data, 0, BUFFER_SIZE)) != -1) {
            dest.write(data, 0, count);
        }
        dest.flush();
        dest.close();
    }

    public boolean cachedExists() {
        try {
            getAsStream();
        } catch (FileNotFoundException e) {
            return false;
        }
        return true;
    }

    private String getChecksumCacheDir(){
    	String checksumDir = checksum.toString();
    	return DownloadItem.zipCacheDir + File.separator + checksumDir;
    }
    private String getCachedAbsoluteFileName() {
        return getChecksumCacheDir()+ File.separator + this.getFileName();
    }

    public InputStream getAsStream() throws FileNotFoundException {
        return new FileInputStream(this.getCachedAbsoluteFileName());
    }

    public void createCachedVersion() throws IOException {
        BufferedOutputStream dest = null;
        int count;
        byte data[] = new byte[BUFFER_SIZE];

        InputStream is = getAsStream();

        // write the file to the disk
        FileOutputStream fos = new FileOutputStream(getCachedAbsoluteFileName());
        dest = new BufferedOutputStream(fos, BUFFER_SIZE);
        while ((count = is.read(data, 0, BUFFER_SIZE)) != -1) {
            dest.write(data, 0, count);
        }
        dest.flush();
        dest.close();
    }

    public static void setBaseUrl(String baseUrl) {
        DownloadItem.baseUrl = baseUrl;
    }

    public static boolean setZipCacheDir(File dir, RestrictedAction ra) {
    	DownloadItem.ra = ra;
        zipCacheDir = dir;
        if (ra.exists(zipCacheDir)) {
            // todo: set isDirectory back in!
            //if (ra.exists(zipCacheDir) && ra. zipCacheDir.isDirectory()){
            return true;
        } else if (ra.mkdirs(zipCacheDir)) {
            return true;
        } else {
            return false;
        }
    }

//    public static String getBaseUrl() {
//        return baseUrl;
//    }
}
