/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: FileLog.java,v 1.6 2006/08/12 18:09:44 cara Exp $ */

package org.openoces.opensign.client.applet.bootstrap;

/**
 * This class implements logging
 *
 * @author Jens Bo Friis  <jbf@it-practice.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DateFormat;
import java.util.Date;

public class FileLog {

	public final static int DEBUG = 5;
	public final static int INFO = 4;
	public final static int WARN = 3;
	public final static int ERROR = 2;
	public final static int FATAL = 1;

    private static int verbose = DEBUG;

	public static boolean isDebugEnabled() {
        return verbose >= DEBUG;
    }

    public static boolean isInfoEnabled() {
        return verbose >= INFO;
    }

    public static boolean isWarningEnabled() {
        return verbose >= WARN;
    }

    public static boolean isErrorEnabled() {
        return verbose >= ERROR;
    }

    public static String getLoglevelString() {
        if ( verbose == DEBUG ) return "DEBUG";
        if ( verbose == INFO ) return "INFO";
        if ( verbose == WARN ) return "WARN";
        if ( verbose == ERROR ) return "ERROR";
        if ( verbose == FATAL ) return "FATAL";
        return "n/a ("+verbose+")";
    }

    public static int getVerbose() {
        return verbose;
    }

    public static void setVerbose(int verbose) {
        FileLog.verbose = verbose;
    }

	public static void info(String s) {
		log(s, null, INFO);
	}

	public static void warn(String s) {
		log(s, null, WARN);
	}

	public static void fatal(String s) {
		log(s, null, FATAL);
	}

	public static void error(String s) {
		log(s, null, ERROR);
	}

	public static void error(String s, Throwable t) {
		log(s, t, ERROR);
	}

	public static void debug(String s) {
		log(s, null, DEBUG);
	}

	private synchronized static void logToConsole(String s) {
		System.out.println(s);
	}

	private static void log(String s, Throwable t, int type) {
		if (verbose < type)
			return;

		PrintWriter pw = null;
		synchronized (FileLog.class) {
			try {
				StringWriter sw = new StringWriter();
				pw = new PrintWriter(sw);
				String date = DateFormat.getDateTimeInstance().format(new Date(System.currentTimeMillis()));
				pw.print(date);
				switch (type) {
					case DEBUG :
						pw.print(" <DEBUG> ");
						break;
					case ERROR :
						pw.print(" <ERROR> ");
						break;
					case FATAL :
						pw.print(" <FATAL> ");
						break;
					case INFO :
						pw.print(" <INFO>  ");
						break;
					case WARN :
						pw.print(" <WARN>  ");
						break;
				}
				pw.print(Thread.currentThread().getName());
				pw.print(" - ");
				if (t != null) {
					pw.println(s);
					t.printStackTrace(pw);
				} else {
					pw.println(s);
				}
				String out = sw.getBuffer().toString();
   			    logToConsole(out);
			} catch (Throwable t1) {
				t1.printStackTrace();
			} finally {
				if (pw != null) {
					pw.close();
				}
			}
		}
	}
}