/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: ProgressBar.java,v 1.4 2006/04/17 19:35:43 cara Exp $ */

package org.openoces.opensign.client.applet.bootstrap;

/**
 * This class implements a progress bar widget
 * from http://examples.oreilly.com/javacrypt/files/oreilly/jonathan/awt/ProgressBar.java
 * package oreilly.jonathan.awt;
 *
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import java.awt.*;

public class ProgressBar
        extends Canvas {
    int mLevel;
    int mMaximum;
    Color mFrameColor;

    public ProgressBar() {
        this(100);
    }
    public void addItem(int size, String from, String to) {
        mMaximum += size;
        FileLog.debug("addItem "+size+ " sum: "+mMaximum);
        if (displayText1 == null){
            displayText1 = "Downloading java applet from "+from;
            displayText2 = "installing in "+to+" if necessary";
        }
    }
    private String displayText1;
    private String displayText2;
    private int width;
    private int height;
    private Rectangle progressRect;
    private int textX;
    private int headText1Y;
    private int headText2Y;
    private int progressTextY;
    private int xMargin = 10;
    private int barHeight = 20;
    private int spacing = 5;
//    public ProgressBar(int max, String displayText) {
//        this(max);
//        this.displayText1 = displayText;
//    }
    private void calculateSize() {
        if (OS.isMsJVM()) {
            width = this.getSize().width;
            height = this.getSize().height;
        } else {
            width = this.getWidth();
            height = this.getHeight();
        }
        FileLog.debug("width: "+width+" height: "+height);
    }

    private void calculateProgressRect() {
        textX = xMargin;
        headText1Y = barHeight;
        headText2Y = 2*barHeight;
        progressRect = new Rectangle();
        progressRect.x = xMargin;
        progressRect.width = width - 2*xMargin;
        progressRect.y = headText2Y + spacing;
        progressRect.height = barHeight;
        progressTextY = progressRect.y + 2*barHeight;
    }
    private void calculateLayout(){
        calculateSize();
        calculateProgressRect();
    }
    public void doLayout(){
        super.doLayout();
        calculateLayout();
    }
    public ProgressBar(int max) {
        setForeground(Color.gray);
        mFrameColor = Color.black;
        setMaximum(max);
        setLevel(0);
    }

    public void setMaximum(int max) {
        mMaximum = max;
        repaint();
    }

    public void setLevel(int level) {
        mLevel = (level > mMaximum) ? mMaximum : level;
        repaint();
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void paint(Graphics g) {
        if (textX == 0){
            calculateLayout();
        }
        double ratio = (double) ((double) mLevel / (double) mMaximum);
        FileLog.debug("level: "+mLevel+"/ max: "+mMaximum+"Ratio: "+ratio);
        int doneWidth = (int) ((double) progressRect.width * ratio);

        g.setColor(mFrameColor);

        g.drawRect(progressRect.x, progressRect.y, progressRect.width - 1, progressRect.height - 1);

        g.setColor(getForeground());
        g.fillRect(progressRect.x+1, progressRect.y+1, doneWidth, progressRect.height  - 2);

        g.setColor(getBackground());
        g.fillRect(progressRect.x+ doneWidth +1, progressRect.y+1, progressRect.width - 2 - doneWidth, progressRect.height  - 2);

//        g.setXORMode(Color.RED);
        // erase the old text:
//        g.setColor(Color.RED);
        g.fillRect(textX, progressTextY-barHeight+2, width-textX, barHeight);
        g.setColor(getForeground());
        g.drawString(displayText1, textX, headText1Y);
        g.drawString(displayText2, textX, headText2Y);
        String progressText = mLevel+ " of "+mMaximum+" ("+((int)(ratio*100))+"%) completed";
        g.drawString(progressText, textX, progressTextY);
    }
    public Dimension getMinimumSize() {
        return new Dimension(10, 1);
    }

    public Dimension getPreferredSize() {
        return new Dimension(100, 10);
    }
}
