/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: Utils.java,v 1.2 2006/10/15 13:49:32 cara Exp $ */

package org.openoces.opensign.client.applet.bootstrap;

/**
 * This class implements common functions available to both the bootstrapping
 * applet and plugins. 
 *
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

import java.net.URL;

public class Utils {
        /**
     * Returns the codebase and corrects for -1 port
     *
     * @return codeBase() with -1 or default port removed.
     */
    public static String getCodeBaseWithoutMinusOnePort(URL codeBase, boolean appendCodeBasePath) {
        String vurl = codeBase.getProtocol() + "://" + codeBase.getHost();
        if (codeBase.getProtocol().equalsIgnoreCase("http")
                && codeBase.getPort() != 80
                && codeBase.getPort() != -1
                || codeBase.getProtocol().equalsIgnoreCase("https")
                && codeBase.getPort() != 443
                && codeBase.getPort() != -1) {
            vurl = vurl + ":";
            vurl = vurl + codeBase.getPort();
        }

        if ( appendCodeBasePath ) {
            // getPath() not implemented in 1.1.4 so:
            // location of first "/" since location of host:

            String codeBaseString = codeBase.toString();
            int pathSearchBegin = codeBaseString.indexOf(codeBase.getHost());
            int pathBegin = codeBaseString.indexOf('/', pathSearchBegin);
            vurl = vurl+codeBaseString.substring(pathBegin);
        }
        return vurl;
    }

    public static String generateBaseUrl(URL codebase, String zipBaseUrl) {
        String s;
        if (zipBaseUrl == null) {
            // if not set: assume relative to here codeBase or documentBase
            s = getCodeBaseWithoutMinusOnePort(codebase,true);
            FileLog.info("ZIP BASE not set - relative to codeBase - set to " + s);
        } else if (zipBaseUrl.startsWith("/")) {
            // relative to this server root
            String prefix = getCodeBaseWithoutMinusOnePort(codebase,false);
            s = prefix + zipBaseUrl;
            FileLog.info("ZIP BASE server root relative - set to " + s);

        } else if (zipBaseUrl.startsWith("http")) {
            // absolute
            s = zipBaseUrl;
            FileLog.info("ZIP BASE absolute - unchanged " + zipBaseUrl);
        } else {
            // assume relative to here codeBase or documentBase
            s = getCodeBaseWithoutMinusOnePort(codebase,true) + zipBaseUrl;
            FileLog.info("ZIP BASE relative to codeBase - set to " + zipBaseUrl);
        }
        return s;
    }
}