/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DownloadChecksumGenerator.java,v 1.6 2006/04/02 08:52:37 cara Exp $ */

package org.openoces.opensign.client.applet.bootstrap.downloads;

import org.openoces.opensign.client.applet.bootstrap.classloader.Sha1Checksum;
import org.openoces.opensign.client.applet.bootstrap.classloader.Sha1ChecksummedInputStream;
import org.openoces.opensign.utils.HexDump;

import java.io.*;
import java.security.NoSuchAlgorithmException;
import java.util.Hashtable;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * This class generates the java class containing the checksums
 *
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Valeur  <prv@tdc.dk>
 */
public class DownloadChecksumGenerator {
    private static String items[] = {"opensign", "capi", "cdcard", "pkcs12", "oces", "jsse", "bcjce","mailsupport","attachment"};
    private Hashtable itemSizes = new Hashtable();

    public DownloadChecksumGenerator(String fname, String pluginDir) throws IOException, NoSuchAlgorithmException {

        File f = new File(fname);
        FileOutputStream os = new FileOutputStream(f);

        os.write(generatePrologue().getBytes());

        for (int i = 0; i < items.length; i++) {
            String item = items[i];
            os.write(generateItemMethod(item, pluginDir).getBytes());
        }

        for (int i = 0; i < items.length; i++) {
            String item = items[i];
            getSize(item+ ".zip", pluginDir);
            os.write(generateItemSizeMethod(item).getBytes());
        }

        os.write(generateEpilogue().getBytes());
        os.close();
    }

    private String generateItemMethod(String pluginName, String pluginDir) throws NoSuchAlgorithmException, IOException {
        StringBuffer sb = new StringBuffer();

        String variableName = pluginName + "Checksum";
        String methodName = "get" + pluginName + "Checksum";

        sb.append("private static final byte[] " + variableName + " = ");
        sb.append(getChecksum(pluginName + ".zip", pluginDir));
        sb.append(";\n");
        sb.append("public static byte[] " + methodName + "() { return " + variableName + "; }\n");

        return sb.toString();
    }

    private int getSize(String filename, java.lang.String path){
        int size = 0;
        try {
            File f = new File(path + File.separator + filename);
            FileInputStream fis = new FileInputStream(f);
            ByteArrayOutputStream bos = new ByteArrayOutputStream();

            byte buf[] = new byte[64 * 1024];
            int c;
            while ((c = fis.read(buf, 0, 64 * 1024)) > -1) {
                bos.write(buf, 0, c);
                size += c;
            }
        } catch (IOException e){

        }
        String itemName = filename.substring(0, filename.indexOf("."));
        itemSizes.put(itemName, new Integer(size));
        return size;
    }
    /**
     * Calculates the checksum and as a side-effect also the total data size which are stored in itemSizes
     *
     * @param filename
     * @param path
     * @return
     * @throws NoSuchAlgorithmException
     * @throws IOException
     */
    private String getChecksum(String filename, java.lang.String path) throws NoSuchAlgorithmException, IOException {
        StringBuffer sb = new StringBuffer();
        InputStream fis;
        byte[] digest;

        // find the item-name again:
        String itemName = filename.substring(0, filename.indexOf("."));

        sb.append("{");

        try {
            File f = new File(path + File.separator + filename);
            fis = new FileInputStream(f);
            Sha1ChecksummedInputStream checksummedStream = new Sha1ChecksummedInputStream(fis, new Sha1Checksum());
            ByteArrayOutputStream bos = new ByteArrayOutputStream();

            byte buf[] = new byte[64 * 1024];
            int c;

            while ((c = checksummedStream.read(buf, 0, 64 * 1024)) > -1) {
                bos.write(buf, 0, c);
            }

            ZipInputStream zis = new ZipInputStream(new ByteArrayInputStream(bos.toByteArray()));
            ZipEntry entry;

            int totalDataSize = 0;
            while ((entry = zis.getNextEntry()) != null) {
                if (entry.isDirectory()) {
                    continue;
                }
                int count;
                int datasize = (int) entry.getSize();
                byte[] data = null;
                int pos = 0;
                if (datasize != -1) {
                    data = new byte[datasize];
                    while ((count = zis.read(data, pos, data.length - pos)) != -1) {
                        pos += count;
                        if (pos == datasize) {
                            break;
                        }
                    }
                } else { // datasize == -1
                    while ((count = zis.read(buf, pos, buf.length - pos)) != -1) {
                        pos += count;
                        if (pos == datasize) {
                            break;
                        }
                    }
                    data = new byte[pos];
                }
//                totalDataSize += pos;
                totalDataSize += entry.getCompressedSize();
            }
            zis.close();

            itemSizes.put(itemName, new Integer(totalDataSize));

            digest = checksummedStream.getCalculatedChecksum().getSha1Value();

        } catch (FileNotFoundException e) {
            digest = new byte[20];
            itemSizes.put(itemName, new Integer(0));
        }

        for (int i = 0; i < digest.length; i++) {
            sb.append("(byte) 0x" + HexDump.tohex(digest[i], 2).toLowerCase() + ",");

        }

        sb.append("}");
        return sb.toString();
    }

    /**
     * To be called after generateItemMethod - when itemSizes are updated
     *
     * @param pluginName
     * @return source code of the method returning the data size of the item
     */
    private String generateItemSizeMethod(String pluginName) {
        StringBuffer sb = new StringBuffer();

        String methodName = "get" + pluginName + "DataSize";
        int itemDateSize = ((Integer) itemSizes.get(pluginName)).intValue();
        sb.append("public static int " + methodName + "() { return " + itemDateSize + "; }\n");

        return sb.toString();
    }

    private String generatePrologue() {
        StringBuffer sb = new StringBuffer();
        sb.append("package org.openoces.opensign.client.applet.bootstrap.downloads;\n\n");
        sb.append("public class DownloadChecksums {\n");
        return sb.toString();
    }

    private String generateEpilogue() {
        return "}";
    }

    public static void main(String[] args) {
        try {
            new DownloadChecksumGenerator(args[0], args[1]);
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }
}
