/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: MsRestrictedAction.java,v 1.6 2006/06/02 18:41:51 cara Exp $ */

package org.openoces.opensign.client.applet.bootstrap.security;

import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.awt.*;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URLConnection;
import java.util.Properties;

/**
 * This interface defines restricted actions for the native Microsoft jvm
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */
public class MsRestrictedAction extends AbstractRestrictedAction {
    // MS CAB section - different for JAR files!
    //
    private static final String DEFAULT_WINDOWS_HOME = "C:";
    public String getUserHome(){
        // check which OS-version we have:
//        os.version:
//        http://lopica.sourceforge.net/os.html:
//        NT/95: 4.0
//        98:4.1
//        ME:4.9
//        2000:5.0
//        XP: 5.1
        String osVersion = getProperty("os.version");
        if (osVersion.length() != 3 || osVersion.charAt(1) != '.'){
            FileLog.error("unknown Windows version format: "+osVersion+" returning C:\\ as userHome");
            return DEFAULT_WINDOWS_HOME;
        }
        int major = Integer.parseInt(osVersion.substring(0, 1));
        int minor = Integer.parseInt(osVersion.substring(2, 3));
        int version = major*10+minor;
        // debug:
        String userDir = getProperty("user.dir");
        // end debug
        String userHome = DEFAULT_WINDOWS_HOME;
        switch (version){
            case 40: // NT/95
                FileLog.error("Windows NT/95 not supported");
            break;
            case 41: // 98
                FileLog.info("Windows 98");
            break;
            case 49: // ME
                FileLog.info("Windows ME");
            break;
            case 50: // 2000
            case 51: // XP
                // check user.dir and REGISTRY and find out - maybe
                com.ms.wfc.app.RegistryKey regKey =  com.ms.wfc.app.Registry.LOCAL_MACHINE.getSubKey("Software\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList");
                String profileHome;
                if (regKey == null) {
                    profileHome = "Unable to get ProfilesDirectory from Registry!";
                } else {
                    profileHome = (String) regKey.getValue("ProfilesDirectory");
                }
                // usually it says %SystemDrive% so replace that with C: for now:
                // more general: replace %xx% with getEnv()...
                String sysDriveVar = "SystemDrive";
                String sysDriveString = "%"+sysDriveVar+"%";
                String defaultSysDrive = "C:";
                int sysDriveIndex = profileHome.indexOf(sysDriveString);
                if (sysDriveIndex != -1){
                    // see if we can get the SystemDrive:
// deprecated!                    String sysDrive = System.getenv(sysDriveVar);
                    profileHome = profileHome.substring(0, sysDriveIndex)+ defaultSysDrive+ profileHome.substring(sysDriveIndex+sysDriveString.length());
                }
                String userName = getProperty("user.name");
                userHome = profileHome+"\\"+userName;
                if (userHome.indexOf("%") != -1){
                    // unexpected - we get desparate - take the parent of userDir...
                    FileLog.error("userHome contained %: "+userHome+" - so we try with userDir: "+userDir);
                    userHome = userDir.substring(0, userDir.lastIndexOf('\\'));
                }
            break;
                // 2003? Longhorn?
            default:
                FileLog.error("Unexpected value of os.version: "+version);
            break;
        }
        // check if userHome can be written to:
        FileLog.error("UserHome: "+userHome+" version: "+version+" userDir: "+userDir);
        return userHome;
    }

    public String getProperty(String key) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PROPERTY);
        } catch (Throwable t) {
        }
        ;
        return System.getProperty(key);
    }

    public FileInputStream getFileInputStream(String fileName) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return new FileInputStream(fileName);
        } catch (Throwable t) {
            return null;
        }
    }

    public FileInputStream getFileInputStream(File file) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return new FileInputStream(file);
        } catch (Throwable t) {
            return null;
        }
    }

    public FileOutputStream getFileOutputStream(String fileName) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return new FileOutputStream(fileName);
        } catch (Throwable t) {
            return null;
        }
    }

    public FileOutputStream getFileOutputStream(File file) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return new FileOutputStream(file);
        } catch (Throwable t) {
            return null;
        }
    }

    public FileOutputStream getFileOutputStream(String fileName, boolean append) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return new FileOutputStream(fileName, append);
        } catch (Throwable t) {
            return null;
        }
    }

    public InputStream getResourceAsStream(Class clazz, String name) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSTEM);
            return clazz.getResourceAsStream(name);
        } catch (Throwable t) {
            return null;
        }
    }

    public boolean mkdirs(File f) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return f.mkdirs();
        } catch (Throwable t) {
            return false;
        }
    }

    public boolean delete(File f) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return f.delete();
        } catch (Throwable t) {
            return false;
        }
    }

    public boolean exists(File f) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return f.exists();
        } catch (Throwable t) {
            return false;
        }
    }

    public boolean isDirectory(File f) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return f.isDirectory();
        } catch (Throwable t) {
            return false;
        }
    }


    public String[] list(File dir, FilenameFilter filter) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            return dir.list(filter);
        } catch (Throwable t) {
            return null;
        }
    }

    public Dialog createModalDialog(Frame parent, String title) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.UI);
            return new Dialog(parent, title, true);
        } catch (Throwable t) {
            return null;
        }
    }

    public FileDialog getFileDialog(Frame parent, String title, int mode) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.UI);
            return new FileDialog(parent, title, mode);
        } catch (Throwable t) {
            return null;
        }
    }
    // actually no problem on windows for some reason - only for Linux
    public void showFileDialog(FileDialog dialog){
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            dialog.show();
        } catch (Throwable t) {
            FileLog.error("Couldn't open file dialog: "+t);
        }
    }

    public PrintJob getPrintJob(Frame frame, String title, Properties prefs){
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PRINTING);
            return Toolkit.getDefaultToolkit().getPrintJob(frame, title, prefs);
        } catch (Throwable t) {
            FileLog.error("Couldn't get printJob: "+t);
            return null;
        }
    }

    public void connect(URLConnection con) throws IOException {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.NETIO);
            con.connect();
        } catch (Throwable t) {
        }
    }
    public int getResponseCode(final HttpURLConnection con) throws IOException {
        throw new IOException("Unexpected call of getResponseCode");
    }

    public void setProperty(String name, String value) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PROPERTY);
        } catch (Throwable t) {
        }
        ;
        System.setProperty(name, value);
    }

    public InputStream getInputStream(URLConnection con) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSTEM);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.NETIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.THREAD);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PROPERTY);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.EXEC);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.REFLECTION);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PRINTING);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.REGISTRY);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.CLIENTSTORE);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.UI);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSSTREAMS);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.USERFILEIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.MULTIMEDIA);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SECURITY);
            return con.getInputStream();
        } catch (Throwable t) {
            t.printStackTrace();
        }
        return null;
    }

    public OutputStream getOutputStream(URLConnection con) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSTEM);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.NETIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.THREAD);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PROPERTY);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.EXEC);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.REFLECTION);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PRINTING);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.REGISTRY);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.CLIENTSTORE);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.UI);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSSTREAMS);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.USERFILEIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.MULTIMEDIA);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SECURITY);
//            con.setRequestProperty("User-Agent", USER_AGENT);
            con.setDoOutput(true);
            con.setDoInput(true);
            return con.getOutputStream();
        } catch (Throwable t) {
            t.printStackTrace();
        }
        return null;
    }

    public void addSSLProvider() throws IOException {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PROPERTY);
            System.getProperties().put("java.protocol.handler.pkgs", "iaik.protocol");
        } catch (Throwable t) {
            FileLog.error("addSSLProvider failed " + t.getMessage());
        }
    }

    public void setDefaultSSLSocketFactory(Object sslSocketFactory) {
    }

    public InputStream getInputStream(HttpURLConnection con) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSTEM);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.NETIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.THREAD);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PROPERTY);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.EXEC);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.REFLECTION);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.PRINTING);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.REGISTRY);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.CLIENTSTORE);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.UI);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSSTREAMS);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.USERFILEIO);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.MULTIMEDIA);
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SECURITY);
            return con.getInputStream();
        } catch (Throwable t) {
            t.printStackTrace();
        }
        return null;
    }

    public boolean load(String lib) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            System.load(lib);
            FileLog.info("Library succesfully loaded: " + lib);
            return true;
        } catch (SecurityException e) {
            FileLog.error("SecurityException when loading " + lib + " : " + e.getMessage());
            return false;
        } catch (UnsatisfiedLinkError e) {
            FileLog.error("UnsatisfiedLinkError when loading " + lib + " : " + e.getMessage());
            return false;
        }
    }

    public boolean loadLibrary(String lib) {
        try {
            com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.FILEIO);
            System.loadLibrary(lib);
            return true;
        } catch (UnsatisfiedLinkError e) {
            return false;
        }
    }

    public Thread createThread(Runnable runnable) {
        com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.THREAD);
        return new Thread(runnable);
    }

}
