/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: SunRestrictedAction.java,v 1.6 2006/06/02 18:41:52 cara Exp $ */

package org.openoces.opensign.client.applet.bootstrap.security;

import org.openoces.opensign.client.applet.bootstrap.FileLog;
import sun.security.x509.X509CertImpl;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSocketFactory;
import java.awt.*;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URLConnection;
import java.security.*;
import java.util.Properties;

/**
 * This interface defines restricted actions for the Sun jvm
 *
 * @author Preben Valeur  <prv@tdc.dk>
 */
public class SunRestrictedAction extends AbstractRestrictedAction {
    public String getUserHome(){
        FileLog.info("SunRestrictedAction getUserHome returning: "+getProperty("user.home"));
        return getProperty("user.home");
    }

    public String getProperty(final String key) {
        return (String) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                return System.getProperty(key);
            }
        });
    }

    public InputStream getInputStream(final URLConnection con) {
        return (InputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return con.getInputStream();
                } catch (IOException e) {
                    FileLog.error("IOException in getInputStream: "+e.getMessage());
                    return null;
                }
            }
        });
    }

    public OutputStream getOutputStream(final URLConnection con) {
        return (OutputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return con.getOutputStream();
                } catch (IOException e) {
                    FileLog.error("IOException in getOutputStream: "+e.getMessage());
                    return null;
                }
            }
        });
    }

    public void addSSLProvider() throws IOException {
        try {
            AccessController.doPrivileged(new PrivilegedExceptionAction() {
                public Object run() throws PrivilegedActionException {
                    Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
                    return null;
                }
            });
        } catch (PrivilegedActionException e) {
        }
    }

    public void setDefaultSSLSocketFactory(final Object sslSocketFactory) {
        AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                HttpsURLConnection.setDefaultSSLSocketFactory((SSLSocketFactory) sslSocketFactory);
                return null;
            }
        });
    }

//    public InputStream getInputStream(HttpURLConnection con) {
//        return null;  //To change body of implemented methods use File | Settings | File Templates.
//    }

    public Object getCertificate(final byte[] decoded) {
        return AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
//                    ByteArrayInputStream bis = new ByteArrayInputStream(decoded);
                    FileLog.info(">>>>>>>>>>>> getCertificate Classloader: " + this.getClass().getClassLoader());
                    Object cert = new X509CertImpl(decoded);
                    FileLog.info("Got cert, about to return it..." + cert);
                    return cert;
//                    CertificateFactory cf = CertificateFactory.getInstance("X.509");
//                    return (X509Certificate) cf.generateCertificate(bis);
                } catch (Exception e) {
                    FileLog.info("Exception in getCertificate:" + e.getMessage());
                    return null;
                }
            }
        });
    }

    public FileInputStream getFileInputStream(final String fileName) {
        return (FileInputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileInputStream(fileName);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
    }

    public FileInputStream getFileInputStream(final File file) {
        return (FileInputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileInputStream(file);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
    }

    public FileOutputStream getFileOutputStream(final String fileName) {
        return (FileOutputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileOutputStream(fileName);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
    }

    public FileOutputStream getFileOutputStream(final File file) {
        return (FileOutputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileOutputStream(file);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
    }

    public FileOutputStream getFileOutputStream(final String fileName, final boolean append) {
        return (FileOutputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileOutputStream(fileName, append);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
    }

    public InputStream getResourceAsStream(final Class clazz, final String name) {
        return (InputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                return clazz.getResourceAsStream(name);
            }
        });
    }

    public boolean mkdirs(final File f) {
        try {
            Boolean res = (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
                public Object run() {
                    if (f.mkdirs()) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }
            });
            return res.booleanValue();
        } catch (Throwable t) {
            return false;
        }
    }

    public boolean delete(final File f) {
        try {
            Boolean res = (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
                public Object run() {
                    if (f.delete()) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }
            });
            return res.booleanValue();
        } catch (Throwable t) {
            return false;
        }
    }

    public boolean exists(final File f) {
        try {
            Boolean res = (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
                public Object run() {
                    if (f.exists()) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }
            });
            return res.booleanValue();
        } catch (Throwable t) {
            return false;
        }
    }

    public boolean isDirectory(final File f) {
        try {
            Boolean res = (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
                public Object run() {
                    if (f.isDirectory()) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }
            });
            return res.booleanValue();
        } catch (Throwable t) {
            return false;
        }
    }

    public String[] list(final File dir, final FilenameFilter filter) {
        return (String[]) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                return dir.list(filter);
            }
        });
    }

    public Dialog createModalDialog(final Frame parent, final String title) {
        return (Dialog) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                return new Dialog(parent, title, true);
            }
        });
    }

    public FileDialog getFileDialog(final Frame parent, final String title, final int mode) {
        return (FileDialog) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                return new FileDialog(parent, title, mode);
            }
        });
    }

    public PrintJob getPrintJob(final Frame frame, final String title, final Properties prefs){
        return (PrintJob) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                return Toolkit.getDefaultToolkit().getPrintJob(frame, title, prefs);
            }
        });
    }
    
    public void showFileDialog(final FileDialog dialog){
        AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                dialog.show();
                return null;
            }
        });
    }

    public void connect(final URLConnection con) throws IOException {
        try {
            AccessController.doPrivileged(new PrivilegedExceptionAction() {
                public Object run() throws PrivilegedActionException {
                    try {
                        con.connect();
                    } catch (IOException e) {
                        throw new PrivilegedActionException(e);
                    }
                    return null;
                }
            });
        } catch (PrivilegedActionException e) {
            Throwable cause = e.getCause();
            if (cause instanceof IOException){
                throw (IOException) e.getCause();
            } else {
                throw new IOException("Exception from SunRestrictedAction.connect(): "+e+" "+e.getMessage()+" cause:"+cause);
            }
        }
    }

    public int getResponseCode(final HttpURLConnection con) throws IOException {
        Integer result = null;
        try {
            result = (Integer) AccessController.doPrivileged(new PrivilegedExceptionAction() {
                public Object run() throws PrivilegedActionException {
                    try {
                        return new Integer(con.getResponseCode());
                    } catch (IOException e) {
                        throw new PrivilegedActionException(e);
                    }
                }
            });
        } catch (PrivilegedActionException e) {
            Throwable cause = e.getCause();
            if (cause instanceof IOException){
                throw (IOException) e.getCause();
            } else {
                throw new IOException("Exception from SunRestrictedAction.getResponseCode(): "+e+" "+e.getMessage()+" cause:"+cause);
            }
        }
        return result.intValue();
    }

    public void setProperty(final String name, final String value) {
        AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                System.setProperty(name, value);
                FileLog.debug("setting property " + name + " to " + value);
                return null;
            }
        });
    }

    /**
     * @param lib FULL filepath
     * @return
     * @throws UnsatisfiedLinkError is propagated if the library does not exist.
     */
    public boolean load(final String lib) {
        AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                System.load(lib);
                return null;
            }
        });
        return true;
    }

    /**
     * @param lib Name of library (EXCLUDING extension and PATH!!)
     * @return
     * @throws UnsatisfiedLinkError is propagated if the library does not exist.
     */
    public boolean loadLibrary(final String lib) {
        AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                System.loadLibrary(lib);
                return null;
            }
        });
        return true;
    }

    public Thread createThread(final Runnable runnable) {
        return (Thread) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                return new Thread(runnable);
            }
        });
    }

}
