/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: CertificateDetailsPanel.java,v 1.4 2006/08/12 19:02:37 cara Exp $ */

package org.openoces.opensign.client.applet.dialogs;

/**
 * This class represent the panel of a dialog displaying certificate details
 *
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.x509.KeyUsage;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.awt.*;

public class CertificateDetailsPanel extends Panel {
    private Label lCertificateSubjectDN;
    private Label lCertificateSubjectDNValue;
    private Label lCertificateStore;
    private Label lCertificateStoreValue;
    private Label lCertificateIssuerDN;
    private Label lCertificateIssuerDNValue;
    private Label lCertificateSerialNumber;
    private Label lCertificateSerialNumberValue;
    private Label lCertificateNotBefore;
    private Label lCertificateNotBeforeValue;
    private Label lCertificateNotAfter;
    private Label lCertificateNotAfterValue;
    private Label lCertificateVersion;
    private Label lCertificateVersionValue;
    private Label lCertificateKeyUsage;
    private Label lCertificateKeyUsageValue;
    private AbstractCertificateHandler certificate;

    public CertificateDetailsPanel(AbstractCertificateHandler certificate) throws Exception {
        this.certificate = certificate;
        init();
    }

    private void init() throws Exception {
        lCertificateSubjectDN = new Label();
        lCertificateSubjectDNValue = new Label();
        lCertificateStore = new Label();
        lCertificateStoreValue = new Label();
        lCertificateIssuerDN = new Label();
        lCertificateIssuerDNValue = new Label();
        lCertificateSerialNumber = new Label();
        lCertificateSerialNumberValue = new Label();
        lCertificateNotBefore = new Label();
        lCertificateNotBeforeValue = new Label();
        lCertificateNotAfter = new Label();
        lCertificateNotAfterValue = new Label();
        lCertificateVersion = new Label();
        lCertificateVersionValue = new Label();
        lCertificateKeyUsage = new Label();
        lCertificateKeyUsageValue = new Label();

        addCertificateDetails();

        Panel pValues = new Panel(new GridLayout(8, 1));
        Panel pNames = new Panel(new GridLayout(8, 1));

        pNames.add(lCertificateSubjectDN);
        pValues.add(lCertificateSubjectDNValue);

        pNames.add(lCertificateStore);
        pValues.add(lCertificateStoreValue);

        pNames.add(lCertificateIssuerDN);
        pValues.add(lCertificateIssuerDNValue);

        pNames.add(lCertificateSerialNumber);
        pValues.add(lCertificateSerialNumberValue);

        pNames.add(lCertificateNotBefore);
        pValues.add(lCertificateNotBeforeValue);

        pNames.add(lCertificateNotAfter);
        pValues.add(lCertificateNotAfterValue);

        pNames.add(lCertificateVersion);
        pValues.add(lCertificateVersionValue);

        pNames.add(lCertificateKeyUsage);
        pValues.add(lCertificateKeyUsageValue);

        add(pNames);
        add(pValues);
    }

    /**
     * get certificate information and update gui
     *
     * @throws Exception
     */
    private void addCertificateDetails() throws Exception {
        String storeName;
        FileLog.debug("adding certificateDetails");
        lCertificateSubjectDN.setText(ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_SUBJECTDN") + ":");
        lCertificateSubjectDNValue.setText(certificate.getSubjectDN().getName());

        lCertificateStore.setText(ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_CERTIFICATESTORE") + ":");
        storeName = certificate.getStoreName();
        lCertificateStoreValue.setText(storeName);

        lCertificateIssuerDN.setText(ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_ISSUERDN") + ":");
        String issuerString = certificate.getIssuerDN() == null ? null : certificate.getIssuerDN().getName();

        lCertificateIssuerDNValue.setText(issuerString);

        lCertificateSerialNumber.setText(ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_SERIALNUMBER") + ":");

        String serialNumberString = certificate.getSerialNumber() == null ? null : certificate.getSerialNumber().toString(16).toUpperCase();
        lCertificateSerialNumberValue.setText(serialNumberString);

        lCertificateNotBefore.setText(ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_DATE_ISSUANCE") + ":");

        String notBeforeString = certificate.getNotBefore() == null ? null : java.text.SimpleDateFormat.getInstance().format(certificate.getNotBefore());
        lCertificateNotBeforeValue.setText(notBeforeString);

        String notAfterString = certificate.getNotAfter() == null ? null : java.text.SimpleDateFormat.getInstance().format(certificate.getNotAfter());

        lCertificateNotAfter.setText(ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_DATE_EXPIRY") + ":");
        lCertificateNotAfterValue.setText(notAfterString);

        lCertificateVersion.setText(ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_VERSION") + ":");
        lCertificateVersionValue.setText(Integer.toString(certificate.getVersion()));

        lCertificateKeyUsage.setText(ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE") + ":");

        KeyUsage keyusage = certificate.getIntendedKeyUsage();
        String keyUsageValue = "";

        if (keyusage == null) {
            keyUsageValue = ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE_NOT_AVAILABLE");
        } else {
            if (keyusage.includesDigitalSignature())
                keyUsageValue += (keyUsageValue.length() == 0 ? "" : ", ") + ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE_DIGITAL_SIGNATURE");
            if (keyusage.includesNonRepudiation())
                keyUsageValue += (keyUsageValue.length() == 0 ? "" : ", ") + ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE_NON_REPUDIATION");
            if (keyusage.includesKeyEncipherment())
                keyUsageValue += (keyUsageValue.length() == 0 ? "" : ", ") + ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE_KEY_ENCIPHERMENT");
            if (keyusage.includesDataEncipherment())
                keyUsageValue += (keyUsageValue.length() == 0 ? "" : ", ") + ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE_DATA_ENCIPHERMENT");
            if (keyusage.includesKeyAgreement())
                keyUsageValue += (keyUsageValue.length() == 0 ? "" : ", ") + ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE_KEY_AGREEMENT");
            if (keyusage.includesKeyCertSign())
                keyUsageValue += (keyUsageValue.length() == 0 ? "" : ", ") + ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE_KEY_CERT_SIGN");
            if (keyusage.includesKeyCRLSign())
                keyUsageValue += (keyUsageValue.length() == 0 ? "" : ", ") + ResourceManager.getString("DLG_CERTIFICATEDETAILS_LABEL_KEYUSAGE_CRL_SIGN");
        }

        lCertificateKeyUsageValue.setText(keyUsageValue);
    }
}
