/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DlgCertificateDetails.java,v 1.7 2006/04/02 20:30:53 cara Exp $ */

package org.openoces.opensign.client.applet.dialogs;

/**
 * This class represent a dialog displaying certificate details
 *
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.resources.ResourceManager;

import java.awt.*;
import java.awt.event.*;

public class DlgCertificateDetails extends AbstractDialog {
    private Button bClose;
    private AbstractCertificateHandler certificate;

    private boolean initialized = false;

    public DlgCertificateDetails(RestrictedAction ra, Frame owner, String title, AbstractCertificateHandler certificate) throws Exception {
        super(ra, owner, title);
        this.certificate = certificate;
        initialize();
    }

    /**
     * initialize gui.
     *
     * @throws Exception if some error occurs during dialog update. ex during populate of the dialog with the certificate information
     */
    private void initialize() throws Exception {
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets = new Insets(3, 3, 3, 3);

        dialog.setLayout(new GridBagLayout());

        dialog.add(new CertificateDetailsPanel(certificate), constraints);

        constraints.gridy = 1;
        constraints.anchor = GridBagConstraints.CENTER;

        bClose = new Button(ResourceManager.getString("DLG_CERTIFICATEDETAILS_BUTTON_CLOSE"));
        dialog.add(bClose, constraints);

        bClose.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.hide();
            }
        });

        KeyListener kl = new KeyListener() {
            public void keyTyped(KeyEvent evt) {
            }

            public void keyPressed(KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
                    dialog.hide();
                }
                if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
                    dialog.hide();
                }
            }

            public void keyReleased(KeyEvent evt) {
            }
        };
        bClose.addKeyListener(kl);

        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dialog.hide();
            }
        });

        dialog.pack();
        initialized = true;
    }

}
