/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DlgKeystoreChooser.java,v 1.2 2006/04/02 20:30:53 cara Exp $ */

package org.openoces.opensign.client.applet.dialogs;
/**
 * This class represent a keystore chooser dialog
 *
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class DlgKeystoreChooser extends AbstractDialog {
    private Button bCancel;
    private Button bOk;
    private boolean initialized = false;
    private Vector keyStoreHandlers;
    private KeyStoreHandler selectedKeyStoreHandler;
    private Frame frame;
    private RestrictedAction ra;
    private boolean cancelled = false;

    public DlgKeystoreChooser(RestrictedAction ra, Frame owner, String title, Vector keyStoreHandlers, KeyStoreHandler keyStoreHandler) {
        super(ra, owner, title);
        this.ra = ra;
        this.frame = owner;
        this.keyStoreHandlers = keyStoreHandlers;
        this.selectedKeyStoreHandler = keyStoreHandler;
        initialize();
    }

    private void initialize() {
        cancelled = false;

        dialog.setLayout(new BorderLayout());

        CheckboxGroup keystoreCheckboxGroup = new CheckboxGroup();

        Panel pnlKeystores = new Panel();
        Panel currentPanel = pnlKeystores;
        for (int i = 0; i < keyStoreHandlers.size(); i++) {
            final KeyStoreHandler storeHandlerIter = (KeyStoreHandler) keyStoreHandlers.elementAt(i);

            String localizedName = storeHandlerIter.getLocalizedName();
            Panel nestedPanel = new Panel();
            currentPanel.setLayout(new BorderLayout());
            Panel keystorePanel = new Panel();
            keystorePanel.setLayout(new BorderLayout());
            Checkbox checkBox = new Checkbox(localizedName, storeHandlerIter == selectedKeyStoreHandler, keystoreCheckboxGroup);
            checkBox.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    selectedKeyStoreHandler = storeHandlerIter;
                }
            });

            keystorePanel.add(checkBox, BorderLayout.WEST);
            Panel keystoreButtonsPanel = new Panel();
            storeHandlerIter.addPreferencesComponents(keystoreButtonsPanel, frame);
            keystoreButtonsPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
            keystorePanel.add(keystoreButtonsPanel, BorderLayout.SOUTH);
            currentPanel.add(keystorePanel, BorderLayout.NORTH);
            currentPanel.add(nestedPanel, BorderLayout.CENTER);
            currentPanel = nestedPanel;
        }

        ScrollPane scrollPane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
        scrollPane.setSize(300, 100);
        scrollPane.add(pnlKeystores);

        dialog.add(scrollPane, BorderLayout.CENTER);

        bCancel = new Button(ResourceManager.getString("CANCEL"));
        bOk = new Button(ResourceManager.getString("OK"));

        Panel pnlButtons = new Panel();
        pnlButtons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        pnlButtons.add(bCancel);
        pnlButtons.add(bOk);
        dialog.add(pnlButtons, BorderLayout.SOUTH);

        bCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelled = true;
                dialog.hide();
            }
        });

        bOk.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialog.hide();
            }
        });

        KeyListener kl = new KeyListener() {
            public void keyTyped(KeyEvent evt) {
            }

            public void keyPressed(KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
                    dialog.hide();
                }
                if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
                    dialog.hide();
                }
            }

            public void keyReleased(KeyEvent evt) {
            }
        };
        bCancel.addKeyListener(kl);

        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dialog.hide();
            }
        });

        dialog.pack();
        initialized = true;
    }

    public boolean wasCancelled() {
        return cancelled;
    }

    public KeyStoreHandler getSelectedKeystoreHandler() {
        return selectedKeyStoreHandler;
    }
}
