/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DlgKeystoreManagement.java,v 1.4 2006/08/12 19:02:37 cara Exp $ */

package org.openoces.opensign.client.applet.dialogs;

/**
 * This class represent a keystore management dialog
 *
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 */

import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.certificate.x509.KeyStoreHandler;
import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.certificate.CopyableItem;
import org.openoces.opensign.certificate.RestoreableItem;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;


public class DlgKeystoreManagement extends AbstractDialog {
    private boolean initialized = false;
    private KeyStoreHandler keyStoreHandler;
    protected AbstractCertificateHandler[] certs = null;
    protected ScrollPane scrollPane;
    protected List certListView = null;
    protected Frame frame;
    protected RestrictedAction ra;

    /* this is a hack. We need the button object around to be able to disable them when
       no certificate is selected. Adding an ItemListener to the certListView does not work
       as no event seemingly is fired on deselection */
    protected final Button bExport = new Button(ResourceManager.getString("EXPORT"));
    protected final Button bImport = new Button(ResourceManager.getString("IMPORT"));
    protected final Button bDelete = new Button(ResourceManager.getString("DELETE_KEYFILE"));
    protected final Button bView = new Button(ResourceManager.getString("WINDOW_MAIN_BUTTON_DETAILS"));

    public DlgKeystoreManagement(RestrictedAction ra, Frame owner, String title, KeyStoreHandler keyStoreHandler) {
        super(ra, owner, title);
        this.frame = owner;
        this.ra = ra;
        this.keyStoreHandler = keyStoreHandler;
        initialize();
    }

    protected AbstractCertificateHandler getSelectedCertificate() {
        return certs[certListView.getSelectedIndex()];
    }

    protected void updateList() {
        certs = keyStoreHandler.getCertificates();
        String selectedItem = null;
        if (certListView.getSelectedIndex() != -1) {
            selectedItem = certListView.getSelectedItem();
            certListView.deselect(certListView.getSelectedIndex());
        }

        /* we explicitly disable the buttons, since no deselect events are fired */
        bExport.setEnabled(false);
        bView.setEnabled(false);
        bDelete.setEnabled(false);

        certListView.removeAll();
        if (certs != null) {
            for (int i = 0; i < certs.length; i++) {
                AbstractCertificateHandler h = certs[i];
                String n = h.getUserfriendlyName();
                certListView.add(n);
                if (certListView.getItem(i).equals(selectedItem)) {
                    certListView.select(i);
                }
            }
        }
    }

    private void initialize() {
        dialog.setLayout(new BorderLayout());
        scrollPane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
        certListView = new List();
        scrollPane.add(certListView);
        certListView.setBackground(Color.white);
        scrollPane.add(certListView);
        updateList();
        dialog.add(scrollPane, BorderLayout.CENTER);

        Panel pnlButtons = new Panel();

        Panel pnlManagementButtons = new Panel();

        Button bOk = new Button(ResourceManager.getString("OK"));

        if (keyStoreHandler.supportsImport()) {

            bImport.setEnabled(true);
            bImport.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        keyStoreHandler.execImportDialog(frame);
                        keyStoreHandler.refreshKeystore();
                        updateList();
                    } catch (IOException e1) {
                        // FIXME: handle this exception
                    }
                }
            });
            pnlManagementButtons.add(bImport);
        }

        if (keyStoreHandler.supportsExport())

        {

            bExport.setEnabled(false);
            certListView.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    if (e.getStateChange() == ItemEvent.DESELECTED) {
                        bExport.setEnabled(false);
                    } else if (e.getStateChange() == ItemEvent.SELECTED) {
                        bExport.setEnabled(true);
                    }
                }
            });
            bExport.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        ((CopyableItem) getSelectedCertificate()).backup(frame);
                    } catch (IOException e1) {
                        // FIXME: handle this exception
                    }
                }
            });

            pnlManagementButtons.add(bExport);
        }

        if (keyStoreHandler.supportsDelete())

        {

            bDelete.setEnabled(false);
            certListView.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    System.out.println("state change event fired");
                    if (e.getStateChange() == ItemEvent.DESELECTED) {
                        System.out.println("deselect event fired");
                        bDelete.setEnabled(false);
                    } else if (e.getStateChange() == ItemEvent.SELECTED) {
                        System.out.println("select event fired");
                        bDelete.setEnabled(true);
                    }
                }
            });
            bDelete.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String message = ResourceManager.getString("CONFIRM_DELETE");
                    DlgConfirm dlg = new DlgConfirm(ra, frame, ResourceManager.getString("CONFIRM_DELETE"), message);
                    dlg.show();

                    if (!dlg.isCancelled()) {
                        FileLog.debug("client accepted deletion of certificate");
                        AbstractCertificateHandler selectedCert = getSelectedCertificate();
                        FileLog.debug("selected cert: " + selectedCert);
                        FileLog.debug("it is CopyableItem : " + (selectedCert instanceof CopyableItem));
                        FileLog.debug("certificate to be deleted is copyable");
                        ((CopyableItem) selectedCert).delete(frame);
                        keyStoreHandler.refreshKeystore();
                        updateList();
                    } else {
                        FileLog.debug("client cancelled deletion of certificate");
                    }
                }
            });
            pnlManagementButtons.add(bDelete);
        }


        bView.setEnabled(false);
        bView.addActionListener(new

                ActionListener() {
                    public void actionPerformed
                            (ActionEvent
                                    e) {
                        DlgCertificateDetails dlg = null;
                        try {
                            dlg = new DlgCertificateDetails(ra,
                                    frame,
                                    ResourceManager.getString("CERTIFICATE_DETAILS"),
                                    getSelectedCertificate());
                            dlg.show();
                        } catch (Exception e1) {
                            FileLog.debug("Unable to display certificate details: " + e1.toString());
                        }
                    }
                }

        );

        certListView.addItemListener(new

                ItemListener() {
                    public void itemStateChanged
                            (ItemEvent
                                    e) {
                        if (e.getStateChange() == ItemEvent.DESELECTED) {
                            bView.setEnabled(false);
                        } else if (e.getStateChange() == ItemEvent.SELECTED) {
                            bView.setEnabled(true);
                        }
                    }
                }

        );

        pnlManagementButtons.add(bView);

        pnlManagementButtons.setLayout(new

                FlowLayout(FlowLayout.LEFT)

        );

        Panel pnlOkButton = new Panel();
        pnlOkButton.setLayout(new

                FlowLayout(FlowLayout.RIGHT)

        );
        pnlOkButton.add(bOk);

        pnlButtons.setLayout(new

                BorderLayout()

        );
        pnlButtons.add(pnlManagementButtons, BorderLayout.NORTH);
        pnlButtons.add(pnlOkButton, BorderLayout.SOUTH);

        dialog.add(pnlButtons, BorderLayout.SOUTH);

        bOk.addActionListener(new

                ActionListener() {
                    public void actionPerformed
                            (ActionEvent
                                    e) {
                        dialog.hide();
                    }
                }

        );

        KeyListener kl = new KeyListener() {
            public void keyTyped(KeyEvent evt) {
            }

            public void keyPressed(KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
                    dialog.hide();
                }
                if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
                    dialog.hide();
                }
            }

            public void keyReleased(KeyEvent evt) {
            }
        };

        dialog.addWindowListener(new

                WindowAdapter() {
                    public void windowClosing
                            (WindowEvent
                                    e) {
                        dialog.hide();
                    }
                }

        );

        dialog.pack();
        initialized = true;
    }
}
