/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: DlgPassword.java,v 1.5 2006/08/12 19:02:37 cara Exp $ */

package org.openoces.opensign.client.applet.dialogs;

/**
 * This class represent a password dialog
 *
 * @author Jens Bo Friis  <jbf@it-practice.dk>
 * @author Carsten Raskgaard  <carsten@raskgaard.dk>
 * @author Preben Rosendal Valeur  <prv@tdc.dk>
 */

import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.client.applet.resources.ResourceManager;
import org.openoces.opensign.client.applet.bootstrap.FileLog;

import java.awt.*;
import java.awt.event.*;

public class DlgPassword extends AbstractDialog {

	Panel pnl;
	Label lblPassword;
	protected TextField tfPassword;
	protected Button bCancel;
	protected Button bOk;
    protected InputValidator validator;
    protected Label msgLabel;
	Panel pnlButtons;
	protected boolean cancelled;

	public boolean isCancelled() {
		return cancelled;
	}

	public String getPassword() {
		return tfPassword.getText();
	}

	public DlgPassword(RestrictedAction ra, Frame owner, String title) {
        this(ra, owner, title, ResourceManager.getString("DLG_PASSWORD_LABEL_PASSWORD"));
    }
	public DlgPassword(RestrictedAction ra, Frame owner, String title, String prompt) {
        this(ra, owner, title, prompt, 16, null);
    }
	public DlgPassword(RestrictedAction ra, Frame owner, String title, String prompt, int columns, InputValidator validator) {
		super(ra, owner, title);
        this.validator = validator;
		dialog.setLayout(new BorderLayout());

		lblPassword = new Label(prompt + ":");
		tfPassword = new TextField();
		tfPassword.setColumns(columns);
		tfPassword.setEchoChar('*');
		tfPassword.requestFocus();

        Panel pnlLbl = new Panel();
        Panel pnlTf = new Panel();
        pnlLbl.add(lblPassword);
        pnlTf.add(tfPassword);
        dialog.add(pnlLbl, BorderLayout.WEST);
		dialog.add(pnlTf, BorderLayout.CENTER);

        pnlButtons = new Panel();

        pnlButtons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        bCancel = new Button(ResourceManager.getString("DLG_PASSWORD_BUTTON_CANCEL"));
		bOk = new Button(ResourceManager.getString("DLG_PASSWORD_BUTTON_OK"));
        pnlButtons.add(bCancel);
        pnlButtons.add(bOk);
        dialog.add(pnlButtons,BorderLayout.SOUTH);

        bCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cancelClicked();
			}
		});
		bOk.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				okClicked();
			}
		});

        if (validator != null){
            tfPassword.addTextListener(new TextListener() {
                public void textValueChanged(TextEvent e) {
                    msgLabel.setText("");// clear msg again
                }
            });

            Panel p = new Panel();
            p.setLayout(new FlowLayout(FlowLayout.CENTER));
            msgLabel = new Label(validator.getInfo());
            p.add(msgLabel);
            dialog.add(p,BorderLayout.NORTH);
        }

		KeyListener kl = new KeyListener() {
			public void keyTyped(KeyEvent evt) {
			}
			public void keyPressed(KeyEvent evt) {
				if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
					cancelClicked();
				}
				if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
					okClicked();
				}
			}
			public void keyReleased(KeyEvent evt) {
			}
		};

		bCancel.addKeyListener(kl);
		bOk.addKeyListener(kl);
		tfPassword.addKeyListener(kl);

		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				cancelClicked();
			}

		});
		dialog.pack();
	}

	protected void okClicked() {
        if (validator != null){
            if (!validator.isValid(tfPassword.getText())){
                msgLabel.setForeground(Color.red);
                msgLabel.setText(validator.getErrorMsg());
                FileLog.debug("Pincode was invalid");
                return;
            }
        }
		cancelled = false;
		dialog.hide();
	}

	private void cancelClicked() {
		cancelled = true;
		dialog.hide();
	}
}
