/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: BasicTest.java,v 1.7 2006/08/12 19:02:40 cara Exp $ */

package org.openoces.opensign.test.xml.xmldsig;

import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.utils.IgnoreAllErrorHandler;
import org.apache.xml.security.utils.XMLUtils;
import org.apache.xpath.XPathAPI;
import org.openoces.opensign.certificate.plugin.pkcs12.Pkcs12CertificateHandler;
import org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport;
import org.openoces.opensign.client.applet.OS;

import org.openoces.opensign.client.applet.bootstrap.security.SunRestrictedAction;
import org.openoces.opensign.client.applet.bootstrap.interfaces.RestrictedAction;
import org.openoces.opensign.utils.Base64;
import org.openoces.opensign.client.applet.bootstrap.FileLog;
import org.openoces.opensign.xml.xmldsig.BasicSignatureGenerator;
import org.openoces.opensign.xml.xmldsig.SignatureGenerator;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.security.*;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Properties;

/**
 * This class implements xmldsig tests
 *
 * @author Carsten Raskgaard   <carsten@raskgaard.dk>
 */

public class BasicTest {
    private Pkcs12CertificateHandler certHandler;

    private void setup() {
        RestrictedAction ra = new SunRestrictedAction();
        OS.setRestrictedAction(ra);
        OS.setCryptoSupport((new SunCryptoSupport()));
        FileLog.setVerbose(FileLog.FATAL);
//        certHandler = new Pkcs12CertificateHandler(null, new File("./data/test/TestTestesen.p12"));
        try {
//            certHandler = new Pkcs12Test(new File("./data/test/TestTestesen.p12"), "Test1234".toCharArray());
        } catch (Exception e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    private Document createObject(String s) {
        Document doc = null;

        org.apache.xml.security.Init.init();

        javax.xml.parsers.DocumentBuilderFactory dbf =
                javax.xml.parsers.DocumentBuilderFactory.newInstance();

        dbf.setNamespaceAware(true);
        dbf.setAttribute("http://xml.org/sax/features/namespaces", Boolean.TRUE);

        try {
            javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
            db.setErrorHandler(new IgnoreAllErrorHandler());
            doc = db.parse(new ByteArrayInputStream(s.getBytes("UTF-8")));
        } catch (ParserConfigurationException e1) {
            e1.printStackTrace();
        } catch (SAXException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }

        return doc;
    }

    private void signatureTest() {
        try {
            String toBeSigned = "OpenOces";
            String ssn = "some social security number";
            String optionalfield2 = "another optional value";
            String propertyName0 = "aproperty";
            String propertyValue0 = "avalue";

            Properties invisibleProperties = new Properties();
            invisibleProperties.setProperty(propertyName0,propertyValue0);


            Properties visibleProperties = new Properties();
            if ( toBeSigned != null ) {
                visibleProperties.put(SignatureGenerator.SIGNTEXT,toBeSigned);
            }
            if ( ssn != null ) {
                visibleProperties.put(SignatureGenerator.SOCIALSECURITYNUMBER,ssn);
            }
            if ( optionalfield2 != null ) {
                visibleProperties.put(SignatureGenerator.OPTIONALFIELD2,optionalfield2);
            }

            SignatureGenerator sigGen = new BasicSignatureGenerator();

            String s = sigGen.sign(certHandler,"Test1234".toCharArray(), toBeSigned, visibleProperties, invisibleProperties);
//            String s = sigGen.sign(certHandler,"Test1234".toCharArray(),ssn,optionalfield2,toBeSigned,p);

            Document doc = this.createObject(s);
            Element nscontext = XMLUtils.createDSctx(doc, "ds", org.apache.xml.security.utils.Constants.SignatureSpecNS);

            Element sigElement = (Element) XPathAPI.selectSingleNode(doc, "//ds:Signature[1]", nscontext);
            XMLSignature signature = new XMLSignature(sigElement, null);
            X509Certificate cert = signature.getKeyInfo().getX509Certificate();
            /* CAUTION:
                In a non-demonstration setup one should always check the validity of
                the certificate before continuing beyond this point
            */
            boolean valid = false;

            System.out.println("Validating signature: " + ((valid = signature.checkSignatureValue(cert)) ? "valid" : "invalid"));
            if ( valid ) {
                System.out.println("Checking signed text: "+((new String(Base64.decode(getValue(doc,"signtext").getBytes())).equals(toBeSigned)) ? "valid" : "invalid"));
                System.out.println("Checking signed social security number value: "+((new String(Base64.decode(getValue(doc,"socialsecuritynumber").getBytes())).equals(ssn)) ? "valid" : "invalid"));
                System.out.println("Checking signed second optional field value: "+((new String(Base64.decode(getValue(doc,"optionalfieldtwo").getBytes())).equals(optionalfield2)) ? "valid" : "invalid"));
                System.out.println("Checking signed property value: "+((new String(Base64.decode(getValue(doc,propertyName0).getBytes())).equals(propertyValue0)) ? "valid" : "invalid"));
            }

        } catch (CertificateException e) {
            e.printStackTrace();
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        } catch (SignatureException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (NoSuchProviderException e) {
            e.printStackTrace();
        } catch (UnrecoverableKeyException e) {
            e.printStackTrace();
        } catch (KeyStoreException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String getValue(Document doc,String name) throws TransformerException {
        Element value = (Element) XPathAPI.selectSingleNode(doc, "//Name[.=\""+name+"\"]/../Value");
        return value != null ? value.getFirstChild().getNodeValue() : null;
    }

    public static void main(String args[]) {
        BasicTest t = new BasicTest();

        t.setup();
        t.signatureTest();
    }
}
