/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: SignatureGenerator.java,v 1.6 2006/07/23 21:49:36 cara Exp $ */

package org.openoces.opensign.xml.xmldsig;

import org.openoces.opensign.certificate.AbstractCertificateHandler;
import org.openoces.opensign.appletsupport.Attachment;

import java.util.Properties;

/**
 * This interface describes the interface for xmldsig generators.
 *
 * @author Michael Motet
 */

public interface SignatureGenerator {
    public static final String SIGNTEXT = "signtext";
    public static final String SOCIALSECURITYNUMBER = "socialsecuritynumber";
    public static String OPTIONALFIELD2 = "optionalfieldtwo";
    public static String HOSTNAME = "host";

    static final String ACTION = "action";
    static final String ACTION_SIGN = "sign";
    static final String ACTION_LOGON = "logon";
    static final int BASE64_LINELENGTH = 76;
    /**
     * Generate a Base64 representation of an XMLDSIG document visible and invisible properties are signed
     * by the certificate and password.
     *
     * VisibleProps is supposed to contain what was visible to the user during signing.
     * InvisibleProps is supposed to contain extra info relevant - like layout of applet.
     *
     * @param certificate
     * @param password
     * @param signText will be added to visibleProps - just to make it more explicit
     * @param visibleProps
     * @param invisibleProps
     * @param attachments
     * @return The signed document
     */
    public StringBuffer sign(AbstractCertificateHandler certificate, char[] password, StringBuffer signText, Properties visibleProps, Properties invisibleProps, Attachment[] attachments) throws Exception;
    /**
     * Generate a Base64 representation of an XMLDSIG document visible and invisible properties are signed
     * by the certificate and password.
     *
     * VisibleProps is supposed to contain what was visible to the user during signing.
     * InvisibleProps is supposed to contain extra info relevant - like layout of applet.
     * @param certificate
     * @param password
     * @param visibleProps - can contain text to be signed in visibleProps under key SIGNTEXT when used for signing.
     * @param invisibleProps - can contain text to be signed in invisibleProps under key SIGNTEXT when used for logon.
     * @return the signed document 
     */
    public StringBuffer logon(AbstractCertificateHandler certificate, char[] password, Properties visibleProps, Properties invisibleProps) throws Exception;
}
