/*
    Copyright 2006 IT Practice A/S
    Copyright 2006 TDC Totalløsninger A/S
    Copyright 2006 Jens Bo Friis
    Copyright 2006 Preben Rosendal Valeur
    Copyright 2006 Carsten Raskgaard


    This file is part of OpenSign.

    OpenSign is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    OpenSign is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with OpenOcesAPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    Note to developers:
    If you add code to this file, please take a minute to add an additional
    copyright statement above and an additional
    @author statement below.
*/

/* $Id: SortVector.java,v 1.3 2006/04/17 21:08:28 cara Exp $ */

package org.openoces.opensign.xml.xmldsig;

import java.util.*;

/**
 * This class implements quicksort
 *
 * @author Michael Motet
 */

interface Compare {
	  boolean lessThan(Object lhs, Object rhs);
	  boolean lessThanOrEqual(Object lhs, Object rhs);
	} ///:~ 

public class SortVector extends Vector {
	  private Compare compare; // To hold the callback
	  public SortVector(Compare comp) {
	    compare = comp;
	  }
	  public void sort() {
	    quickSort(0, size() - 1);
	  }
	  private void quickSort(int left, int right) {
	    if(right > left) {
	      Object o1 = elementAt(right);
	      int i = left - 1;
	      int j = right;
	      while(true) {
	        while(compare.lessThan(
	              elementAt(++i), o1))
	          ;
	        while(j > 0)
	          if(compare.lessThanOrEqual(
	             elementAt(--j), o1))
	            break; // out of while
	        if(i >= j) break;
	        swap(i, j);
	      }
	      swap(i , right);
	      quickSort(left, i-1);
	      quickSort(i+1, right);
	    }
	  }
	  private void swap(int loc1, int loc2) {
	    Object tmp = elementAt(loc1);
	    setElementAt(elementAt(loc2), loc1);
	    setElementAt(tmp, loc2);
	  }
	} ///:~ 
